-- To initialise database structure, execute:
-- >createdb bugs
-- >psql --single-transaction -f createdb.sql bugs

CREATE TABLE module
(
    id              SERIAL,
    name            VARCHAR(16)        NOT NULL,
    description     VARCHAR(256)       NOT NULL,
    path            VARCHAR(256)       NOT NULL,
    created         DATE,

    PRIMARY KEY     (id)
);


CREATE TABLE changeset
(
    id		    SERIAL,
    cs_number	    VARCHAR(32),
    description     VARCHAR(8192)      NOT NULL,
    created         DATE               NOT NULL,
    url             VARCHAR(256),

    PRIMARY KEY     (id)
);

CREATE TABLE module_revision
(
    id              SERIAL,
    rev_number      VARCHAR(16)        NOT NULL,
    description     VARCHAR(1024),
    url             VARCHAR(256),
    module          INT		       NOT NULL,
    changeset	    INT		       NOT NULL,

    FOREIGN KEY     (module) REFERENCES module (id), 
    FOREIGN KEY     (changeset) REFERENCES changeset (id), 
    PRIMARY KEY     (id)
);

CREATE TABLE bug_category
(
    id                  SERIAL,
    name 		VARCHAR (32)   NOT NULL,
    description 	VARCHAR (512)  NOT NULL,

    PRIMARY KEY         (id)
);

CREATE TABLE bug_subcategory
(
    id                  SERIAL,
    name 		VARCHAR (32)   NOT NULL,
    description 	VARCHAR (512)  NOT NULL,
    category		INT	       NOT NULL,

    FOREIGN KEY         (category) REFERENCES bug_category (id), 
    PRIMARY KEY         (id)
);

CREATE TABLE bug_severity
(
    id			INT,
    name		VARCHAR(16)    NOT NULL,
    description		VARCHAR(256)   NOT NULL,

    PRIMARY KEY         (id)
);

CREATE TABLE bug
(
    id                  SERIAL,
    description         VARCHAR (2048) NOT NULL,
    revision_inroduced  INT,                     -- when the bug was introduced
    revision_removed    INT            NOT NULL, -- when the bug was removed
    subcategory		INT	       NOT NULL,
    severity		INT	       NOT NULL,

    FOREIGN KEY         (revision_inroduced) REFERENCES module_revision (id),
    FOREIGN KEY         (revision_removed) REFERENCES module_revision (id),
    FOREIGN KEY         (subcategory) REFERENCES bug_subcategory (id),
    FOREIGN KEY         (severity) REFERENCES bug_severity (id),
    PRIMARY KEY         (id)
);
