-----------------------------------------------------------------
-- Overall statistics for all types of drivers
-----------------------------------------------------------------

-- overall number of bugs
select count(id) from bug;

-- overall number of concurrency errors
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');



-----------------------------------------------------------------
-- Statistics for concurrency errors
-----------------------------------------------------------------

-- blocking in atomic context
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='block_in_atomic_context');

-- race condition or deadlock in the disconnect handler
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='disconnect_race');

-- race condition or deadlock in the data path
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='data_race');

-- race condition or deadlock in the control path
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='control_data_race' or
	    name='control_race');

-- race condition or deadlock in power management functions
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='pm_race');

-- forgetting to initialise the synchronisation primitive before using it
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='spinlock_api');

-- calling an OS service without holding an appropriate lock
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='locking_protocol');

-- imbalanced locks
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance');



-----------------------------------------------------------------
-- Statistics for protocol errors
-----------------------------------------------------------------

-- data protocol violations
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='bus_data_protocol' or 
	    name='os_data_protocol');

-- pm protocol violations
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='bus_pm_protocol' or 
	    name='os_pm_protocol');

-- config protocol violations
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='bus_config_protocol' or 
	    name='os_config_protocol');


-- disconnect protocol violations
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='bus_disconnect_protocol' or 
	    name='os_disconnect_protocol');

-- incorectly maintaining resource ownership
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='reference_counting');

-- invalid argument
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument');

-- incorrectly accessing data structures shared acrosss driver interfaces
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='os_data_struct');

-- invalid return status
select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='error_reporting');


-----------------------------------------------------------------
-- Statistics for generic errors
-----------------------------------------------------------------

select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak');

select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='use_after_free');

select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='byte_manipulation');

select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='typo');

select count(id) from bug where subcategory in 
	(select id from bug_subcategory where 
	    name='return_check');



-----------------------------------------------------------------
-- per-driver statistics
-----------------------------------------------------------------

------------------
-- rtl8150
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');


------------------
-- usb hub
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb hub'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb hub')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb hub')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb hub')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb hub')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb hub')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb hub')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');


------------------
-- usb serial
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb serial'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb serial')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb serial')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb serial')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb serial')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb serial')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb serial')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');


------------------
-- usbnet
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usbnet'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usbnet')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usbnet')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usbnet')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usbnet')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usbnet')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usbnet')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');

------------------
-- catc
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='catc'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='catc')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='catc')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='catc')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='catc')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='catc')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='catc')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');

------------------
-- kaweth
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='kaweth'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');


------------------
-- usb storage
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb storage'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb storage')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb storage')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb storage')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb storage')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb storage')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='usb storage')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');


------------------
-- sbp2
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');


------------------
-- eth1394
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='eth1394'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');


------------------
-- bnx2
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');

------------------
-- mthca
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='mthca'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_config_protocol' or 
	    name='os_disconnect_protocol' or 
	    name='os_pm_protocol' or 
	    name='os_data_protocol' or 
	    name='bus_disconnect_protocol' or 
	    name='bus_config_protocol' or 
	    name='bus_data_protocol' or 
	    name='bus_pm_protocol' or 
	    name='api_usage' or 
	    name='wrong_argument' or 
	    name='return_check' or 
	    name='reference_counting' or
	    name='os_data_struct' or 
	    name='error_reporting');



-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic');



------------------
-- i810 fb
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='i810 fb'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='i810 fb')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='i810 fb')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='i810 fb')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='i810 fb')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='i810 fb')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='i810 fb')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');	


------------------
-- cmipci
------------------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='cmipci'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='cmipci')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='cmipci')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='cmipci')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='cmipci')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='cmipci')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='cmipci')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');	




-----------------------------------------------------------------
-- per-susbsystem statistics
-----------------------------------------------------------------

----------
-- USB
----------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150' or name='usb hub' or name='usb storage' or name='usb serial' or name = 'usbnet' or name = 'catc' or name = 'kaweth'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150' or name='usb hub' or name='usb storage' or name='usb serial' or name = 'usbnet' or name = 'catc' or name = 'kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150' or name='usb hub' or name='usb storage' or name='usb serial' or name = 'usbnet' or name = 'catc' or name = 'kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150' or name='usb hub' or name='usb storage' or name='usb serial' or name = 'usbnet' or name = 'catc' or name = 'kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150' or name='usb hub' or name='usb storage' or name='usb serial' or name = 'usbnet' or name = 'catc' or name = 'kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150' or name='usb hub' or name='usb storage' or name='usb serial' or name = 'usbnet' or name = 'catc' or name = 'kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='rtl8150' or name='usb hub' or name='usb storage' or name='usb serial' or name = 'usbnet' or name = 'catc' or name = 'kaweth')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');



-----------
-- Firewire
-----------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2' or name='eth1394'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2' or name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2' or name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2' or name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2' or name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2' or name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='sbp2' or name='eth1394')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');


-----------
-- PCI
-----------

-- overall number of bugs
select count (id) from  bug where revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2' or name='i810 fb' or name='cmipci' or name='mthca'));


-- overall number of concurrency errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2' or name='i810 fb' or name='cmipci' or name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='lock_balance' or 
	    name='spinlock_api' or 
	    name='disconnect_race' or 
	    name='control_data_race' or 
	    name='control_race' or 
	    name='pm_race' or 
	    name='data_race' or 
	    name='block_in_atomic_context' or 
	    name='locking_protocol');


-- overall number of protocol errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2' or name='i810 fb' or name='cmipci' or name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='os_protocol' or 
	    name='reference_counting' or 
	    name='bus_protocol' or 
	    name='dma_protocol');


-- overall number of data errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2' or name='i810 fb' or name='cmipci' or name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='wrong_argument' or 
	    name='os_data_struct' or 
	    name='error_reporting');

-- overall number of errors in device management code
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2' or name='i810 fb' or name='cmipci' or name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='device_protocol' or 
	    name='device_glitch' or 
	    name='device_race' or 
	    name='dma_consistency' or 
	    name='device_data_structures' or 
	    name='endianness');

-- overall generic errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2' or name='i810 fb' or name='cmipci' or name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 
	    name='resource_leak' or 
	    name='use_after_free' or 
	    name='byte_manipulation' or 
	    name='typo' or 
	    name='return_check');

-- overall other errors
select count(id) from bug where 
	revision_removed in 
	(select id from module_revision where module in 
	    (select id from module where name='bnx2' or name='i810 fb' or name='cmipci' or name='mthca')) and
	subcategory in 
	(select id from bug_subcategory where 	    
	    name='collateral' or 
	    name='logic' or
	    name='api_usage');

