-- To initialise database content
-- >psql --single-transaction -f populatedb.sql bugs

insert into bug_severity (id,name,description) values(0,'unknown','severity has not been established');
insert into bug_severity (id,name,description) values(1,'low','minor cosmetic or consistency issue');
insert into bug_severity (id,name,description) values(2,'medium','minor functionality problems');
insert into bug_severity (id,name,description) values(3,'high','major functionality problems');
insert into bug_severity (id,name,description) values(4,'very high','system crash or data loss');

insert into bug_category (name, description) values('generic', 'generic programming bugs not specific to device drivers');
insert into bug_category (name, description) values('hw interface', 'bugs in the hardware interfacing code of the driver');
insert into bug_category (name, description) values('sw interface', 'bugs in the OS interfacing code of the driver'); 

-- generic
insert into bug_subcategory (name, description, category) select 
	'resource_leak',
	'Memory or other resource leak', id from bug_category where name=
	'generic';

insert into bug_subcategory (name, description, category) select 
	'use_after_free',
	'Attempt to access a deallocated resource', id from bug_category where name=
	'generic';

insert into bug_subcategory (name, description, category) select 
	'use_unallocated',
	'Attempt to access memory that has not been allocated', id from bug_category where name=
	'generic';

insert into bug_subcategory (name, description, category) select 
	'free_unallocated',
	'Attempt to free a resource that has not been allocated', id from bug_category where name=
	'generic';

	
insert into bug_subcategory (name, description, category) select 
	'byte_manipulation',
	'Errors in byte and bit-level operations like masking, swapping etc. that are commonly used in drivers but are not unique to hardware programming', id from bug_category where name=
	'generic';

insert into bug_subcategory (name, description, category) select 
	'logic',
	'An error in the program logic that is not caused by device or os specifics, e.g., using ">" instead of "<"', id from bug_category where name=
	'generic';

insert into bug_subcategory (name, description, category) select 
	'typo',
	'Typo that slipped through compilation', id from bug_category where name=
	'generic';




-- sw interface
insert into bug_subcategory (name, description, category) select 
	'api_usage',
	'Incorrect use of the kernel API that does not fall into one of other subcategories', 
	id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'wrong_argument',
	'Driver provided an invalid argument when invoking an OS service, e.g., 
	NULL when a non-NULL pointer is required or a statically allocated 
	instead of kmalloc''ed pointer', id from bug_category where name=
	'sw interface';
	
insert into bug_subcategory (name, description, category) select 
	'return_check',
	'Driver forgets to check value returned by an API call', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'lock_balance',
	'Unbalanced locks', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'spinlock_api',
	'Incorrect use of spinlock OR other synchronisation API', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'disconnect_race',
	'Race condition between hot unplug event handler and other driver operations', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'control_data_race',
	'Race condition between control and data path, e.g., close() and start_xmit().  This does not include disconnect races covered by a separate subcategory.', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'control_race',
	'Race condition between different control requests.', id from bug_category where name=
	'sw interface';
	
insert into bug_subcategory (name, description, category) select 
	'pm_race',
	'Race condition between a power management event and other driver operations (but not hot unplug, which is covered by a separate subcategory)', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'data_race',
	'Race condition in the data path', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'block_in_atomic_context',
	'Driver performed a potentially blocking operation in a non-blocking context', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'os_data_protocol',
	'Data request protocol between driver and the OS was violated.', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'os_config_protocol',
	'Configuration protocol between driver and the OS was violated.', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'os_pm_protocol',
	'Power management protocol between driver and the OS was violated.', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'os_disconnect_protocol',
	'Hot unplug protocol between driver and the OS was violated.', id from bug_category where name=
	'sw interface';



insert into bug_subcategory (name, description, category) select 
	'reference_counting',
	'Incorrect reference counting for an OS resource.  This is a special case of the os_protocol subcategory.', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'os_data_struct',
	'Driver incorrectly uses interface data structures shared with the kernel (e.g., various device descriptors, data buffers, etc)', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select
	'bus_pm_protocol',
	'Power management protocol between driver and the underlying bus was violated.', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select
	'bus_config_protocol',
	'Configuration protocol between driver and the underlying bus was violated.', id from 
	bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select
	'bus_data_protocol',
	'Data request protocol between driver and the underlying bus was violated.', id from bug_category where name=
	'sw interface';

insert into bug_subcategory (name, description, category) select
	'bus_disconnect_protocol',
	'Hot unplug protocol between driver and the underlying bus was violated.', id from bug_category where name=
	'sw interface';



--insert into bug_subcategory (name, description, category) select 
--	'dma_protocol',
--	'Driver violates the API for managing DMA regions, e.g., deallocating a region still in use by the device.', id from bug_category where name=
--	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'locking_protocol',
	'Locking protocol between driver and the OS was violated, e.g., driver calling a function without holding a necessary lock.', id from bug_category where name=
	'sw interface';
	
insert into bug_subcategory (name, description, category) select 
	'collateral',
	'Bug introduced as a side effect of a change elsewhere in the kernel', id from bug_category where name=
	'sw interface';

--insert into bug_subcategory (name, description, category) select 
--	'circular_lock',
--	'Deadlock caused by circular locking', id from bug_category where name=
--	'sw interface';

insert into bug_subcategory (name, description, category) select 
	'error_reporting',
	'Driver returns wrong error code or does not return an error to notify the OS about a hw or a sw failure.', id from bug_category where name=
	'sw interface';



-- hw interface

insert into bug_subcategory (name, description, category) select 
	'device_ordering',
	'Ordering defect', id from bug_category where name=
	'hw interface';

insert into bug_subcategory (name, description, category) select 
	'device_timing',
	'Timing defect', id from bug_category where name=
	'hw interface';

insert into bug_subcategory (name, description, category) select 
	'device_value',
	'Value defect', id from bug_category where name=
	'hw interface';

insert into bug_subcategory (name, description, category) select 
	'device_race',
	'DMA race', id from bug_category where name=
	'hw interface';


--
--insert into bug_subcategory (name, description, category) select 
--	'device_protocol',
--	'Driver does not respect the protocol of the device.  This includes ordering and format of messages exchanged with the device', id from bug_category where name=
--	'hw interface';
--
--insert into bug_subcategory (name, description, category) select 
--	'device_glitch',
--	'The driver fails to control the device correctly as a result of a hardware design bug or a poorely documented/undocumented feature of the device', id from bug_category where name=
--	'hw interface';
--
--insert into bug_subcategory (name, description, category) select 
--	'device_race',
--	'Several clients inside the driver try to access a hardware resource concurrently without proper synchronisation', id from bug_category where name=
--	'hw interface';
--
--insert into bug_subcategory (name, description, category) select 
--	'dma_consistency',
--	'Memory consistency issue between hw and the driver', id from bug_category where name=
--	'hw interface';
--
--insert into bug_subcategory (name, description, category) select 
--	'device_data_structures',
--	'Error in code that manages device-related data structures, e.g., a circular buffer', id from bug_category where name=
--	'hw interface';
--
--insert into bug_subcategory (name, description, category) select 
--	'endianness',
--	'Endianness errors', id from bug_category where name=
--	'hw interface';
--
		
	
		
-------------------
-- RTL8150
-------------------

insert into module (name,description,path,created) values(
	'rtl8150', 
	'rtl8150 based usb-ethernet driver',
	'drivers/net/usb/rtl8150.c','2002-03-22');


-- changeset '1.369.79.54'

insert into changeset (cs_number,description,created,url) values (
	'1.369.79.54',
	'fix the "small packet" problem and debug messages cleanup',
	'2002-04-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3cab9a02n7uTrpxUHQgDraOy23MPAg');

insert into module_revision (rev_number, module, changeset) select 
	'1.2', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.369.79.54';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Race between rtl8150_disconnect () and rtl8150_close().  RTL8150_UNPLUG flag added to solve the problem.
As far as I can tell the race is still there in case disconnect() sets the flag after close () checks it.', 			
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.2' 			and module_revision.module in (select id from module where name = 
	'rtl8150'		) and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The driver dropped short packets received from the device.  Apparently, short packets are actually ok and 
should be forwarded to the IP stack', 			
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.2' 			and module_revision.module in (select id from module where name = 
	'rtl8150'		)and bug_subcategory.name=
	'device_value'	and bug_severity.name=
	'unknown';


-- changeset '1.373.22.2',

insert into changeset (cs_number,description,created,url) values (
	'1.373.22.2',
	'USB pegasus and rtl8150 fixes and improvements',
	'2002-04-15',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3cbb4a87zM29lRLP_ongeyrxk9XTSA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.6',
	'* better tasklet handling and a few races fixed;
* introducing new flag for Rx urb resubmission;
* GFP_KERNEL to GFP_ATOMIC flag change in Tx path.', module.id, changeset.id from module, changeset where module.name=
	'rtl8150' 		and changeset.cs_number=  
	'1.373.22.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Race condition between read_bulk_callback() and rtl8150_disconnect().  The race happens when disconnect ()
	has already deallocated dev, when read_bulk_callback() attempts to access it to obtain the netdev structure. 
	Fixed by checking the RTL8150_UNPLUG flag in read_bulk_callback()',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.6' 			and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'very high';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Race condition between rtl8150_close() and rtl8150_start_xmit ().  rtl8150_close() disabled the controller 
	 before stopping the tx queue.  As a result, start_xmit could be called and enqueue a packets at the
	 disabled controller.  Fixed by stopping the queue before disabling the controller.  Not sure if this solves
	 the problem completely.  What if start_xmit is already running when close is called?',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.6' 			and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'control_data_race'	and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'usb_submit_urb() called with GFP_KERNEL flag from an interrupt context (urb completion callback).',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.6' 			and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'very high';


-- changeset '1.403.108.8',

insert into changeset (cs_number,description,created,url) values (
	'1.403.108.8',
	'[PATCH] pegasus & rtl8150 I chose a little bit more restrictive license for my drivers.
Rx skb pool introduced in pegasus driver and the pool locking in rtl8150
is refined.',
	'2002-07-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d2540d786qIYHbCQoT9ZjeswPeHMw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.9',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.403.108.8';


insert into bug (description, revision_removed, subcategory, severity) select 
	'rtl8150_open() always allocated a new dev->rx_skb even if the previous copy still existed.  This
could happen if the network interface was opened and closed several times',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.9' 			and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'resource_leak' 	and bug_severity.name=
	'medium';


-- changeset '1.1119.2.6',

insert into changeset (cs_number,description,created,url) values (
	'1.1119.2.6',
	'[PATCH] USB: DMA coherency issue with rtl8150
DMA to a part of a structure is forbidden on the noncoherent architectures.',
	'2003-08-06',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f318665eSStBkIB0CudmXV6SRmyGg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.30',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.1119.2.6';


insert into bug (description, revision_removed, subcategory, severity) select 
	'Buffer for interrupt transfers was allocated as part of the driver struct.  This is illegal, 
because USB transfers can only be performed to and from memory allocate by malloc.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.30' 			and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'wrong_argument' 	and bug_severity.name=
	'unknown';


-- changeset '1.1832.16.20',

insert into changeset (cs_number,description,created,url) values (
	'1.1832.16.20',
	'[PATCH] usb alignment fixes Several places did le16_to_cpup() on misaligned address, which blows on
any little-endian platform that doesn''t like misaligned reads.',
	'2004-08-25',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=412d1c8diSUjmoRq0yjr05qtbNP2cQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.38',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.1832.16.20';


insert into bug (description, revision_removed, subcategory, severity) select 
	'- *reg = le16_to_cpup(data);
+ *reg = data[0] | (data[1] << 8);',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.38' 			and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'byte_manipulation' 	and bug_severity.name=
	'unknown';


-- changeset '1.1832.62.11',

insert into changeset (cs_number,description,created,url) values (
	'1.1832.62.11',
	'[PATCH] USB: small rtl8150 patch
this one make rtl8150 auto-load its register values at reset.  Not doing
so is known to cause improper setup when the device is being reseted
frequently.',
	'2004-09-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41385542pAQIiGSXKGm2IFuOgrdXsw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.38.1.1',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.1832.62.11';


insert into bug (description, revision_removed, subcategory, severity) select 
	'The fix itself is just writing 0x11 instead of 0x10 to one of device registers.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.38.1.1'		and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'device_ordering' 	and bug_severity.name=
	'medium';



-- changeset '1.1867.4.7',

insert into changeset (cs_number,description,created,url) values (
	'1.1867.4.7',
	'[PATCH] USB: rtl8150.c ethernet driver : usb_unlink_urb ->usb_kill_urb
here is a way to avoid one full page of annoying messages at shutdown/module unload.',
	'2004-09-14',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41473e836kM9AFu0oodilBwkNmU4tw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.38.1.2',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.1867.4.7';


insert into bug (description, revision_removed, subcategory, severity) select 
'The driver used unlink_urb() to deallocate outstanding requests, but did not 
wait for urb''s to complete afterwards.  The fix is to use kill_urb to make the 
driver wait for abort to complete',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.38.1.2'		and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'bus_data_protocol'	and bug_severity.name=
	'medium';


-- changeset '1.1867.4.47',

insert into changeset (cs_number,description,created,url) values (
	'1.1867.4.47',
	'USB: fix incorrect usage of usb_kill_urb in rtl8150 driver.',
	'2004-09-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41509ffcdzSlfheGKGJKQF9w-n3iZw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.38.1.3',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.1867.4.47';


insert into bug (description, revision_removed, subcategory, severity) select 
	'A fix to a previous bug replaced usb_unlink_urb() with usb_kill_urb() in several locations, including one in
rtl8150_tx_timeout(), which is apparently called in an interrupt context (or is forbidden to block for another reason)',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.38.1.3'		  and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'very high';


-- changeset '1.3332.15.33',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.15.33',
	'[PATCH] NETDEV: fix receiving multicast frames.
Some USB ethernet drivers did not accept multicast frames appropriately.
IPv6 did not work with those drivers without this patch.',
	'2005-06-27',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=42bf8419gQQQIUUUPNftznVRUVPATw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.44',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.3332.15.33';


insert into bug (description, revision_removed, subcategory, severity) select 
	'This seems to be a mechanical error that prevented turning on multicast mode of the device',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.44'		  	and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'logic' 		and bug_severity.name=
	'medium';



-- changeset '1.3332.522.3',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.522.3',
	'[PATCH] USB: disable tasklet if rtl8150 device is removed while active.
This fixes a oops seen in the wild.',
	'2005-10-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4362b89bDMXXB0i7hQBrTKqdbw_X8w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.47',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.3332.522.3';


insert into bug (description, revision_removed, subcategory, severity) select 
	'rtl8150_disconnect() deallocated all driver structures without disabling the tasklet.  
The tasklet immediately accesses the deallocate rtl8150_t struct.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.47'		  	and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'very high';


-- changeset '1.4685.79.15',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.79.15',
	'USB: Patch for rtl8150 to fix unplug problems 
The RTL8150 driver currently crashes the kernel if the USB lead is unplugged
while the device is active.  The attached patch adds error handling to
tell the network layer that the device has gone away when the device is
unplugged.  With this patch, the device can be plugged and unplugged
to one''s hearts'' content, without crashing anything.',
	'2006-08-02',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44d13843rJNVC2smGGxnn3wa4svDfA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.52',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.4685.79.15';


insert into bug (description, revision_removed, subcategory, severity) select 
	'This one is interesting.  Apparently, after several fixes there is still a race condition
between disconnect and data requests.  The problem was mitigated by adding device status checks
after usb_submit_urb calls.  Most probably, the race is still there, just became less likely to happen.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.52'		  	and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'very high';


-- changeset '1.5342.13.1',

insert into changeset (cs_number,description,created,url) values (
	'1.5342.13.1',
	'[PATCH] fix rtl8150
That code doesn''t do what its author apparently thought it would do...',
	'2007-02-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45c56311c-2sEEBGiHZsIlkGFm5CiQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.58',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.5342.13.1';


insert into bug (description, revision_removed, subcategory, severity) select 
	'-	*(data + 1) = cpu_to_le16p(&reg);
+	data[1] = reg & 0xff;
+	data[2] = (reg >> 8) & 0xff;',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.58'		  	and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'byte_manipulation'	and bug_severity.name=
	'unknown';


-- changeset '1.7946.160.19',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.160.19',
	'rtl8150: use default MTU of 1500
The RTL8150 driver uses an MTU of 1540 by default, which causes a
bunch of problems -- it prevents booting from NFS root, for one.',
	'2008-02-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47a5b2ea-MjNUl8yDaWJTIDNnF037w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.64',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'rtl8150' 		and changeset.cs_number=  
	'1.7946.160.19';


insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.64'		  	and module_revision.module in (select id from module where name = 'rtl8150')and bug_subcategory.name=
	'device_value'	and bug_severity.name=
	'medium';

	

-------------------
-- usb storage
-------------------


insert into module (name,description,path,created) values(
	'usb storage', 
	'generic USB storage driver',
	'drivers/usb/storage/usb.c',
	'1999-01-01');


-- changeset '1.6555.11.135',

insert into changeset (cs_number,description,created,url) values (
	'1.6555.11.135',
	'USB: allow usbstorage to have LUNS greater than 2Tb
Attached is a very small patch (several comment lines) and a one-line
coded change) that allows for USB storage devices that are larger than
2TB. At the company where I work we need such support, and one of my
co-workers, Jane Liu, pointed out that SCSI low-layer drivers need to
specify what size CDBs they accept. After looking through the code it
became obvious that the current USB Storage code accepted the default of
12-byte CDBs, so I changed it to accept 16-byte CDBs. This allows our
device to work.',
	'2007-10-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=470fedb8JeQQS37DzpSvHW1DyqMX1w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.145',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.6555.11.135';


insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.145'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'device_value'	and bug_severity.name=
	'medium';


-- changeset '1.6171.117.34',

insert into changeset (cs_number,description,created,url) values (
	'1.6171.117.34',
	'usb-storage: fix bugs in the disconnect pathway
This patch (as961) fixes a couple of bugs in the disconnect pathway of
usb-storage.

The first problem, which apparently has been around for a while
although nobody noticed it, shows up when an aborted command is still
pending when a disconnect occurs.  The SCSI error-handler will
continue to wait in command_abort() until the us->notify completion is
signalled.  Thus quiesce_and_remove_host() needs to signal it.

The second problem was introduced recently along with autosuspend
support.  Since usb_stor_scan_thread() now calls
usb_autopm_put_interface() before exiting, we can''t simply leave the
scanning thread running after a disconnect; we must wait until the
thread exits.  This is solved by adding a new struct completion to the
private data structure.  Fortuitously, it allows the removal of the
rather clunky mechanism used in the past to insure that all threads
have finished before the module is unloaded.',
	'2007-08-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46e934a4Hz6uCaGHnoP14wl1Xgvkgg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.143',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.6171.117.34';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.143'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.143'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';


-- changeset '1.5353.5.63',

insert into changeset (cs_number,description,created,url) values (
	'1.5353.5.63',
	'usb-storage: use first bulk endpoints, not last

According to the Bulk-Only spec, usb-storage is supposed to use the
_first_ bulk-in and bulk-out endpoints it finds, not the _last_.  And
while we''re at it, we ought to test the direction of the interrupt
endpoint as well.  This patch (as842) makes both changes.',
	'2007-02-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45ca649caBchUxunyyGdRkf6WQjlTg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.135',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.5353.5.63';


insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				 module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.135'		  	 and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'device_value' and bug_severity.name=
	'medium';

	
-- changeset '1.4685.33.16',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.33.16',
	'[PATCH] usb-storage: fix race between reset and disconnect

My recent patch converting usb-storage to use
usb_reset_composite_device() added a bug, a race between reset and
disconnect.  It was necessary to drop the private lock while executing a
reset, and if a disconnect occurs at that time it will cause a crash.

This patch (as722) fixes the problem by explicitly checking for an early
termination after executing each command.',
	'2006-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44b57fc7LhTJyLgsqkf93h1ol-Tfig');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.129',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.4685.33.16';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.129'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'very high';


-- changeset '1.3332.522.78',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.522.78',
	'[PATCH] PATCH: usb-storage: allocate separate sense buffer

This patch is from Alan Stern (as560).  It has been rediffed against a
current tree.

This patch allocates a separate buffer for usb-storage to use when
auto-sensing.  Up to now we have been using the sense buffer embedded in a
scsi_cmnd struct, which is dangerous on hosts that (a) don''t do
cache-coherent DMA or (b) have DMA alignment restrictions.',
	'2005-10-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4362b8e6zaj0mIoxYon0pexCBpSktQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.116',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.3332.522.78';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.116'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'wrong_argument'	and bug_severity.name=
	'unknown';


-- changeset '1.3332.522.77',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.522.77',
	'[PATCH] PATCH: usb-storage: move GetMaxLUN later in time

Some time ago we introduced a delay before device scanning, because many
devices do not like to receive SCSI commands right after enumeration.
Now it turns out there''s a device that doesn''t like to receive
Get-Max-LUN right after enumeration either.  Accordingly this patch
delays the Get-Max-LUN request until the beginning of the scanning
procedure.  This fixes Bugzilla entry #5010.',
	'2005-10-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4362b8e5Lao5bxe09gvfy_FTe1weNA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.115',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.3332.522.77';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.115'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'device_timing'		and bug_severity.name=
	'medium';


-- changeset '1.3332.327.96',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.327.96',
	'[PATCH] USB: storage: Fix messed-up locking

Apparently someone changed the SCSI core so that it no longer holds the
host lock when doing a device or bus reset.  usb-storage was updated at
the time, but the change was done carelessly.  Some of the code depends
on that lock being held.

This patch reintroduces the host lock where needed and tries to clarify
the comments explaining why the lock is necessary.  It also moves the
code that clears the TIMED_OUT and ABORTING bitflags so that it executes
as soon as the timed-out command has completed (and while the host lock
is held).',
	'2005-09-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4325d5caGwJsxmaM1sTZTzG9ZuGqpg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.113',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.3332.327.96';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.113'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'collateral'		and bug_severity.name=
	'unknown';


-- changeset '1.3332.300.27',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.300.27',
	'[PATCH] USB Storage: close a race condition in disconnect near queuecommand

usb-storage has a small loophole, a window between the time queuecommand
accepts a new command and the time the control thread starts to execute
it.  If disconnect is called during that window, the driver won''t cancel
the pending command -- we''ve been relying on the SCSI core to cancel it
for us during host removal.  But it''s better for usb-storage to cancel
it;  this avoids races and reduces reliance on the SCSI core.
Fortunately cancelling these commands is easy to do; the key is to do it
_before_ calling scsi_remove_host.',
	'2005-09-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4320c7d0H-7m1djkooOmKUUC7qB9Gw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.111',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.3332.300.27';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.111'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';


-- changeset '1.1938.388.8',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.388.8',
	'[PATCH] USB Storage: fixes to usb-storage scanning thread

It removes the PF_NOFREEZE flag; if someone suspends their system
immediately after plugging in a USB storage device then the scanning thread
should be frozen along with everything else.  The delay will be restarted
after the system wakes up.',
	'2004-11-19',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=419e2f97OFxpBBp6dfQD3yyZKZlcXA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.94',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.1938.388.8';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.94'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'pm_race'		and bug_severity.name=
	'unknown';


-- changeset 1.1938.2.50,

insert into changeset (cs_number,description,created,url) values (
	'1.1938.2.50',
	'[PATCH] USB storage: delayed device scanning

This patch started life as as366, got some modifications, and lives now as
as366b.  It implements a delay in SCSI-layer device scanning for
usb-storage devices at insertion time.

Many devices work better with this delay.  We believe we can remove several
US_FL_FIX_INQUIRY unusual_devs.h entries with this patch.  (That''s a hint,
Phil!)

The delay is adjustable via a sysfs parameter which is global to the
usb-storage module.  The default is 5 seconds.',
	'2004-09-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=415b0214s404iJtqZ6oEZoSvLwIpVw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.93',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.1938.2.50';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.93'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'device_timing'		and bug_severity.name=
	'medium';


-- changeset 1.1722.123.49,

insert into changeset (cs_number,description,created,url) values (
	'1.1722.123.49',
	'[PATCH] USB Storage: Fix race when removing the SCSI host

This patch fixes a race is disconnecting a usb-storage device that occurs
with the SCSI layer.  It''s primarily reproducable via adding delays into
various disconnect and reset processing paths, but has also been
encountered in the field.

This patch started life as as281b, and was modified by me only to patch
properly against current kernels.

The main features of the patch are:

        Store the host pointer at the start of the control thread
        rather than trying to get it from srb->device; after the host
        is removed the SCSI device structure may no longer exist.

        Keep dev_semaphore locked during the entire time the control
        thread or reset handlers are using the us_data structure.

        Reorder the items in dissociate_dev() and release_resources()
        so that things are released in the opposite order from the way
        they were acquired originally.  Don''t bother to increment and
        decrement the usb_device''s reference count; it''s unnecessary.

        In disconnect(), first set the DISCONNECTING flag so that no
        more I/O will take place and no more requests will be accepted.
        Next, cut short the current command and wait for it to finish.
        Then call scsi_remove_host().  The SCSI core guarantees that
        when scsi_remove_host() returns, the host will not be in error
        recovery and all outstanding commands will have been cancelled.

        Remove some old useless left-over code that was #if''ed out.

        Use a wait_queue for the 6-second delay during device resets
        so that we can be woken up in the middle if a disconnect occurs.

The key point here is that after scsi_remove_host(), everything is idle as
far as the SCSI midlayer is concerned.  But if there was a command in
progress at the time, the midlayer will abort it without telling us or
waiting for it to complete.  Hence we have to wait for the control thread
to be idle before we can try to kill it.  This should happen quickly,
since all I/O attempted by the thread will fail immediately.',
	'2004-06-14',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40cde1d33UiczVnFQecKSxzTn-_tEw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.88',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.1722.123.49';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.88'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';


-- changeset 1.1722.123.18,

insert into changeset (cs_number,description,created,url) values (
	'1.1722.123.18',
	'[PATCH] USB Storage: GetMaxLUN tightening

This patch started life from Alan Stern as as274, and has been heavily
modified.  It narrows the case where a clear_halt() is issued after a
failed GetMaxLUN command to only a STALL case.

Since the only legimate responses to a GetMaxLUN are STALL or data,
anything else is now considered a fatal error and we give up on the device.',
	'2004-06-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40c4c910E5XrajknzM0g4WETzfd4cQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.87',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.1722.123.49';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.87'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset 1.1371.80.15

insert into changeset (cs_number,description,created,url) values (
	'1.1371.80.15',
	'[PATCH] USB storage: Fix scatter-gather buffer access in usb-storage core

This patch adds a routine to protocol.c that correctly transfers data to
or from a scatter-gather buffer.  According to Jens Axboe, we''ve been
using page_address() incorrectly - it''s necessary to use kmap() instead
- and in fact it doesn''t give the desired result when the buffers are
located in high memory.  This could affect anyone using a system with 1 GB
or more of RAM, and one user has already reported such a problem (as you
know).

The three fixup routines in protocol.c and usb.c have been changed to use
the new s-g access routine.  When similar adjustments have been made to
all the subdrivers, we will be able to eliminate the raw_bulk.c source
file entirely.',
	'2003-12-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3fd52931krX6Se5SZ7IlABPaDxwHOQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.79',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.1371.80.15';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.79'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'bus_data_protocol'	and bug_severity.name=
	'unknown';


-- changeset 1.1371.24.2

insert into changeset (cs_number,description,created,url) values (
	'1.1371.24.2',
	'[PATCH] USB: fix a thread-exit problem at module unload

This patch fixes a thread-exit problem when the usb-storage module is
unloaded with a preemptable kernel.  Please refer to the comments in the
code for more detail.',
	'2003-11-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3fabdf47ZaHwlzIxspuW6vNWZEc_nw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.78',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.1371.80.15';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.78'		  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';


-- changeset 1.1123.22.8

insert into changeset (cs_number,description,created,url) values (
	'1.1123.22.8',
	'[PATCH] add usb_reset_configuration()

Unfortunately, usb_set_configuration() is widely mis-used as a
lightweight device reset.  That''s trouble because setting a
configuration must sometimes involve things that don''t relate
at all to a light reset, and can''t be done in contexts like
driver probe() calls.

This patch updates most usb_set_configuration() users to use a call
that provides more appropriate functionality:

  - Adds a new usb_reset_configuration() call, which never needs
    to change very much usbcore state.

  - Uses it to replace most usb_set_configuration() calls, in
    many serial drivers, hisax, dvb, irda, and so on.

  - Modifies usb_reset_device() so it issues the control request
    directly.  It''s both more of a reset (hides a USB reset) and
    less of one (altsettings are unchanged).

  - Makes usbfs return the error code instead of discarding it.

Once this goes in, then usb_set_configuration() can be made to
work properly (including from sysfs).',
	'2003-08-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f3824cdgzFddJr4BbO4_MeqR6oRWw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.74.1.1',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.1123.22.8';

insert into bug (description, revision_removed, subcategory, severity) select 
	'As the patch description explains, it is illegal to call usb_set_configuration() from probe (),
and this is exactly what the driver was doing.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.74.1.1'	  	and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'bus_config_protocol'	and bug_severity.name=
	'unknown';


-- changeset 1.1046.422.4

insert into changeset (cs_number,description,created,url) values (
	'1.1046.422.4',
	'[PATCH] USB storage: avoid sending URBs when disconnecting

This patch avoids sending URBs which are part of the reset system when the
device is being disconnected.  It also makes the control thread avoid
starting anything for a device in that state.',
	'2003-06-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f007a45HYz6ieD95HLBRixerqoHZA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.70',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.1046.422.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'As the patch description explains, it is illegal to call usb_set_configuration() from probe (),
and this is exactly what the driver was doing.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.70'	  		and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';


-- changeset 1.889.53.24

insert into changeset (cs_number,description,created,url) values (
	'1.889.53.24',
	'1.889.53.24  	[PATCH] usb-storage: fix oops

It should fix the OOPS on attach.

This fixes a silly error where I fail to initialize a pointer early enough
for the scanning code.  If this isn''t a perfect example of why
scsi_register() and scsi_add_host() aren''t two separate functions, I don''t
know what is.  :)

Oh, and I added a couple of comments, too.

  -  Fix an OOPS by moving the setting of the hostdata[] pointer to _before_
     the device scan starts.',
	'2003-02-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e3f0102Xe2eEnCscSWdSBX0OsMImA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.51.1.3',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.889.53.24';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Called scsi_add_host and scsi_set_device in a wrong order',
				 module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.51.1.3'  		 and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'os_config_protocol' 	and bug_severity.name=
	'unknown';





-- changeset 1.838.68.1

insert into changeset (cs_number,description,created,url) values (
	'1.838.68.1',
	'[PATCH] USB: usb-storage bugfix
Don''t try to dereference the interrupt endpoint if it doesn''t exist.',
	'2002-12-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3dfffc2cIL7gdqjNlOjU1jZrq37K7Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.50',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.838.68.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				 module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.50'	  		 and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'device_value' 		 and bug_severity.name=
	'unknown';


-- changeset 1.536.18.5

insert into changeset (cs_number,description,created,url) values (
	'1.536.18.5',
	'[PATCH] USB storage: add error checks, remove useless code

This patch removes attempts to clear halts on a control endpoint (think
about it for a minute if you don''t see why this is pointless....) and also
adds return-code checks for all places where halts are cleared.

This _should_ be just redundant code, but recent tests suggest that this
is, in fact, not the case.  People should _heavily_ test this patch.  I''m
going to pause here for a while (in the patch stream) until we''ve got this
sorted out - initial results on my test setup seem to show some problems
still remain.  Where those problems are (HCD or usb-storage) remains to be
seen.',
	'2002-09-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d861587NP0tV9lL1buoBxxcRxOQAw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.39',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.536.18.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Cleaning stalls on control endpoints is forbidden by the Linux protocol for USB devices',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.39'	  		and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'bus_data_protocol'	and bug_severity.name=
	'unknown';


-- changeset 1.497.76.7

insert into changeset (cs_number,description,created,url) values (
	'1.497.76.7',
	'[PATCH] PATCH: usb-storage: fix software eject

This patch fixes the recently broken software eject of media.  At least, it
should.  I''m back to having compile problems again, but the fix should
be pretty self-evident.',
	'2002-09-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d763247A-MikLVkHWq9cNJJhbsUXg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.36',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.497.76.7';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.36'	  		and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset 1.373.137.8

insert into changeset (cs_number,description,created,url) values (
	'1.373.137.8',
	'[PATCH] usb-storage locking fixes

I found several SMP and UP locking errors in usb-storage, attached is a
patch:

Changes:
* srb->result is a bitfield, several << 1 were missing.
* add scsi_lock calls around midlayer calls, release the lock before
  calling usb functions that might sleep.
* replace the queue semaphore with a queue spinlocks, queuecommand is
  called from bh context.',
	'2002-05-13',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3ce02f3f0BTcljSGUk0uSyOU-JNCDg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.19',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb storage' 		and changeset.cs_number=  
	'1.373.137.8';

insert into bug (description, revision_removed, subcategory, severity) select 
	'replace the queue semaphore with a queue spinlocks, queuecommand is
  called from bh context.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.19'	  		and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'add scsi_lock calls around midlayer calls, release the lock before
calling usb functions that might sleep.  TODO: what is the protocol here?  In what cases do we need to hold
the SCSI host lock before calling scsi_done?',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.19'	  		and module_revision.module in (select id from module where name = 'usb storage')and bug_subcategory.name=
	'locking_protocol' 	and bug_severity.name=
	'unknown';



-------------------
-- usb hub
-------------------


insert into module (name,description,path,created) values(
	'usb hub', 
	'USB hub driver',
	'drivers/usb/core/hub.c',
	'1999-01-01');


-- changeset '1.7946.147.15',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.147.15',
	'USB: use GFP_NOIO in reset path

this function will run in the context of the scsi error handler thread.
It must use GFP_NOIO instead of GFP_KERNEL to avoid a possible
deadlock.',
	'2008-02-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47a39e90HallMBUUwSTd7szTBJ2Ryg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.145',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.7946.147.15';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.145'	  		  and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'unknown';


-- changeset '1.7859.16.6',

insert into changeset (cs_number,description,created,url) values (
	'1.7859.16.6',
	'USB: fix locking loop by avoiding flush_scheduled_work

This patch (as1027) replaces a call to flush_scheduled_work() - a
dangerous routine to invoke, especially while holding any sort of lock
- with calls to cancel_work_sync() and cancel_delayed_work_sync().

This fixes Bugzilla #9532.',
	'2007-12-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4766c437nxYORWZ2_S7p-Zg5kzfaVg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.308',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.7859.16.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.308'	  		  and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'control_race' 	  	  and bug_severity.name=
	'unknown';


-- changeset '1.7712.5.5',

insert into changeset (cs_number,description,created,url) values (
	'1.7712.5.5',
	'USB: Fix priority mistakes in drivers/usb/core/hub.c
	Fixes priority mistakes similar to ''!x & y''',
	'2007-11-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=474de48cWSbpAGbTP5OHnpEFiaUH8w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.307',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.7712.5.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	if (!hub->disconnected & list_empty(&hub->event_list)) {
+	if (!hub->disconnected && list_empty(&hub->event_list)) {',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.307'	  		  and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'logic' 	  	  and bug_severity.name=
	'unknown';


-- changeset '1.6555.11.108',

insert into changeset (cs_number,description,created,url) values (
	'1.6555.11.108',
	'USB: fix limited_power setting mistake in hub.c

This patch (jx001) fixes a variable assignment mistake in hub driver.
limited_power should be set to 0 if the hub is self-powered,and 1 if
the hub is bus-powered.

However, the effect of the code was exactly opposite to the spec''s
statement for the Local Power Source field. The spec says, this field
is 1 meaning Local power supply lost while this field is 0 indicating
Local power supply good.(This statement is very confusing.)
So this patch switchs the 0 and 1.',
	'2007-10-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=470fed9do_mKb8D1i198MuJHiemzCA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.303',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.6555.11.108';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.303'	  		  and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_value'  	  and bug_severity.name=
	'medium';


-- changeset '1.6171.117.32',

insert into changeset (cs_number,description,created,url) values (
	'1.6171.117.32',
	'USB: accept 1-byte Device Status replies, fixing some b0rken devices

Some devices have a bug which causes them to send a 1-byte reply to
Get-Device-Status requests instead of 2 bytes as required by the
spec.  This doesn''t play well with autosuspend, since we look for a
valid status reply to make sure the device is still present when it
resumes.  Without both bytes, we assume the device has been
disconnected.

Lack of the second byte shouldn''t matter much, since the spec requires
it always to be equal to 0.  Hence this patch (as959) causes
finish_port_resume() to accept a 1-byte reply as valid.',
	'2007-08-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46ccaa60G7vAc6Imkqyhj9-dlZZ2vg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.295',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.6171.117.32';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.295'	  		  and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_value'  	  and bug_severity.name=
	'medium';


-- changeset '1.6154.2.124',

insert into changeset (cs_number,description,created,url) values (
	'1.6154.2.124',
	'USB: fix warning caused by autosuspend counter going negative

This patch (as937) fixes a minor bug in the autosuspend usage-counting
code.  Each hub''s usage counter keeps track of the number of
unsuspended children.  However the current driver increments the
counter after registering a new child, by which time the child may
already have been suspended and caused the counter to go negative.
The obvious solution is to increment the counter before registering
the child.',
	'2007-07-19',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46a005d4b2q20Zlw9ZKWJV6gAg1t7A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.294',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.6154.2.124';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.294'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'logic'  	 	and bug_severity.name=
	'medium';



-- changeset '1.6048.146.45',

insert into changeset (cs_number,description,created,url) values (
	'1.6048.146.45',
	'USB Core: hub.c: prevent re-enumeration on HNP

Patch is to prevent the OTG host of doing 3 times enumeration of
device when the Host suspends for HNP.  The error code used in
this case is ENOTSUPP.',
	'2007-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4696b994NAN2i7oRyCaSpJ5uOZpvIg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.286',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.6048.146.45';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.286'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'logic'  	 	and bug_severity.name=
	'medium';


-- changeset '1.6048.146.25',

insert into changeset (cs_number,description,created,url) values (
	'1.6048.146.25',
	'USB: Make device reset stop retrying after disconnect

This patch (as898) changes the port reset code in the hub driver.  If
a connect change occurs, it is reported the same way as a disconnect
(which of course is what it really is).

It also changes usb_reset_device(), to prevent the routine from futilely
retrying the reset after a disconnect has occurred.',
	'2007-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4696b980aUjrkT5uOSPCjLTMRoTLUw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.284',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.6048.146.25';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.284'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'disconnect_race' 	and bug_severity.name=
	'unknown';


-- changeset '1.6048.146.16',

insert into changeset (cs_number,description,created,url) values (
	'1.6048.146.16',
	'USB: make hub driver''s release more robust

This revised patch (as893c) improves the method used by the hub driver
to release its private data structure.  The current code is non-robust,
relying on a memory region not getting reused by another driver after
it has been freed.  The patch adds a reference count to the structure,
resolving the question of when to release it.',
	'2007-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4696b977-_hYaD13iMJR7Ce6PKgHXw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.281',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.6048.146.16';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.281'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'use_after_free' 	and bug_severity.name=
	'unknown';


-- changeset '1.6048.146.13',

insert into changeset (cs_number,description,created,url) values (
	'1.6048.146.13',
	'USB: don''t unsuspend for a new connection

This patch (as889) prevents the hub driver from trying to resume a
port when there is a new connection.  For one thing, the resume is not
needed - the upcoming port reset will clear the suspend feature
automatically.  For another, on some systems the resume fails and
causes problems.',
	'2007-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4696b974JmFmKC1xJOMr-bU7iE0GbA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.279',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.6048.146.13';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.279'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering' 	and bug_severity.name=
	'unknown';


-- changeset '1.6022.9.1',

insert into changeset (cs_number,description,created,url) values (
	'1.6022.9.1',
	'USB: replace flush_workqueue with cancel_sync_work

This patch (as912) replaces a couple of calls to flush_workqueue()
with cancel_sync_work() and cancel_rearming_delayed_work().  Using a
more directed approach allows us to avoid some nasty deadlocks.  The
prime example occurs when a first-level device (the parent is a root
hub) is removed while at the same time the root hub gets a remote
wakeup request.  khubd would try to flush the autosuspend workqueue
while holding the root-hub''s lock, and the remote-wakeup workqueue
routine would be waiting to lock the root hub.

The patch also reorganizes the power management portion of
usb_disconnect(), separating it out into its own routine.  The
autosuspend workqueue entry is cancelled immediately instead of
waiting for the device''s release routine.  In addition,
synchronization with the autosuspend thread is carried out even for
root hubs (an oversight in the original code).',
	'2007-05-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=465c8f6bckj2igO5DAtOf73jyJqBVQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.276',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.6022.9.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.276'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'disconnect_race' 	and bug_severity.name=
	'very high';


-- changeset '1.5997',

insert into changeset (cs_number,description,created,url) values (
	'1.5997',
	'USB: fix ratelimit call semantics

This patch (as910) fixes a ratelimit modification so that the
original error-handling path will be followed even when the log-rate
limitation kicks in.',
	'2007-05-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4653e334BXOUFmiYb2kmgk0mmmxNaA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.275',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.5997';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Apparently, a useful goto was dropped accidentally when fixing one of previous bugs',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.275'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'logic' 		and bug_severity.name=
	'unknown';


-- changeset '1.5989',

insert into changeset (cs_number,description,created,url) values (
	'1.5989',
	'USB: remove short initial timeout for device descriptor fetch

This patch (as905) removes a micro-optimization from the hub port
initialization code.  Previously we had been using a short timeout on
the first attempt the read the device descriptor; now we will use the
standard timeout length.

It''s not clear that the short timeout ever provided any benefit.  And
now we know of one case where it actually hurts: The device can''t meet
the short timeout and then it gets terminally confused.

This fixes Bugzilla #8444.',
	'2007-05-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4653e32c1eXiNvN8lZUFf47WwAXIog');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.274',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.5989';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Not sure what exactly is happening here.  My best guess is that the control transfer
gets aborted after the setup packet has been sent successfully, which breaks some devices,
although it should not',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.274'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_timing'		and bug_severity.name=
	'unknown';


-- changeset '1.5394.4.14',

insert into changeset (cs_number,description,created,url) values (
	'1.5394.4.14',
	'USB: fix concurrent buffer access in the hub driver

This patch (as849) fixes a bug in the USB hub driver.  A single
pre-allocated buffer is used for all port status reads, but nothing
guarantees exclusive use of the buffer.  A mutex is added to provide
this guarantee.',
	'2007-02-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45d63f0embzgOIRPRxu26s1humu6sw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.265',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.5394.4.14';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.265'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.5620.3.8',

insert into changeset (cs_number,description,created,url) values (
	'1.5620.3.8',
	'USB: ratelimit debounce error messages

flaky hardware can cause a lot of debounce failed messages. To limit
the performance impact, a ratelimit should be used.',
	'2007-03-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45f22b7ddow_mvjnXPZ3ONR4MV_IjA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.267',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.5620.3.8';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.267'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'medium';


-- changeset '1.5394.4.10',

insert into changeset (cs_number,description,created,url) values (
	'1.5394.4.10',
	'usbcore: small changes to hub driver''s suspend method

This patch (as847) makes some small changes to the hub driver''s
suspend method:

For root hubs, the status URB should be unlinked and other
activity stopped _before_ the bus_suspend method is called.',
	'2007-03-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45d63f0aVNVas0SkeICF_7MuJDSXvQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.264',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.5394.4.10';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The driver violates the ordering of requests required by the USB interface protocol. This seems to 
be caused by an undocumented corner case in the protocol for the virtual root hub interface',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.264'	  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'bus_pm_protocol'	and bug_severity.name=
	'unknown';


-- changeset '1.4900.93.101',

insert into changeset (cs_number,description,created,url) values (
	'1.4900.93.101',
	'USB hub: simplify remote-wakeup handling

This patch (as817) simplifies the remote-wakeup processing in the hub
driver.  Now instead of using a specialized code path, it relies on
the standard USB resume routines.  The hub_port_resume() function does
an initial get_port_status() to see whether the port has already
resumed itself (as it does when a remote-wakeup request is sent).
This will slow down handling of other resume events slightly, but not
enough to matter.

The patch also changes the hub_port_status() routine, making it return
an error if a short reply is received.',
	'2006-12-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4570abfahj7EKQKllX-0gwEaEr4ijw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.255.1.5',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4900.93.101';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.255.1.5'	  	and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'error_reporting'	and bug_severity.name=
	'unknown';


-- changeset '1.4685.201.260',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.201.260',
	'USB: force root hub resume after power loss

This patch(as785) forces the PM core to resume a root hub after a
power loss during system sleep.  If the root hub had been suspended
before the system sleep then normally the PM core would not resume it
afterward.  Without this resume, various sorts of wakeup events (like
port change events) can get lost.',
	'2006-09-27',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=451aca63rsX_oukEXC1GP3UQQYRQKw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.251',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4685.201.260';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The USB interface protocol does not guarantee that the device will be woken up
after system resume if it was suspended before the resume.  The driver assumes that it does.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.251'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'bus_pm_protocol'	and bug_severity.name=
	'unknown';


-- changeset '1.4685.33.20',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.33.20',
	'[PATCH] usbcore: fixes for hub_port_resume

This patch (as731) makes a couple of small fixes to the hub_port_resume
routine:

        Don''t return status >= 0 when an error occurs; (this fix seems to be a nop)

        Clear the port-change-suspend status indicator after
        resuming a device.',
	'2006-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44b57fcbvUgn0EyVuIp69vzpQzScPA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.234',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4685.33.20';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Port status change bits need to be cleared after reading them',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.234'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.4685.33.17',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.33.17',
	'[PATCH] USB hub: don''t return status > 0 from resume

finish_device_resume() in the hub driver isn''t careful always to return
a negative code in all the error pathways.  It also doesn''t return 0 in
all the success pathways.  This patch (as724) fixes the behavior.',
	'2006-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44b57fc8IFqH6C2V_ZX8oVyzFhGCGg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.233',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4685.33.17';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.233'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'error_reporting'	and bug_severity.name=
	'unknown';


-- changeset '1.4112.323.80',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.323.80',
	'[PATCH] usbcore: Fix broken RNDIS config selection

RNDIS devices don''t get configured owing to a typo in
choose_configuration().  This patch from Giridhar Pemmasani fixes the
typo.',
	'2006-06-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4499c291v6CXP_8_Qe_MT2S69NYvow');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.229',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4112.323.80';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.229'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'typo'			and bug_severity.name=
	'medium';


-- changeset '1.4112.323.36',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.323.36',
	'[PATCH] USB: usbcore: always turn on hub port power

Some hubs claim not to support port-power switching, and right now the
hub driver believes them and does not enable power to their ports.
However it turns out that even though they don''t actually switch power,
they do ignore all events on a port until told to turn on the power!
This problem has been reported by several users.

This revised patch (as672b) makes the hub driver always try to turn on
port power to all hubs, regardless of what the hub descriptor says.  It
also adds a comment explaining the need for this.',
	'2006-06-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4499c265v8kMXYR9DEIQd1zj-GeZsg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.227',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4112.323.36';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.227'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'medium';


-- changeset '1.4112.206.6',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.206.6',
	'[PATCH] USB: usbcore: don''t check the device''s power source

The choose_configuration() routine contains code the determine the
device''s power source, so that configurations requiring external power
can be ruled out if the device is running on bus power.  Unfortunately
it turns out that some devices have errors in their config descriptors
and other devices don''t like the GET_DEVICE_STATUS request.

Since that information wasn''t used for anything else, this patch (as673)
removes the code, leaving only a comment.  It fixes bugzilla entry
#6448.',
	'2006-05-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44603a2fzJV0cEjxFcNGAcOhK3wfvw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.226',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4112.206.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.226'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';


-- changeset '1.4087.299.65',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.299.65',
	'[PATCH] USB: usbcore: Don''t assume a USB configuration includes any interfaces

In a couple of places, usbcore assumes that a USB device configuration
will have a nonzero number of interfaces.  Having no interfaces may or
may not be allowed by the USB spec; in any event we shouldn''t die if we
encounter such a thing.  This patch (as662) removes the assumptions.',
	'2006-03-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=441f31d0KIhbQ3ojJ5-7CILnwQjEKA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.223',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4087.299.65';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.223'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';


-- changeset '1.4081.52.42',

insert into changeset (cs_number,description,created,url) values (
	'1.4081.52.42',
	'[PATCH] USB: Don''t assume root-hub resume succeeds

This patch (as614) makes a small change to the part of the hub driver
responsible for remote wakeup of root hubs.  When these wakeups occur
the driver is suspended, and in case the resume fails the driver should
remain suspended - it shouldn''t try to proceed with its normal
processing.

This will hardly ever matter in normal use, but it did crop up while I
was debugging a different problem.',
	'2006-01-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43bc436fudpIQn2x_xNOix_V-5ksGg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.219',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4081.52.42';

insert into bug (description, revision_removed, subcategory, severity) select 
	' 		if (i) {
 			dpm_runtime_resume(&hdev->dev);
 			dpm_runtime_resume(&intf->dev);
+			usb_put_intf(intf);
+			continue;
 		}',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.219'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'reference_counting'	and bug_severity.name=
	'unknown';



-- changeset '1.4081.52.29',

insert into changeset (cs_number,description,created,url) values (
	'1.4081.52.29',
	'[PATCH] USB: Fix locking for USB suspend/resume

The earlier USB locking updates didn''t touch the suspend/resume
routines.  They need updating as well, since now the caller holds the
device semaphore.  This patch (as608) makes the necessary changes.  It
also adds a line to store the correct power state when a device is
resumed, something which was unaccountably missing.',
	'2006-01-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43bc42c8GZuEaSZBjDeCVNUOURgwBg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.216',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.4081.52.29';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.216'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'pm_race'		and bug_severity.name=
	'unknown';


-- changeset '1.3332.327.95',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.327.95',
	'[PATCH] USB: relax usbcore reset timings

This appears to help some folk, please merge.
This patch relaxes reset timings.  There are some reports that it
helps make enumeration work better on some high speed devices.',
	'2005-09-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4325d5c9lOeCgtTPVRFjM7gi8V2TqQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.191',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.3332.327.95';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.191'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_timing'		and bug_severity.name=
	'unknown';


-- changeset '1.3332.300.34',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.300.34',
	'[PATCH] USB: Disconnect children when unbinding the hub driver

This patch (as554) makes the hub driver disconnect any child USB devices
when it is unbound from a hub.  Normally this will never happen, but
there are a few oddball ways to unbind the hub driver while leaving the
children intact.  For example, the new "unbind" sysfs attribute can be
used for this purpose.

Given that unbinding hubs with children is now safe, the patch also
removes the code that prevented people from doing so using usbfs.',
	'2005-09-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4320c914j5uvdUJS7qWW1zZBb6Pg7A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.188',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.3332.300.34';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.188'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'os_config_protocol'	and bug_severity.name=
	'unknown';


-- changeset '1.3332.22.68',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.22.68',
	'[PATCH] USB: usbcore: inverted test for resuming interfaces

This one-liner fixes a test for interfaces that are already resumed.

It would be nice if this could get into 2.6.12, but it''s not critical
since it only affects people doing selective (runtime) suspend/resume.',
	'2005-06-27',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=42c0734fnMhZe1o-agRP0M0paBCvsg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.183',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.3332.22.68';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.183'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';


-- changeset '1.2181.4.21',

insert into changeset (cs_number,description,created,url) values (
	'1.2181.4.21',
	'[PATCH] USB: Prevent hub driver interference during port reset

This patch causes the hub driver to ignore ports that are being reset by
another driver.  Without this protection khubd will sometimes notice that
the port is behaving funny while the reset is taking place and will
unilaterally disconnect it, which is not what we want.',
	'2005-03-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=423a359b1ALcaA7CStSsdD2G4Iy4Kg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.172',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.2181.4.21';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.172'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'pm_race'		and bug_severity.name=
	'unknown';


-- changeset '1.1994.7.17',

insert into changeset (cs_number,description,created,url) values (
	'1.1994.7.17',
	'[PATCH] USB: Retry more aggressively during device initialization

This patch make the hub driver''s device initialization routine more
aggressive about detecting errors and retrying.  It checks the result of
the 64-byte GET-DESCRIPTOR request to verify that the descriptor tag is
set correctly and the ep0-maxpacket value is legal; if either is not true
it will retry the request immediately.  David Brownell has said that this
kind of approach is necessary to make certain buggy devices work.',
	'2005-03-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=422d46fcGR-NzYUB58JbU1dPb7otUw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.162',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1994.7.17';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.162'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.1982.90.3',

insert into changeset (cs_number,description,created,url) values (
	'1.1982.90.3',
	'[PATCH] USB Hub driver: Add reset recovery-time delay

This patch is clearly needed for us to be in compliance with the USB spec.
It adds the mandated recovery-time delay following a port reset.
Regardless of anything else we do to alter the device initialization
sequence, this is necessary.',
	'2005-02-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4213c8addYh_Nj9zqwelW7aBnv-0_w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.161',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1982.90.3';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.161'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_timing'		and bug_severity.name=
	'unknown';


-- changeset '1.1938.455.47',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.455.47',
	'[PATCH] USB: Hub driver: several bug fixes and simplifications [11/13]

This patch adds several relatively minor bug fixes and code
simplifications for the hub driver.

        Perhaps most significantly, a test is added to usbfs to prevent
        users from unbinding the hub driver from a hub with children.
        That could cause some bad headaches, and it''s best to avoid the
        situation.

        Don''t set unused bits in the change_bits and event_bits fields
        (this aids in debugging, nothing more).

        Use the correct spinlock in the hub ioctl routine!

        Add an argument to hub_port_disable, indicating whether the
        device behind the disabled port should be marked NOTATTACHED.
        Normally it should, but for retries during usb_reset_device
        it should not.

        Pass the child device as an argument to hub_port_suspend and
        hub_port_resume rather than making them recalculate it.

        Call usb_set_device_state in the suspend/resume code rather
        than setting udev->state directly.

        Set udev->power.power_state in the suspend/resume code so that
        the correct values are visible in sysfs.

        Recognize during hub_port_resume that the child device udev
        might be NULL (this will happen when a new device is plugged
        into a suspended port).

        Don''t allow hub_port_init to drop the usb_address0_sem lock
        while a device on the bus is still using address 0!

        Make khubd acquire a reference to the hub interface rather than
        the hub device.  This simplifies disconnect/unbind testing.

        Don''t print a debugging message about hub events while not
        holding some lock to protect the hub.

        Remember to drop the reference acquired above if we fail to
        lock the hub.',
	'2004-12-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41c75996taW7a94ZsvqdmlTg4yWuHQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.151',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1938.455.47';

insert into bug (description, revision_removed, subcategory, severity) select 
        'Set udev->power.power_state in the suspend/resume code so that
        the correct values are visible in sysfs.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.151'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'os_data_struct'	and bug_severity.name=
	'unknown';
	
insert into bug (description, revision_removed, subcategory, severity) select 
        'Recognize during hub_port_resume that the child device udev
        might be NULL (this will happen when a new device is plugged
        into a suspended port).',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.151'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'os_data_struct'	and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
        'Don''t allow hub_port_init to drop the usb_address0_sem lock
        while a device on the bus is still using address 0!',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.151'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';


-- changeset '1.1938.455.46',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.455.46',
	'[PATCH] USB: Hub driver: improve error checking and retries [10/13]

Here''s another "cleanup" type patch, most of which was written by David
Brownell.  It causes the hub driver to retry a couple of different types
of transfers if they timeout (which seems to help with some flakey
devices), and it adds some error checking for the ep0 maxpacket value sent
by the device.',
	'2004-12-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41c7597ea9U3H0CCHkjbmC-qh0y_fg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.150',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1938.455.46';

insert into bug (description, revision_removed, subcategory, severity) select 
	'It causes the hub driver to retry a couple of different types
of transfers if they timeout (which seems to help with some flakey
devices)',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.150'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'		and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'and it adds some error checking for the ep0 maxpacket value sent
by the device',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.150'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';



-- changeset '1.1938.313.2',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.313.2',
	'usb: don''t "unsuspend" ports that aren''t suspended

It apparently really confuses some things (Apple keyboard
internal hub for one, but possibly others).',
	'2004-11-05',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=418bcc55gPsIQbVLSqkET15uFmTmuQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.146',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1938.313.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.146'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.1938.292.34',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.292.34',
	'[PATCH] usbcore and system sleep states

This patch makes usbcore behave better with system sleep states, fixing
some PM/PCI integration problems as well as some PM/USB ones.

 - PCI HCDs should behave better, even if they don''t support PCI PM;
   tested with both ACPI and APM suspend cycles, and swsusp.

 - Systems that advertise ACPI S1 support don''t necessarily support the
   PCI D1 or D2 states for their USB controllers.  (Examples include
   Centrino laptops.)  This patch makes them suspend into D3hot, which
   always works (and won''t take much longer to resume, either).

 - Handle the "go into deeper suspend" transition the way PCI drivers
   are supposed to.

 - Understand that USB only has one suspend state:  if the device is
   suspended, it can''t do any better.

 - Export some symbols to HCDs, so they can reuse more of the existing
   usbcore framework.

Plus minor related cleanups.

The PCI D1/D2 state issue is a variant of an earlier problem, with the
same root cause:  the PM core thinking there''s a one-to-one mapping
between system states (roughly:  ACPI S0/S1/S3/S4) and PCI states that
a given device can support (only D0 is required, D3hot is common).
That same issue comes up again with USB.',
	'2004-11-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4186ca7b7AJTJzzu8nXVkq-4eXZQMQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.144',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1938.292.34';

insert into bug (description, revision_removed, subcategory, severity) select 
	'- Systems that advertise ACPI S1 support don''t necessarily support the
   PCI D1 or D2 states for their USB controllers.  (Examples include
   Centrino laptops.)  This patch makes them suspend into D3hot, which
   always works (and won''t take much longer to resume, either).',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.144'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	' - Understand that USB only has one suspend state:  if the device is
suspended, it can''t do any better.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.144'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.1938.1.88',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.1.88',
	'[PATCH] Missed usb devices on boot

This marks sets change_bits to all ones when bringing up a hub, since
not all hubs seem to send change events for devices that were plugged in
when the hub was reset. 

David Miller confirms this fixes his boot-time lost keyboard/mouse
problem',
	'2004-10-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4179ae31VuUn-18_-Ulh3S_i5Zm2rw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.143',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1938.1.88';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.143'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.1832.62.70',

insert into changeset (cs_number,description,created,url) values (
	'1.1832.62.70',
	'[PATCH] USB: Fix off-by-one error in the hub driver

This one-liner fixes a simple mistake in a newly-added part of the hub
driver.  When a connect change takes place on a suspended port the code
clears the suspend, but the port number it passes is origin-0 and it
should be origin-1.',
	'2004-09-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=414224dexfXI0llezH99W2jaHpVviw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.135',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1832.62.70';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.135'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';


-- changeset '1.1832.62.65',

insert into changeset (cs_number,description,created,url) values (
	'1.1832.62.65',
	'[PATCH] USB: khubd looks at ports after probe

Fix the hub probe logic so that when khubd wakes up, it will actually
look at all ports on the new hub.  This resolved a root hub enumeration
issue, and I expect the "boot from USB disk" folk will like this too.

The fix includes cleanup: centralizing the logic to make khubd look
at a given hub, instead of cloning it three times (or in the case of
hub probing, frankensteining it).

This also adds a FIXME to the new routine centralizing disconnect
processing:  we also want "power down port but don''t wake khubd"
(for PM_SUSPEND_DISK as well as SRP).',
	'2004-09-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4140e23cmItsuTW3L0Qz0p1mJLlJvg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.134',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1832.62.65';

insert into bug (description, revision_removed, subcategory, severity) select 
	'+	/* scan all ports ASAP on new hubs */
+	hub->change_bits[0] = ~0;',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.134'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'bus_pm_protocol'	and bug_severity.name=
	'unknown';


-- changeset '1.1803.61.42',

insert into changeset (cs_number,description,created,url) values (
	'1.1803.61.42',
	'[PATCH] USB: fix deadlock in hub_reset

usb_reset_device takes hdev->serialize.  However, hub_reset is
(only) called by hub_events, which already holds the lock.  Thanks
to Tyler Nielsen for the bug report and helpful backtrace.',
	'2004-08-05',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4112adf6WeBrgvvZgvNoLRwwCSeqXQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.129',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1803.61.42';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.129'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';


-- changeset '1.1757.90.38',

insert into changeset (cs_number,description,created,url) values (
	'1.1757.90.38',
	'[PATCH] USB: usb hub, don''t check speed before reset

Signaling (even D- vs D+ pullup) may not be stable
for a while.

At least one OTG root hub won''t reliably (<10%) report device speed until
after reset (and the OTG state transitions have quiesced for a while).

This patch makes that not matter.',
	'2004-07-14',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40f5aed5YjPRJCgiI80e-GykQHoGQQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.124',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1757.90.38';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.124'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.1757.90.36',

insert into changeset (cs_number,description,created,url) values (
	'1.1757.90.36',
	'[PATCH] USB: usb host side updates, mostly for suspend

This adds some of the infrastructure needed to support some more
USB capabilities:

 -  CONFIG_USB_SUSPEND, so Linux can put individual devices
    into the USB "suspend" state.  They can (sometimes) use
    "remote wakeup" to resume the host; or they can each be
    resumed by the host.

      + New usbcore device selective suspend/resume APIs
        * Define them, as stubs for now
        * Call them on the paths sysfs uses (renamed functions)
      + HCD support
        * Define root hub suspend calls; delegate them to HCDs.
        * OHCI and EHCI can suspend/resume root hubs that way.
        * Not called yet, until suspend/resume calls exist

 -  CONFIG_USB_OTG, which depends on the selective suspend APIs
    to allow devices to switch roles (host to peripheral, etc).
    This patch just adds a few key flags in usb_bus, needed by
    usbcore (during enumeration) and by HCD and OTG controllers
    on OTG-capable boards.

 -  Related bugfix:  power budgeting is supposed to place a
    100mA per port (non-OTG) for bus-powered devices.

This patch changes no behavior; later patches will do that,
and they''ll be smaller because of this.',
	'2004-07-14',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40f5aeb4PiLA5zOseQMKrbPGYWNBtw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.123',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1757.90.36';

insert into bug (description, revision_removed, subcategory, severity) select 
	'power budgeting is supposed to place a
100mA per port (non-OTG) for bus-powered devices',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.123'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';


-- changeset '1.1757.90.6',

insert into changeset (cs_number,description,created,url) values (
	'1.1757.90.6',
	'[PATCH] USB: Remove hub''s children upon unbinding

This patch fixes a logical hole in the hub driver.  It''s possible for the
driver to be unbound from a hub without physically unplugging the hub.
For example, writing 0 into the bConfigurationValue attribute file will
have this effect.  When this happens, we need to make sure that all the
child devices of the hub are logically disconnected and their ports
disabled.

That''s what this patch does.  It''s a little bit tricky because we can''t
simply call usb_disconnect() from within the hub driver''s disconnect()
routine.  While that routine is running it holds the usb bus writelock,
but usb_disconnect() would try to acquire it again.  Instead
schedule_work() is used, so after a brief delay the children will be
removed.',
	'2004-07-14',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40f5a8d0u21q0KGoQW6RpqrX2iKrbA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.121',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1757.90.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.121'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'os_config_protocol'	and bug_severity.name=
	'unknown';


-- changeset '1.1722.123.23',

insert into changeset (cs_number,description,created,url) values (
	'1.1722.123.23',
	'[PATCH] USB: Fix resource leakage in the hub driver

The hub driver is very careless about returning resources when an error
occurs while installing a new device.  This patch attempts to put some
order back into the situation.  Details:

        Since usb_new_device() allocates neither the device structure
        nor the device address, it shouldn''t release either one.

        Because usb_new_device() no longer releases the device structure,
        usb_register_root_hub() doesn''t need to take an extra reference
        to it.

        Since the device address selection and TT setup code is used
        only for new devices, not ones being reset, move that code from
        hub_port_init() to hub_port_connect_change().  By the same token,
        hub_port_init() doesn''t have to release the device address or
        the device structure.

        Just to make things look better, move the failure code in
        hub_port_init() to the end of the routine.  And when disabling
        endpoint 0, disable both the IN and OUT parts of the endpoint.

        In hub_port_connect_change(), make all the failure paths
        execute the same code so that resources are always released.
        These resources comprise: the pointer from the parent to the
        new child device, the HCD state for ep0, the device''s address,
        and the device structure itself -- in short, everything that''s
        set up before calling usb_new_device().',
	'2004-06-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40c50065tDjV0_CJa-XlGru_etRsiw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.110',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1722.123.23';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.110'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'resource_leak'		and bug_severity.name=
	'unknown';


-- changeset '1.1722.123.22',

insert into changeset (cs_number,description,created,url) values (
	'1.1722.123.22',
	'[PATCH] USB: Check port reset return code

This patch adds checking for the SET-FEATURE request that actually does a
port reset.  Without the check, the hub driver just assumes that the port
reset command actually was transferred okay.',
	'2004-06-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40c50049MGEEjk58dmoMsves8o1XnQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.109',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1722.123.22';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.109'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'return_check'		and bug_severity.name=
	'unknown';


-- changeset '1.1722.123.16',

insert into changeset (cs_number,description,created,url) values (
	'1.1722.123.16',
	'USB: make usb devices remove their sysfs files when disconnected.',
	'2004-06-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40c4b390nT4xCUwKqgn5B9kIVKc8bA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.108',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1722.123.22';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.108'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'os_disconnect_protocol' and bug_severity.name=
	'unknown';



-- changeset '1.1722.92.22',

insert into changeset (cs_number,description,created,url) values (
	'1.1722.92.22',
	'[PATCH] USB: Genuine changes to hub_port_debounce()

This patch includes the algorithmic changes I would like to see in
hub_port_debounce().  They are:

        Increase the total timeout period from 400 ms to 1500 ms.

        Check the port''s connect-changed status during the polling
        loop.

        Return as soon as the connection has been stable for the
        required time, even if it has been stably _dis_-connected.
        (The current code waits for the full timeout period if there
        isn''t a connection.)

In previous emails I have responded to all the concerns raised by others
about these changes, and I can''t imagine how they could cause any trouble.

        Increasing the total timeout won''t affect people with properly
        functioning hardware.  Their connections will quickly stabilize
        and the routine will return just as before.  People with flaky
        hardware that takes a long time to settle down will now be able
        to use their devices.

        Checking the connect-changed status during the polling loop will
        make the test more conservative.  The code will be able to
        detect transient disconnections that it would have missed
        before, and it won''t return until the connection really _is_
        stable.  Furthermore, this makes the test compliant with the
        USB specification, which requires the stability timer to be
        restarted whenever a connection change occurs.

        Returning early for disconnections is a simple optimization.
        It''s more important now that the total timeout length is 1.5
        seconds rather than 0.4 seconds.

I urge you to apply this patch and for people to try it out.  If there do
turn out to be problems... the patch is very small, well-contained, and
easy to revert.',
	'2004-06-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40c0f46ccES5quhsblVZCUseFa0w3Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.107',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1722.92.22';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Increase the total timeout period from 400 ms to 1500 ms',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.107'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_timing'		and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Checking the connect-changed status during the polling loop will
make the test more conservative.  The code will be able to
detect transient disconnections that it would have missed
before, and it won''t return until the connection really _is_
stable.  Furthermore, this makes the test compliant with the
USB specification, which requires the stability timer to be
restarted whenever a connection change occurs',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.107'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';



-- changeset '1.1587.3.52',

insert into changeset (cs_number,description,created,url) values (
	'1.1587.3.52',
	'[PATCH] USB: khubd turns port power back on after reset

This goes with the OHCI anti-deadlock patch, and is what
ensures that when a root hub loses power during suspend,
khubd can turn port power back on so devices can enumerate.',
	'2004-05-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40a15a6f9AsI28DKkbICst6lrVQNAg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.97',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1587.3.52';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.97'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.1371.258.7',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.258.7',
	'   diff -Nur style patch
[PATCH] USB: ehci - clear TT buffer command patch

I have a patch for a typo in the 2.6.0 kernel, drivers/usb/core/hub.c,

in hub_clear_tt_buffer, the value USB_DIR_IN | USB_RECIP_OTHER is used
for bmRequestType, when section 11.24.2 of the spec says it should
really be USB_RT_PORT, or 00100011b.

After limited testing, this change exhibited better recovery from TT
errors, here''s the patch :',
	'2004-01-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4019a31cys4qHNApt5FXb9LzTNX-EA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.85',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1371.258.7';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.85'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'wrong_argument'	and bug_severity.name=
	'unknown';






-- changeset '1.342.3.2',

insert into changeset (cs_number,description,created,url) values (
	'1.342.3.2',
	'usb hub:
* fixed problem where usb_port_status was on the stack.',
	'2002-02-15',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3c6d6862sjafXRqEnkvAeuBmMhkd2g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.17',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.342.3.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.17'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'wrong_argument'	and bug_severity.name=
	'unknown';


-- changeset '1.369.8.5',

insert into changeset (cs_number,description,created,url) values (
	'1.369.8.5',
	'usb hub:
* fix problem with us not delaying for any ammount of time after a new device
  has been powered up, as the USB spec indicates should happen.',
	'2002-02-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3c743726f63lbAJ10AE6DH-aaEymSw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.18',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.369.8.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.18'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_timing'		and bug_severity.name=
	'unknown';


-- changeset '1.373.8.2',

insert into changeset (cs_number,description,created,url) values (
	'1.373.8.2',
	'[PATCH] Re: removing sleep_on in hub.c

On Mon, Apr 08, 2002, Oliver Neukum <oliver@neukum.org> wrote:
> using sleep_on is a race. The attached patch replaces it with the
> new wait_event macro from sched.h. The patch is against the 2.5
> from Greg.

Greg, I''ve tested this against 2.4.19-pre6 as well and the attached
patch is relative to that.',
	'2002-04-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3cb47939nhlYsPZkpUyqgahq8wNSkw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.24',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.373.8.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'interruptible_sleep_on() was later removed from the kernel because it leads to a race when
the event happens before interruptible_sleep_on() is called.  ',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.24'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'data_race'	and bug_severity.name=
	'unknown';


-- changeset '1.437.64.7',

insert into changeset (cs_number,description,created,url) values (
	'1.437.64.7',
	'[PATCH] USB: driverfs paths

I noticed a minor goof, basically an open issue coming home to
roost.  For the root hub, usb_device->devpath was "/" so it
caused problems using devpath when constructing names for
interfaces.  Driverfs doesn''t reject it, but of course the
resulting directory names can''t be used...

This patch (untested, but compiles) should make devpath in that
case be "0" ... which can''t collide with any path through hub
ports (first port == "1") so it''s possible to use driverfs
paths again.',
	'2002-08-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d497410OrNzgbimSPBNpwfer6Kwjw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.32',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.437.64.7';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.32'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'wrong_argument'	and bug_severity.name=
	'unknown';


-- changeset '1.838.23.1',

insert into changeset (cs_number,description,created,url) values (
	'1.838.23.1',
	'[PATCH] Patch for debounce in 2.5

I was getting annoyed that nobody fixed the obviously broken
debounce loop, so I had to go ahead and fix that.',
	'2002-12-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3df66840Z6CJCdyPT1yHBgkVrsoX1A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.46',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.838.23.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'When a USB device is connected or disconnected, the driver has to wait for the connection
status to stabilise.  Instead the driver just made several port status reads in a row and reported
the last value.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.46'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_timing'		and bug_severity.name=
	'unknown';



-- changeset '1.437.66.5',

insert into changeset (cs_number,description,created,url) values (
	'1.437.66.5',
	'[PATCH] swsusp: Vojtech pointed error in usb/hub.c

We do not want threads exiting because of suspend, so refrigerator
should be just before test for signals pending (so it has chance to
kill them).',
	'2002-08-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d4987f2a3C4vDSlfae44bsd4MvAwQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.31.1.1',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.437.66.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'As far as I can tell, there was a simple bug in the driver logic that broke hub suspend.
The hub driver thread first checked for signals sent to it and then checked whether the thread
has to be frozen because the hub is being suspended (by reading a flag).  However, a suspend request will
also send a signal to the thread, which will be checked before checking the suspend flag, forcing
the thread to exit before it could realise that it ha do freeze.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.31.1.1'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'pm_race'		and bug_severity.name=
	'unknown';


-- changeset '1.838.79.14',

insert into changeset (cs_number,description,created,url) values (
	'1.838.79.14',
	'[PATCH] usbcore: rm hub oops, message cleanups, unlink

These changes are unrelated except I ran into them all at once:

* Fixes an oops from a partial hub_configure() clean; let
   hub_disconnect() do the whole thing, simpler.

* Since I was there, modify that routine''s err() messages
   to use dev_err().  Then eliminate a redundant diagnostic
   in hub_probe(), and merge the "bad descriptor" cases into
   one diagnostic.  Saves a few hundred rodata bytes, and
   the messages now say what hub''s involved.

* Unlink fixes:  if lower level code reports a submit error,
   make sure the urb gets unlinked from the device''s urb_list;
   and report "it''s already being unlinked" as -EBUSY so callers
   can do something smarter than wonder "what did I do wrong".',
	'2002-12-19',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e0252bbr-Gwrnq-79ururQObibKTw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.48',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.838.79.14';

insert into bug (description, revision_removed, subcategory, severity) select 
	'hub_configure() frees a structure that is also freed by disconnect()',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.48'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'use_after_free'	and bug_severity.name=
	'very high';


-- changeset '1.889.220.24',

insert into changeset (cs_number,description,created,url) values (
	'1.889.220.24',
	'USB: fix bug that prevented usbcore from shutting down.',
	'2003-02-26',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e5d1349FcTofyWLRkkt5eKkcb-PLA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.56',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.889.220.24';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	ret = kill_proc(khubd_pid, SIGTERM, 1);
+	ret = kill_proc(khubd_pid, SIGKILL, 1);',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.56'  		and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'wrong_argument'	and bug_severity.name=
	'unknown';


-- changeset '1.971.16.12',

insert into changeset (cs_number,description,created,url) values (
	'1.971.16.12',
	'[PATCH] USB: Another memory allocation in block IO error handling path
* memory allocation in block io error code path with GFP_KERNEL',
	'2003-03-25',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e80c8b57qIwCwEpoNlyZCLfLGZPmQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.61',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.971.16.12';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	portsts = kmalloc(sizeof(*portsts), GFP_KERNEL);
+	portsts = kmalloc(sizeof(*portsts), GFP_NOIO);',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.61'  		   and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';


-- changeset '1.971.87.2',

insert into changeset (cs_number,description,created,url) values (
	'1.971.87.2',
	'[PATCH] USB: locking reset/probe
* mutual exclusion between reset() and probe()',
	'2003-04-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e8e1f7crkFMI_4a1TUjO8CBIZge9Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.62',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.971.87.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.62'  		   and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'control_race'  	   and bug_severity.name=
	'unknown';


-- changeset '1.1046.236.5',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.236.5',
	'[PATCH] USB: Don''t allocate transfer buffers on the stack in hub.c
 Allocate hub status input buffers separately and not on the stack.',
	'2003-06-06',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3ee10f9bapRrYbsgkMVG9K4xfXPzfQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.68',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1046.236.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.68'  		   and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'wrong_argument'  	   and bug_severity.name=
	'unknown';


-- changeset '1.1046.580.35',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.580.35',
	'[PATCH] USB: usb root hubs need longer timeout
Root hubs should use 50 msec timeouts, not 10 msec.  This will
if nothing else eliminate some messages when using EHCI, which
insists on the whole timeout!',
	'2003-07-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f2a0012p3i79BVX4icKc8GYs-yu7g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.74',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1046.580.35';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.74'  		   and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'device_timing'  	   and bug_severity.name=
	'unknown';


-- changeset '1.1371.76.14',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.76.14',
	'[PATCH] USB: fix race with hub devices disconnecting while stuff is still happening to them.',
	'2003-12-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f2a0012p3i79BVX4icKc8GYs-yu7g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.80',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'usb hub' 		and changeset.cs_number=  
	'1.1371.76.14';

insert into bug (description, revision_removed, subcategory, severity) select 
	'If (!hub) return -ENODEV; was added to hub_port_status().  Not sure if this clears the race completely.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.80'  		   and module_revision.module in (select id from module where name = 'usb hub')and bug_subcategory.name=
	'disconnect_race'  	   and bug_severity.name=
	'unknown';



---------------------------------------
-- SBP-2 protocol driver for IEEE-1394
---------------------------------------

insert into module (name,description,path,created) values(
	'sbp2', 
	'SBP-2 protocol driver for IEEE-1394',
	'drivers/ieee1394/sbp2.c',
	'2000-07-25');


-- changeset '1.8101.39.9',

insert into changeset (cs_number,description,created,url) values (
	'1.8101.39.9',
	'ieee1394: sbp2: fix for SYM13FW500 bridge (Datafab disk)

Fix I/O errors due to SYM13FW500''s inability to handle larger request
sizes.  

This fix is necessary because sbp2''s default request size limit has been
lifted since 2.6.25-rc1.',
	'2008-03-14',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47d9bf52MFKHUSz9LbJ1_IRaoIDljA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.193',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.8101.39.9';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.193'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';


-- changeset '1.8057.10.7',

insert into changeset (cs_number,description,created,url) values (
	'1.8057.10.7',
	'ieee1394: sbp2: add INQUIRY delay workaround

Add the same workaround as found in fw-sbp2 for feature parity and
compatibility of the workarounds module parameter.',
	'2008-02-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47b6f5e6gGPMa6qvOcIvvVrtTnAx3Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.191',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.8057.10.7';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.191'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_timing'  	   and bug_severity.name=
	'unknown';


-- changeset '1.7946.130.27',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.130.27',
	'ieee1394: sbp2: fix bogus s/g access change

sg_dma_len(sg) is invalid before the s/g list is DMA-mapped.

This fixes a post 2.6.24 regression which prevents access to SBP-2
devices on several architectures, introduced by "ieee1394: sbp2: s/g
list access cosmetics", commit 825f1df545ab0289185373b0eaf06fb0b3487422.',
	'2008-02-02',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47a46690sPFiODhyB8fB8sS2Pzutwg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.190',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.7946.130.27';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.190'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'bus_data_protocol'  	   and bug_severity.name=
	'unknown';



-- changeset '1.7145.3.4',

insert into changeset (cs_number,description,created,url) values (
	'1.7145.3.4',
	'ieee1394: sbp2: fix unsafe iteration over list of devices

sbp2_host_reset and sbp2_handle_status_write are not serialized against
sbp2_alloc_device and sbp2_remove_device.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4715345cZemIhIRDokFD-2L9c--A6Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.183',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.7145.3.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.183'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'control_race' 	   	   and bug_severity.name=
	'unknown';


	
-- changeset '1.6171.53.1',

insert into changeset (cs_number,description,created,url) values (
	'1.6171.53.1',
	'ieee1394: revert "sbp2: enforce 32bit DMA mapping"

Revert commit 0555659d63c285ceb7ead3115532e1b71b0f27a7 from 2.6.22-rc1.
The dma_set_mask call somehow failed on a PowerMac G5, PPC64:
http://lkml.org/lkml/2007/8/1/344

Should there ever occur a DMA mapping beyond the physical DMA range, a
proper SBP-2 firmware will report transport errors.  So let''s leave it
at that.',
	'2007-08-02',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46b223a85HdDBQVt2AiM_1UWAFepyw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.181',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.6171.53.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.181'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_ordering'    	   and bug_severity.name=
	'unknown';


-- changeset '1.5342.34.10',

insert into changeset (cs_number,description,created,url) values (
	'1.5342.34.10',
	'ieee1394: sbp2: remove bogus "emulated" host flag
There is no emulation going on here.',
	'2007-02-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45cb8134Ilooqf-FDEkgqEza-DjufQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.172',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.5342.34.10';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.172'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'os_data_struct'    	   and bug_severity.name=
	'unknown';



-- changeset '1.5225.24.1',

insert into changeset (cs_number,description,created,url) values (
	'1.5225.24.1',
	'ieee1394: sbp2: fix probing of some DVD-ROM/RWs

Since commit 98e238cd42be6c0852da519303cf0182690f8d9f in Linux 2.6.19,
"ieee1394: sbp2: don''t prefer MODE SENSE 10", some FireWire DVD-ROMs and
DVD-RWs were mistaken as CD-ROM because sr_mod now sent MODE SENSE 6.
The MMC command set includes only MODE SENSE 10.

This fix lets sbp2 switch scsi_device.use_10_for_rw on for MMC LUs.
This should rather be done in the command set driver sr_mod, not in the
sbp2 transport driver, and an according patch will follow for a next
Linux release.',
	'2007-01-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45a53d7az1IfFOBYAEyw1o33guFsXQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.170',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.5225.24.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.170'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'collateral'    	   and bug_severity.name=
	'unknown';


-- changeset '1.5064.36.2',

insert into changeset (cs_number,description,created,url) values (
	'1.5064.36.2',
	'ieee1394: sbp2: fix bogus dma mapping

Need to use a PCI device, not a FireWire host device.  Problem found by
Andreas Schwab, mistake pointed out by Benjamin Herrenschmidt.
http://ozlabs.org/pipermail/linuxppc-dev/2006-December/029595.html',
	'2006-12-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45966923YuxKqocINYN-UE2yA2Afmg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.169',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.5064.36.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.169'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'os_data_struct'    	   and bug_severity.name=
	'unknown';



-- changeset '1.5064.36.1',

insert into changeset (cs_number,description,created,url) values (
	'1.5064.36.1',
	'ieee1394: sbp2: pass REQUEST_SENSE through to the target

Delete some incorrect code, left over from the initial driver submission
in March 2001.

SBP-2 targets should provide sense data via the SBP-2 status block
(autosense).  We have to pass the REQUEST_SENSE command through to
targets which don''t implement autosense, if there are any, and to
accomodate application clients which use this command.',
	'2006-12-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=459669112L97vfzLCd1b4BMk9JDq0Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.168',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.5064.36.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.168'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_ordering'    	   and bug_severity.name=
	'unknown';


-- changeset '1.5009.1.68',

insert into changeset (cs_number,description,created,url) values (
	'1.5009.1.68',
	'ieee1394: sbp2: slightly reorder sbp2scsi_abort

Put the target''s fetch agent into reset state before the underlying ORB
DMA is unmapped and the ->done handler is called.  It is highly unlikely
but the target could access that ORB right before sbp2 sends the reset
request.',
	'2006-12-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=459669112L97vfzLCd1b4BMk9JDq0Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.148',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.5009.1.68';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.148'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'bus_data_protocol'    	   and bug_severity.name=
	'unknown';


-- changeset '1.4685.138.48',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.48',
	'ieee1394: sbp2: enable auto spin-up for all SBP-2 devices

This is a follow-up to patch "ieee1394: sbp2: enable auto spin-up for
Maxtor disks".  When I ''ejected'' an OXUF922 based HDD from a Mac OS X
box, it was spun down by the Mac and did not spin up by itself when
attached to a Linux box right after that.  The first SCSI command that
required the bridge to access the drive ended in
sda:<6>sd 18:0:0:0: Device not ready: <6>: Current: sense key: Not Ready
    Additional sense: Logical unit not ready, initializing cmd. required

Therefore the flag which instructs scsi_mod to send START STOP UNIT with
START=1 ("make medium ready") after such a condition is now enabled
unconditionally for all FireWire storage devices.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d8b7dEUpV7UdbIu4WAdmOQoQW_A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.146',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.48';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.146'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_ordering'    	   and bug_severity.name=
	'unknown';



-- changeset '1.4685.138.42',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.42',
	'ieee1394: sbp2: don''t prefer MODE SENSE 10

In the old days, sbp2 used to coerce all MODE SENSE commands into the
10 bytes version.  When all command set conversions were removed from
sbp2 several months ago, sdev->use_10_for_ms = 1 was added.  Meaning,
higher SCSI layers preferred the 10 bytes version but would try the 6
bytes version if the former failed.

Recently, a problem with the 10 bytes version was discovered.  An Initio
INIC-1530 firmware accepted the 10 bytes version but replied with bogus
data, showing the HDD incorrectly as write-protected.  Since RBC
actually mandates MODE SENSE (6), I checked which version was sent by
Windows XP and Mac OS X 10.3 to an SBP-2 target hosted by Linux --- it
was the 6 bytes version.  (Exception: OS X sent the 10 bytes version to
an MMC target.  RBC and SBC got MODE SENSE (6).)

Therefore, drop the use_10_for_ms flag from sbp2.  Now the upper layers
will try MODE SENSE (6) before MODE SENSE (10) on all SBP-2 devices.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d882cM0vJfKbv25TQwSKWs6h-xQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.145',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.42';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.145'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'    	   and bug_severity.name=
	'unknown';


-- changeset '1.4685.138.38',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.38',
	'ieee1394: sbp2: prevent rare deadlock in shutdown

Scsi_remove_device() may go into uninterruptible sleep if blocked.
Therefore sbp2_remove() unblocks the Scsi_Host before the device is
requested to be removed.  But there could be another 1394 bus reset
after that which would block the host again.  The 1394 subsystem won''t
call sbp2_update() concurrently to sbp2_remove(), which is why there is
no chance for sbp2_remove() to be unblocked by sbp2_update().

The fix is to tell sbp2''s bus reset handler when a device is to be shut
down so that it skips scsi_block_requests() on that host.  As before,
any new commands after a reset without reconnect will be failed quickly
by sbp2scsi_queuecommand().

In the long term, means to go without scsi_block_requests() should be
found.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d8734uzkrcLjS1xcULEZOitgvOQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.144',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.38';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.144'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'disconnect_race'    	   and bug_severity.name=
	'unknown';


-- changeset '1.4685.138.36',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.36',
	'ieee1394: sbp2: better handling of transport errors

If the target signals a transport failure via status block, complete the
request with DID_BUSY to indicate to the SCSI subsystem that the command
may succeed when retried.

Also log diagnostic information if the status block shows a transport
related problem.  It may point to hardware faults.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d872dbMaqFGzRfiozECOphhia4A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.142',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.36';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.142'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'error_reporting'    	   and bug_severity.name=
	'unknown';



-- changeset '1.4685.138.35',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.35',
	'ieee1394: sbp2: recheck node generation in sbp2_update

While sbp2_update() is doing its duties after a bus reset, another reset
could happen.  Don''t accept new requests until the next undisturbed
sbp2_update() or until sbp2_remove().',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d8729nTOWbw6JrzcVul967xujpQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.141',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.35';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.141'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'control_race'    	   and bug_severity.name=
	'unknown';


-- changeset '1.4685.138.34',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.34',
	'ieee1394: sbp2: safer agent reset in error handlers

The scsi_host_template''s eh_abort_handler and eh_device_reset_handler
are allowed to sleep.  Use this to run sbp2_agent_reset in the more
reliable mode which returns _after_ its write transaction was finished.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d8726grz8TaXfJD0Zo0G4EljAZw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.140',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.34';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The exact race condition is not described but the fix looks like it is aimed to prevent one.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.140'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'control_race'    	   and bug_severity.name=
	'unknown';


-- changeset '1.4685.138.33',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.33',
	'ieee1394: sbp2: handle "sbp2util_node_write_no_wait failed"

Fix for http://bugzilla.kernel.org/show_bug.cgi?id=6948

Because sbp2 writes to the target''s fetch agent''s registers from within
atomic context, it cannot sleep to guaranteedly get a free transaction
label.  This may repeatedly lead to "sbp2util_node_write_no_wait failed"
and consequently to SCSI command abortion after timeout.  A likely cause
is that many queue_command softirqs may occur before khpsbpkt (the
ieee1394 driver''s thread which cleans up after finished transactions) is
woken up to recycle tlabels.

Sbp2 now schedules a workqueue job whenever sbp2_link_orb_command fails
in sbp2util_node_write_no_wait.  The job will reliably get a transaction
label because it can sleep.

We use the kernel-wide shared workqueue because it is unlikely that the
job itself actually needs to sleep.  In the improbable case that it has
to sleep, it doesn''t need to sleep long since the standard transaction
timeout is 100ms.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d8716ki9wTolsBmCEiCWbGYF6Jw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.139',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.33';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Attempt to allocate resources from atomic context failed because resources from the 
pool (transaction labels) can be exhausted by the interrupt handler.  The solution is to allocate
resources in a non-atomic context.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.139'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'data_race'    	   	   and bug_severity.name=
	'unknown';


-- changeset '1.4685.138.26',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.26',
	'ieee1394: sbp2: more checks of status block

 - Add checks for the (very unlikely) cases that the target writes too
   little or too much status data or writes unsolicited status. (not marking this as 
   a bug, because it is not clear whether this has actually ever happened or the
   author is just trying to be paranoid)
 - Indicate that these and similar conditions are unlikely().
 - Check the ''resp'' and ''sbp_status'' fields for possible failure status.
 - Slightly optimize access macros for the status block bitfields.
 - Unify a few related log messages.

TODO:  Check if ''src''==1, then withhold the respective ORB from reuse
until status for any subsequent ORB was received.  This is an old bug
whose fix requires more complex command queue handling.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d85f42hs5go4Qf8-v0CxVKiuJog');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.135',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.26';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.135'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value' 	   	   and bug_severity.name=
	'unknown';



-- changeset '1.4685.138.25',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.25',
	'ieee1394: sbp2: safer initialization of status fifo

Sbp2''s copy of the status fifo was cleared when management ORBs or new
command ORBs were prepared.  The latter had potential for a race
condition if the block layer''s soft IRQ and the 1394 LLD''s interrupt
handler ran on different CPUs.  It would also yield wrong status if a
command was completed with non-zero completion status before other
commands that had zero completion status, and no new command was
enqueued in the meantime.

Now, the status buffer is cleared right before it is written.  Thus it
ends up in the following simpler and safer access pattern:
 * sbp2_alloc_device: allocates and implicitly clears once,
 * sbp2_handle_status_write: clears, writes, and reads,
 * sbp2_query_logins, sbp2_login_device, sbp2_reconnect_device: read.
The latter three do not race with sbp2_handle_status_write because of
how the protocol works.

As a tiny optimization, the first two quadlets of the status never need
to be cleared.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d85f2aaOLtNnskWUMk4S7uu0vYg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.134',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.25';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.134'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'data_race' 	   	   and bug_severity.name=
	'unknown';



-- changeset '1.4685.138.22',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.138.22',
	'ieee1394: sbp2: safer last_orb and next_ORB handling

The sbp2 initiator has two ways to tell a target''s fetch agent about new
command ORBs:
 * Write the ORB''s address to the ORB_POINTER register.  This must not
   be done while the fetch agent is active.
 * Put the ORB''s address into the previously submitted ORB''s next_ORB
   field and write to the DOORBELL register.  This may be done while the
   fetch agent is active or suspended.  It must not be done while the
   fetch agent is in reset state.
Sbp2 has a last_orb pointer which indicates in what way a new command
should be announced.  That pointer is concurrently accessed at various
occasions.  Furthermore, initiator and target are accessing the next_ORB
field of ORBs concurrently and asynchronously.

This patch does:
 * Protect all initiator accesses to last_orb by sbp2_command_orb_lock.
 * Add pci_dma_sync_single_for_device before a previously submitted
   ORB''s next_ORB field is overwritten.
 * Insert a memory barrier between when next_ORB_lo and next_ORB_hi are
   overwritten.  Next_ORB_hi must not be updated before next_ORB_lo.
 * Remove the rather unspecific and now superfluous qualifier "volatile"
   from the next_ORB fields.
 * Add comments on how last_orb is connected with what is known about
   the target''s fetch agent''s state.',
	'2006-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=450d839d9NzAeIHhX1zjJzlVpoIvlQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.131',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4685.138.22';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Concurrent access to a sensitive pointer inside the driver',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.131'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'data_race' 	   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Concurrent access to a sensitive pointer by the driver and the device',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.131'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_race'   	   and bug_severity.name=
	'unknown';



-- changeset '1.4112.295.22',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.295.22',
	'sbp2: fix deregistration of status fifo address space

The proper designator of an invalid CSR address is ~(u64)0, not (u64)0.
Use the correct value in initialization and deregistration.
Also, scsi_id->sbp2_lun does not need to be initialized twice.
(scsi_id was kzalloc''d.)',
	'2006-06-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=448de778QSYe-LerzyXYgpQZz6RYIg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.125',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4112.295.22';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The proper designator of an invalid CSR address is ~(u64)0, not (u64)0',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.125'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value' 	   	   and bug_severity.name=
	'unknown';

	
-- changeset '1.4112.295.16',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.295.16',
	'sbp2: fix S800 transfers if phys_dma is off

If sbp2 is forced to move data via ARM handler, the maximum packet size
allowed for S800 transfers exceeds ohci1394''s buffer size on platforms
where PAGE_SIZE is 4096.',
	'2006-06-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=448de736IkgnzoNQ-ypjd4puK6peyw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.124',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4112.295.16';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.124'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'wrong_argument' 	   and bug_severity.name=
	'unknown';


-- changeset '1.4112.270.13',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.270.13',
	'[PATCH] sbp2: fix check of return value of hpsb_allocate_and_register_addrspace()

From: Stefan Richter <stefanr@s5r6.in-berlin.de>

I added a failure check in patch "sbp2: variable status FIFO address (fix
login timeout)" --- alas for a wrong error value.  This is a bug since
Linux 2.6.16.  Leads to NULL pointer dereference if the call failed, and
bogus failure handling if call succeeded.',
	'2006-06-05',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44848615LLkOwI75KHVtsqaRRpye9Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.118',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4112.295.16';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.118'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'return_check' 	   	and bug_severity.name=
	'unknown';


-- changeset '1.4112.230.8',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.230.8',
	'[PATCH] ohci1394, sbp2: fix "scsi_add_device failed" with PL-3507 based devices

Re-enable posted writes for status FIFO.

Besides bringing back a very minor bandwidth tweak from Linux 2.6.15.x
and older, this also fixes an interoperability regression since 2.6.16:

   http://bugzilla.kernel.org/show_bug.cgi?id=6356
   (sbp2: scsi_add_device failed. IEEE1394 HD is not working anymore.)',
	'2006-05-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=446bf8d8t9AhxDkG2Tb9SMVRU-639A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.117',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4112.230.8';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.117'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'	   	and bug_severity.name=
	'unknown';


-- changeset '1.4112.230.6',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.230.6',
	'[PATCH] sbp2: add read_capacity workaround for iPod

Apple decided to copy some USB stupidity over to FireWire.

The sector number returned by iPods from read_capacity is one too many.
This may cause I/O errors, especially if the kernel is configured for EFI
partition support. We use the same workaround as usb-storage but have to
check for different model IDs.

        http://marc.theaimsgroup.com/?t=114233262300001
        https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=187409',
	'2006-05-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=446bf8d6A2xO67BZ6aYirLoi2OiEJQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.115',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4112.230.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.115'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'	   	and bug_severity.name=
	'unknown';



-- changeset '1.4112.64.23',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.64.23',
	'[PATCH] sbp2: fix spinlock recursion

sbp2util_mark_command_completed takes a lock which was already taken by
sbp2scsi_complete_all_commands.  This is a regression in Linux 2.6.15.

 Reported by Kristian Harms at
        https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=187394

[ More complete commentary, as response to questions by Andrew: ]

> This changes the call environment for all implementations of
> ->Current_done().  Are they all safe to call under this lock?

Short answer: Yes, trust me.  ;-) Long answer:

The done() callbacks are passed on to sbp2 from the SCSI stack along
with each SCSI command via the queuecommand hook.  The done() callback
is safe to call in atomic context.  So does
Documentation/scsi/scsi_mid_low_api.txt say, and many if not all SCSI
low-level handlers rely on this fact.  So whatever this callback does,
it is "self-contained" and it won''t conflict with sbp2''s internal ORB
list handling.  In particular, it won''t race with the
sbp2_command_orb_lock.

Moreover, sbp2 already calls the done() handler with
sbp2_command_orb_lock taken in sbp2scsi_complete_all_commands().  I
admit this is ultimately no proof of correctness, especially since this
portion of code introduced the spinlock recursion in the first place and
we didn''t realize it since this code''s submission before 2.6.15 until
now.  (I have learned a lesson from this.)

I stress-tested my patch on x86 uniprocessor with a preemptible SMP
kernel (alas I have no SMP machine yet) and made sure that all code
paths which involve the sbp2_command_orb_lock were gone through multiple
times.',
	'2006-04-02',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44302cd10uYlLn4lOy9PB2MG8QkFQA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.113',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4112.64.23';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.113'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'data_race'	   	and bug_severity.name=
	'unknown';


-- changeset '1.4112.51.6',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.51.6',
	'sbp2: check for ARM failure

Sbp2 did not check for successful registration of the lower address range
when CONFIG_IEEE1394_SBP2_PHYS_DMA was set. If hpsb_register_addrspace
failed, a "login timed-out" would occur which is misleading. Now sbp2 logs
a sensible error message.',
	'2006-03-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4429dbfeoFmhynT0WVfG3lxosTlv0g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.110',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4112.51.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.110'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'return_check'	   	and bug_severity.name=
	'unknown';


-- changeset '1.4112.51.1',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.51.1',
	'sbp2: prevent unloading of 1394 low-level driver

When a new SBP-2 unit is added, sbp2 now takes a reference on the 1394
low-level driver (ohci1394 or pcilynx).  This prevents the 1394 host driver
module from being unloaded, e.g. by an administrative routine cleanup of
unused kernel modules or when another 1394 driver which depends on ohci1394
is unloaded.

The reference is dropped when the SBP-2 unit was disconnected, when sbp2 is
unloaded or detached from the unit, or when addition of the SBP-2 unit failed.',
	'2006-03-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4429dadcs0JFx-sM9pwqkRlW9bQp-Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.109',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4112.51.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.109'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'reference_counting'   	and bug_severity.name=
	'unknown';


-- changeset '1.4087.197.83',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.197.83',
	'sbp2: variable status FIFO address (fix login timeout)

Let the ieee1394 core select a suitable 1394 address range for sbp2''s
status FIFO instead of using a fixed range. Since the core only selects
addresses which are guaranteed to be out of the "physical range" as per
OHCI 1.1, this patch also fixes an old bug:

OHCI controllers which implement a writeable PhysicalUpperBound register
included sbp2''s status FIFO in the physical range. That way sbp2 was
never notified of a succesful login and always failed after timeout.
Affected OHCI host adapters include ALi and Fujitsu controllers.

As another side effect of this patch, the status FIFO is no longer
located in a range for which OHCI chips perform "posted writes". Each
status write now requires a response subaction. But since large data
transfers involve only few status writes, there is no measurable
decrease of I/O throughput. What''s more, the status FIFO is now safe
from potential host bus errors. Nevertheless, posted writes could be
re-enabled by extensions to the ARM features of the 1394 stack.',
	'2006-02-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43fd4814XPFbLYSgv9ffmAjsifxa_w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.107',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4087.197.83';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.107'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';


-- changeset '1.4087.197.83',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.197.82',
	'sbp2: fix another deadlock after disconnection

If there were commands enqueued but not completed before an SBP-2 unit
was unplugged (or an attempt to reconnect failed), knodemgrd or any
process which tried to remove the device would sleep uninterruptibly
in blk_execute_rq().  Therefore make sure that all commands are
completed when sbp2 retreats.',
	'2006-02-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43fd4734x1CzkllMa3dkU7Ap41R9TA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.106',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4087.197.82';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.106'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';

	
-- changeset '1.4087.210.2',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.210.2',
	'[PATCH] don''t mangle INQUIRY if cmddt or evpd bits are set

sbp2.c mangles INQUIRY response in a way that only applies to standard
inquiry data (i.e. when both cmddt and evpd bits are 0).  Leave other cases
alone; e.g. when asking for VPD the length of reply is in byte 3, not 4
and byte 4 is the first byte of device serial number.',
	'2006-02-18',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43f78975XQxBa8tuy1LFWuC_c2J-mQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.105',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4087.210.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.105'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';


-- changeset '1.4022.9.6',

insert into changeset (cs_number,description,created,url) values (
	'1.4022.9.6',
	'sbp2: did not clean up after scsi_add_device() failed

If scsi_add_device() at the end of sbp2_start_device() fails, e.g. due to
transport errors during SCSI inquiry, sbp2 needs to log out of the device
and release all associated resources.',
	'2005-12-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=439e4812I__YM7h_lBoBhFbucf9qww');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.102',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.4022.9.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.102'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'bus_config_protocol'	and bug_severity.name=
	'unknown';


-- changeset '1.3891.26.1',

insert into changeset (cs_number,description,created,url) values (
	'1.3891.26.1',
	'sbp2_command_orb_lock must be held when accessing the _orb_inuse list.
Fixes an oops in sbp2util_find_command_for_SCpnt after sbp2scsi_abort:
https://bugzilla.novell.com/show_bug.cgi?id=113734',
	'2005-11-18',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4381c678uerGZ8WT4EeUmqA1kJjiuw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.92.1.1',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.3891.26.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.92.1.1'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';


-- changeset '1.3482.16.1',

insert into changeset (cs_number,description,created,url) values (
	'1.3482.16.1',
	'sbp2_command_orb_lock must be held when accessing the _orb_inuse list.
Fixes an oops in sbp2util_find_command_for_SCpnt after sbp2scsi_abort:
https://bugzilla.novell.com/show_bug.cgi?id=113734',
	'2005-11-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=437a03d0WT5HHffzm9oIT7HRk_yUkQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.93',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.3482.16.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.93'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';


-- changeset '1.3332.372.86',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.372.86',
	'[PATCH] sbp2: fix deadlocks and delays on device removal/rmmod

Fixes for deadlocks of the ieee1394 and scsi subsystems and long delays in
futile error recovery attempts when SBP-2 devices are removed or drivers are
unloaded.

 - Complete commands quickly with DID_NO_CONNECT if the 1394 node is gone or if
   the 1394 low-level driver was unloaded.
 - Skip unnecessary work in the eh_abort_handler and eh_device_reset_handler if
   the node or 1394 low-level driver is gone.
 - Let scsi''s high-level shut down gracefully when sbp2 is being unloaded or
   detached from the 1394 unit. A call to scsi_remove_device is added for this
   purpose, which requires us to store a scsi_device pointer.
 - scsi_device pointer is obtained from slave_alloc hook and cleared by
   slave_destroy. This avoids usage of the pointer after the scsi device was
   deleted e.g. by the user via scsi_mod''s sysfs interface.',
	'2005-09-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=433d94e9y_moYAceD9PBl7NHoxmDFw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.91',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.3332.372.86';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.91'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';


-- changeset '1.3192.8.1',

insert into changeset (cs_number,description,created,url) values (
	'1.3192.8.1',
	'[PATCH] sbp2 slab corruption fix

This fixed a problem that showed up in the Fedora development tree a few
weeks before the Fedora Core 4 release, initially as slab corruption, later
as hard crashes on boot up, when slab debugging was disabled for the
release.  More details on the history at
https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=158424

The problem is caused by sbp2''s use of scsi_host->hostdata[0] to hold a
scsi_id, without explicitly requesting space for it.  Since hostdata is
declared as a zero-sized array, we don''t get any such space by default, so
it must be explicitly requested.  The patch below implements just that.',
	'2005-06-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=42b1a2b3hk5P7zZ9D70WGkrgu9L0wg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.81.1.1',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.3192.8.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.81.1.1'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'use_unallocated'	and bug_severity.name=
	'unknown';


-- changeset '1.1982.7.24',

insert into changeset (cs_number,description,created,url) values (
	'1.1982.7.24',
	'Dan Dennedy:

bugfix for logical unit directory lun number',
	'2005-01-31',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41fdcc8eyeiFmEOcnRhQPjgu0JQYFA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.79',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1982.7.24';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.79'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';


-- changeset '1.1982.7.6',

insert into changeset (cs_number,description,created,url) values (
	'1.1982.7.6',
	'fix nodemgr parsing subdirectories in unit directory and tiny bugfix in
sbp2 unit directory parsing (for PowerFile)',
	'2005-01-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41fbf1fa4HOI9uZApsoqbcAzALSiSQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.75',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1982.7.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.75'  		and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';


-- changeset '1.1046.203.2',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.203.2',
	'[PATCH] Update IEEE1394 (r946)

 ETH1394: Use 64bit EUI as the hardware address.
 ETH1394: Support broadcast packets.
 SBP2   : Fix max_payload for > S400
 CORE   : Fix iso.c compilation by including linux/sched.h directly.',
	'2003-06-02',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3edbe0e1mt7sTEZalzJ5FKEMnuUb1Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.34',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1046.203.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Maximum DMA payload was computed without taking into account device capabilities',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.34'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';



-- changeset '1.1046.562.7',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.562.7',
	'[PATCH] Update IEEE1394 (r1014)

 misc:      * Consolidate and make consistent, the NODE_BUS_{FMT,ARGS}
              macros.
            * Add HPSB_VERBOSE() macro to make VERBOSEDEBUG code a
              little cleaner.
 ieee1394 : * Replace panic() with BUG and return value in dma routine.
            * Reset reset_retries on success or loop recover.
            * Add RECV_FLUSH to flush buffers on > 1 irq interval or in
              buffer-fill mode.
            * Probe logic-unit-subdirectories as children of the
              unit-dir.
            * Fix compile warning in nodemgr thread.
            * Replace tightly scheduled timeout loop with a 50ms timer.
            * Better return and usage of BUG_ON() in dma handler.
            * Calculate timeout expiry interval at change instead of at
              every timeout check.
 SBP2     : * Fix logical-unit-directory LUN''s.
            * Remove procfs interface.
            * Fix usage of initdata after init.
 Host con : * Change irq output to use __irq_itoa() for sparc.
            * Do more to notify bus when module is unloaded, to clear
              IRM, etc.
 ohci1394 : Fix endianess of async stream packet headers.',
	'2003-07-25',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f21a3e5p_YiWmW2AO6aKAo7KoV_Jg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.39',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1046.562.7';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Probe logic-unit-subdirectories as children of the unit-dir.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.39'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';



-- changeset '1.1153.111.1',

insert into changeset (cs_number,description,created,url) values (
	'1.1153.111.1',
	'[PATCH] Update IEEE1394 (r1047)

subsys   : - Added hpsb_make_streampacket() helper
           - Fix re-probe of bus after reset, when node''s generation
             changes
           - Missing spinlock.h include for csr.c

general  : - Fix a few warnings from size mismatches in printk''s

raw1394  : - Added RAW1394_REQ_ASYNC_STREAM
           - Use vmalloc/vfree for larger buffer support

ohci1394 : - Fix a few endianess bugs in ISO routines

eth1394  : - Fix cleanup of failed packets',
	'2003-09-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f540b8bVEjkGJCiuTR4IGyghpkU4A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.43',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1153.111.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'In sbp2_start_device: we may be so close to a bus reset, that the device is not available for logins.
This can happen when the bus reset is caused by the host connected to the sbp2 device being removed. That 
host would have a certain amount of time to relogin before the sbp2 device allows someone else to login 
instead. One second makes sense.  Solution: add a 1s timeout',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.43'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'device_timing'   	   and bug_severity.name=
	'unknown';


-- changeset '1.1371.354.14',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.354.14',
	'IEEE1394(r1137): Check to make sure we have a hostinfo in sbp2_host_reset.',
	'2004-02-18',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4033f447z5Re4u4ETdhDGI9Rv_EDkQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.54',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1371.354.14';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.54'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'return_check' 	   	   and bug_severity.name=
	'unknown';



-- changeset '1.1371.436.1',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.436.1',
	'IEEE1394(r1162): Check return value for errors from hpsb_register_protocol.',
	'2004-02-25',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=403cb34bTTdO9MOs9v5T78P6TgqRew');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.61',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1371.436.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.61'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'return_check' 	   	   and bug_severity.name=
	'unknown';



-- changeset '1.1371.460.6',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.460.6',
	'IEEE1394/SBP2(r1170): Unblock scsi requests specifically in the update callback.',
	'2004-02-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4040aca5mH2uVY8d60mMYIMvxL4vJQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.63',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1371.460.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Forgetting to enable scsi request queue after reset (sbp2_host_reset disabled the queue)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.63'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'os_config_protocol' 	   and bug_severity.name=
	'unknown';



-- changeset '1.1722.40.3',

insert into changeset (cs_number,description,created,url) values (
	'1.1722.40.3',
	'[sbp2]: Fix use-after-free bug.',
	'2004-05-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40b78984VQWhdLNQo94wpsZTgIBaAA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.69',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'sbp2' 			and changeset.cs_number=  
	'1.1722.40.3';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.69'  		   and module_revision.module in (select id from module where name = 'sbp2')and bug_subcategory.name=
	'use_after_free'   	   and bug_severity.name=
	'unknown';



---------------------------------------------
-- IPv4 driver for Linux IEEE-1394 Subsystem
---------------------------------------------


insert into module (name,description,path,created) values(
	'eth1394', 
	'IPv4 driver for Linux IEEE-1394 Subsystem',
	'drivers/ieee1394/eth1394.c',
	'2000-01-01');



-- changeset '1.7145.3.2',

insert into changeset (cs_number,description,created,url) values (
	'1.7145.3.2',
	'ieee1394: eth1394: fix lock imbalance
bad_proto can be reached from points which did not take priv->lock.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4715345aeJir7NbBUvucLFXYY-73VA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.103',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.7145.3.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.103'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'lock_balance' 	   	   and bug_severity.name=
	'unknown';



-- changeset '1.6048.27.33',

insert into changeset (cs_number,description,created,url) values (
	'1.6048.27.33',
	'ieee1394: fix to ether1394_tx in ether1394.c

This patch fixes a problem that occurs when packets cannot be sent across
the ieee1394 bus and we return NETDEV_TX_BUSY in the net driver "hard start
xmit" routine ether1394_tx. When we return NETDEV_TX_BUSY the stack will
call ether1394_tx again with the same skb. So we need to restore the header
to look like it did before we munged it for xmit over ieee1394.',
	'2007-06-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4673bec9H_q-DgpQORqUOmqYW4F9sA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.96',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394'		and changeset.cs_number=  
	'1.6048.27.33';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.96'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';



-- changeset '1.6022.2.3',

insert into changeset (cs_number,description,created,url) values (
	'1.6022.2.3',
	'ieee1394: eth1394: handle tlabel exhaustion

When eth1394 was unable to acquire a transaction label, it just dropped
outgoing packets without attempt to resend them later.

The transmit queue is now halted if no tlabel is available to
->hard_start_xmit().  A workqueue job is then scheduled to catch the
moment when ieee1394 recycled the next lot of tlabels.',
	'2007-05-27',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4673bec9H_q-DgpQORqUOmqYW4F9sA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.94',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394'		and changeset.cs_number=  
	'1.6022.2.3';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.94'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'os_data_protocol' 	   and bug_severity.name=
	'unknown';



-- changeset '1.6022.2.2',

insert into changeset (cs_number,description,created,url) values (
	'1.6022.2.2',
	'ieee1394: eth1394: remove bogus netif_wake_queue
When we are within hard_start_xmit, the queue is already awake.',
	'2007-05-27',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4659f63bkgw43iNk6BmCV7ruWmRE9Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.93',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.6022.2.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.93'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'os_data_protocol' 	   and bug_severity.name=
	'unknown';


-- changeset '1.5791.80.51',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.80.51',
	'ieee1394: eth1394: send async streams at S100 on 1394b buses

eth1394 did not work on buses consisting of S100B...S400B hardware
because it attempted to send GASP packets at S800.',
	'2007-04-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=463515admovShTzTI0L4-Wy6w9sPmA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.92',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.5791.80.51';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.92'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'device_value' 	   	   and bug_severity.name=
	'unknown';


-- changeset '1.5791.80.50',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.80.50',
	'ieee1394: eth1394: fix error path in module_init

This patch fixes some error handlings in eth1394:

* check return value of kmem_cache_create()
* cleanup resources if hpsb_register_protocol() fails',
	'2007-04-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=463515acB4Oyt0J8gV2uV8AFQZnQLw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.91',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.5791.80.50';

insert into bug (description, revision_removed, subcategory, severity) select 
	'check return value of kmem_cache_create()',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.91'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'return_check' 	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'cleanup resources if hpsb_register_protocol() fails',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.91'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'resource_leak' 	   and bug_severity.name=
	'unknown';


-- changeset '1.5791.80.48',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.80.48',
	'ieee1394: eth1394: hard_start_xmit is called in atomic context',
	'2007-04-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=463515aaryaJIcbcfiWGMv3KIkDeOw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.89',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.5791.80.48';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.89'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';



-- changeset '1.5791.80.42',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.80.42',
	'ieee1394: eth1394: allow MTU bigger than 1500

RFC 2734 says: "IP-capable nodes may operate with an MTU size larger
than the default [1500 octets], but the means by which a larger MTU is
configured are beyond the scope of this document."

Allow users to set an MTU bigger than 1500.',
	'2007-04-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=463515a4TEtjrfsCSBhGiAWYgySDGQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.83',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.5791.80.42';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.83'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';



-- changeset '1.5791.80.30',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.80.30',
	'ieee1394: eth1394: unregister address space in failure case

Warn if hpsb_allocate_and_register_addrspace() failed.
Unregister the address space if something else failed.',
	'2007-04-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46351598U1bAAmEKzeOhSK30HIoFPQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.72',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.5791.80.30';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.72'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'bus_config_protocol'  	   and bug_severity.name=
	'unknown';


-- changeset '1.5700.1.1',

insert into changeset (cs_number,description,created,url) values (
	'1.5700.1.1',
	'ieee1394: fix oops on "modprobe -r ohci1394" after network class_device conversion

The networking subsystem has been converted from class_device to device
but ieee1394 hasn''t.  This results in a 100% reproducible NULL pointer
dereference if the ohci1394 driver module is unloaded while the eth1394
module is still loaded.
http://lkml.org/lkml/2006/11/16/147
http://lkml.org/lkml/2007/3/14/4

This is a regression in 2.6.21-rc1.',
	'2007-03-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4603a40dbSDpC50vLeeTck-Hs4j7cw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.69',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.5700.1.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.69'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'collateral'  	   	   and bug_severity.name=
	'unknown';


-- changeset '1.4112.295.21',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.295.21',
	'[PATCH] eth1394: endian fixes',
	'2006-06-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=448de76fOlJ_HOb2pJvLXCIADmRmKw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.59',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.4112.295.21';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.59'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'device_value'  	   	   and bug_severity.name=
	'unknown';


-- changeset '1.1982.7.22',

insert into changeset (cs_number,description,created,url) values (
	'1.1982.7.22',
	'Fixed removal of old partial datagrams.  Since max_partial_datagrams can
be changed at runtime, there may be _more_ than max_partial_datagrams
in the list.  Also removed obsolote comment.',
	'2005-01-31',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41fdcb45rQP1mxs1qxjKtbPVnEV9UQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.49',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.1982.7.22';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.49'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'logic'  	   	   and bug_severity.name=
	'unknown';



-- changeset '1.1722.194.82',

insert into changeset (cs_number,description,created,url) values (
	'1.1722.194.82',
	'[PATCH] memory allocation checks in eth1394_update()
Adds memory allocation checks in eth1394_update().',
	'2004-06-18',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40d3050eKYc444rwstH1gLSQnu9Pyg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.41',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.1722.194.82';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.41'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'return_check'  	   	   and bug_severity.name=
	'unknown';


-- changeset '1.1371.1.231',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.1.231',
	'eth1394(r1156): Fix oddities when MacOSX is IRM. Also MacOSX sets an invalid sspd in the ARP reply.',
	'2004-02-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=403a836cufnQIvU5vtdsolAZ61RO9A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.32',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.1371.1.231';

insert into bug (description, revision_removed, subcategory, severity) select 
	'MacOSX seems to be sending us 131 in this field (sspd)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.32'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'device_value'    	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'-max_payload = priv->bc_maxpayload;
+max_payload = priv->bc_maxpayload - ETHER1394_GASP_OVERHEAD;',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.32'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'device_value'    	   and bug_severity.name=
	'unknown';


-- changeset '1.1046.607.2',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.607.2',
	'[PATCH] Update IEEE1394 (r1023)

 misc      : - Signed cleanups.
             - Convert to correct min/max macros.

 subsystem : - Attempt to fix sleeping call in hpsb_get_tlabel when
               preempt is enabled.

 eth1394   : - Fix packet sizes for S800+.',
	'2003-08-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f2eb124ssaKSG_QoOcphHqVzFXFsg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.21',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.1046.607.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.21'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'device_value'    	   and bug_severity.name=
	'unknown';


-- changeset '1.1046.429.1',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.429.1',
	'[PATCH] Update IEEE1394 (r986)

 ETH1394  : - Minor cleanups.
            - Fix incorrect error handling with no broadcast channel.
            - Set max/default MTU to 1500.
            - Fix broken spinlock handling on device open.
 IEEE1394 : Make bus-nodeid printk''s of a consistent format.
 OHCI1394 : Change one printk to a verbose/debug only message.',
	'2003-06-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3eff3528uEDXiDJVs9OEKTniB4z6hQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.19',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.1046.429.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Added in_interrupt check before calling hpsb_iso_shutdown() in eth1394_iso_shutdown()',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.19'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Forgetting to close connection in error path',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.19'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'bus_config_protocol'  and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'Forgetting to close connection in error path because of (apparently wrong) assumption that 
the corresponding call may block',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.19'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'bus_config_protocol'  and bug_severity.name=
	'unknown';



-- Is it a device_value error or something else?
---- changeset '1.1046.294.1',
--
--insert into changeset (cs_number,description,created,url) values (
--	'1.1046.294.1',
--	'[PATCH] Update IEEE1394 (r952)
--
-- OHCI1394: Merge CONFIG_PPC_ALL changes.
-- DV1394  : Fix broken endian conversions.
-- ETH1394 : Fix oopses due to non-linear sk_buff''s.',
--	'2003-06-13',
--	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3eeaabeb2ZhN2ledvub6ZsbBZVoWig');
--
--insert into module_revision (rev_number, description, module, changeset) select 
--	'1.16',
--	'', 			module.id, changeset.id from module, changeset where module.name=	      
--	'eth1394' 		and changeset.cs_number=  
--	'1.1046.294.1';
--
--insert into bug (description, revision_removed, subcategory, severity) select 
--	'Did not call skb_linearize() for non-linear buffers',
--				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
--	'1.16'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
--	'api_usage'  		   and bug_severity.name=
--	'very high';
--
--

-- changeset '1.1042.17.43',

insert into changeset (cs_number,description,created,url) values (
	'1.1042.17.43',
	'[PATCH] 1394 updates

- New irq handler prototypes.
- Lots of un-enumurated locking fixes/cleanups (thanks in large part to
  spinlock debug compile options in the kernel).
- Other various trivial fixes.',
	'2003-04-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3ea4383ecRG9-hCrQVhKLlmxe3vbmw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.10',
	'', 			module.id, changeset.id from module, changeset where module.name=	      
	'eth1394' 		and changeset.cs_number=  
	'1.1042.17.43';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Forgot to call spin_lock_init(&priv->lock);',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.10'  		   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'spinlock_api'  	   and bug_severity.name=
	'unknown';



-- changeset '1.889.175.27',

insert into changeset (cs_number,description,created,url) values (
	'1.889.175.27',
	'[PATCH] IEEE-1394 Updates

- Converts Video1394 to PCI dma.
- Converts ioctl''s to standard interface.
- Various minor fixes
- Merges from 2.5.x tree',
	'2003-02-15',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e4ea2160SiSajWXYXePbrkii9-n9A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.5',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'eth1394' 		and changeset.cs_number=  
	'1.889.175.27';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Forgot to call kfree (hi) in an error path',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.5'  		   	   and module_revision.module in (select id from module where name = 'eth1394')and bug_subcategory.name=
	'resource_leak'  	   and bug_severity.name=
	'unknown';


---------------------------------------------
-- Broadcom NX2 network driver
---------------------------------------------

insert into module (name,description,path,created) values(
	'bnx2', 
	'Broadcom NX2 network driver',
	'drivers/net/bnx2.c',
	'2005-05-26');


-- changeset '1.3332.146.283',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.146.283',
	'[BNX2]: Fix rtnl deadlock in bnx2_close

This fixes an rtnl deadlock problem when flush_scheduled_work() is
called from bnx2_close(). In rare cases, linkwatch_event() may be on
the workqueue from a previous close of a different device and it will
try to get the rtnl lock which is already held by dev_close().

The fix is to set a flag if we are in the reset task which is run
from the workqueue. bnx2_close() will loop until the flag is cleared.
As suggested by Jeff Garzik, the loop is changed to call msleep(1)
instead of yield() in the original patch.

flush_scheduled_work() is also moved to bnx2_remove_one() before the
netdev is freed.',
	'2005-08-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=431395e5hfw0J_dk-y_A3H8Yfn9hEg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.3',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.3332.146.283';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.3'  		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'control_race'  	   and bug_severity.name=
	'unknown';



-- changeset '1.3332.146.287',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.146.287',
	'[BNX2]: update version and minor fixes

Update version and add 4 minor fixes, the last 2 were suggested by
Jeff Garzik:

1. check for a valid ethernet address before setting it
2. zero out bp->regview if init_one encounters an error and unmaps
   the IO address. This prevents remove_one from unmapping again.
3. use netif_rx_schedule() instead of hand coding the same.
4. use IRQ_HANDLED and IRQ_NONE.',
	'2005-08-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=431395f8FAROhNL1L0sowwGqsSc-mw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.7',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.3332.146.287';

insert into bug (description, revision_removed, subcategory, severity) select 
	'check for a valid ethernet address before setting it',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.7'  		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'return_check'  	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'zero out bp->regview if init_one encounters an error and unmaps
the IO address. This prevents remove_one from unmapping again',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.7'  		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'logic'			   and bug_severity.name=
	'unknown';


-- changeset '1.3332.244.38',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.244.38',
	'[BNX2]: Fix bug in irq handler and add prefetch

Fix bug in bnx2_interrupt() that caused an unnecessary register read.
The BNX2_PCICFG_MISC_STATUS should only be read when the status tag
has not changed.

Add prefetch of the status block in bnx2_msi() similar to tg3_msi().
The status block is not touched in bnx2_msi() and prefetching it will
speed up bnx2_poll() that will run on the same CPU that received the
MSI.',
	'2005-09-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43209be46mFbfUkGG6pDuQs5ZQcgEQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.9',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.3332.244.38';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.9'  		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';


-- changeset '1.3839.10.9',

insert into changeset (cs_number,description,created,url) values (
	'1.3839.10.9',
	'[BNX2]: check return of dev_alloc_skb in bnx2_test_loopback

Check return of dev_alloc_skb in bnx2_test_loopback, and handle
appropriately.',
	'2005-11-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=437a056cls64r2PyYze2toGogpZUOw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.18',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.3839.10.9';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.18' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'return_check'  	   and bug_severity.name=
	'unknown';


-- changeset '1.4087.113.1',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.113.1',
	'[BNX2]: Fix VLAN on ASF

Always set up the device to strip incoming VLAN tags when ASF is
enabled. ASF firmware will not parse packets correctly if VLAN tags
are not stripped.',
	'2006-01-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43d56f6eRSmcMg-AnpxkvZ_pc5cefA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.20',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4087.113.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.20' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';



-- changeset '1.4087.113.2',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.113.2',
	'[BNX2]: Improve handshake with firmware

Improve handshake with bootcode with the following changes:

1. Increase timeout to 100msec and use msleep instead of udelay.
2. Add more error checking for timeouts and errors.',
	'2006-01-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43d56faeGaaPF0GSW4R4TsBzhJgnwg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.21',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4087.113.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Increase timeout to 100msec and use msleep instead of udelay',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.21' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_timing'  	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Add more error checking for timeouts and errors',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.21' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';


-- changeset '1.4087.113.3',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.113.3',
	'[BNX2]: Misc. fixes

Some misc. fixes for WoL, 5708 B1, and a typo ''='' instead of ''==''.',
	'2006-01-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43d56feeoR4vBXRqmogASQ5q5q1zrA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.22',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4087.113.3';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Some misc. fixes for WoL (did not understand the exact nature of the fixes)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.22' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'typo ''='' instead of ''==''.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.22' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'typo'  	   	   and bug_severity.name=
	'unknown';



-- changeset '1.4087.113.4',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.113.4',
	'[BNX2]: Fix UDP checksum verification

Fix TCP/UDP checksum verification. Use status bits in the buffer
descriptor instead of the checksum value to verify rx checksum.
Using the checksum value will be incorrect if the UDP packet has
zero in the UDP checksum field.

Firmware update required for this fix.',
	'2006-01-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43d5704fa6loUpOLDMgR28JR1AknAA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.23',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4087.113.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.23' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';



-- changeset '1.4087.113.5',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.113.5',
	'[BNX2]: Workaround hw interrupt bug

Add workaround for a hardware interrupt issue. When using INTA,
unmasking of the interrupt and the tag update should be done
separately to avoid some spurious interrupts,',
	'2006-01-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43d57097NHXOStAR82xnOkZdLyxZyg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.24',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4087.113.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.24' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering'  	   and bug_severity.name=
	'unknown';


-- changeset '1.4087.113.6',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.113.6',
	'[BNX2]: Fix nvram sizing

Add code to correctly determine nvram size.',
	'2006-01-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43d570bezKIcDBElT5r5hspTwuNKIw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.25',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4087.113.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.25' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';


-- changeset '1.4087.300.56',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.300.56',
	'[BNX2]: Fix bug when rx ring is full

Fix the rx code path that does not handle the full rx ring correctly.

When the rx ring is set to the max. size (i.e. 255), the consumer and
producer indices will be the same when completing an rx packet. Fix
the rx code to handle this condition properly.',
	'2006-03-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=441f5b8eSZQDEkSvYRjFBzAsRNcn7A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.32',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4087.300.56';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.32' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';




-- changeset '1.4087.300.59',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.300.59',
	'[BNX2]: Update version

Update version to 1.4.37.

Add missing flush_scheduled_work() in bnx2_suspend as noted by Jeff
Garzik.',
	'2006-03-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=441f5bd0jwgfTrS_Pt-s9aPqY_IbXA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.35',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4087.300.59';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.35' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'pm_race'   		   and bug_severity.name=
	'unknown';



-- changeset '1.4112.17.3',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.17.3',
	'[BNX2]: Fix link change handling

Fix some link-related problems by doing a coalesce_now after link
change interrupt to flush out the transient link status.

To facilitate this, the host coalesce cmd register value is cached in
the device structure.',
	'2006-03-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4422665cZNl7ABQWOW9koOez5A6qlQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.37',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4112.17.3';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.37' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


-- changeset '1.4112.241.2',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.241.2',
	'[BNX2]: Fix bug in bnx2_nvram_write()

Fix a bug in bnx2_nvram_write() caused by a counter variable not
correctly incremented by 4.',
	'2006-05-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44724b7e9gQvyxTcVqLTzXLWLm8xgQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.43',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4112.241.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.43' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';



-- changeset '1.4112.241.3',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.241.3',
	'[BNX2]: Use kmalloc instead of array

Use kmalloc() instead of a local array in bnx2_nvram_write().

Update version to 1.4.40.',
	'2006-05-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44724ba8A8_veBHdwYS80ORN8155DA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.44',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4112.241.3';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.44' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'wrong_argument'   	   and bug_severity.name=
	'unknown';


-- changeset '1.4112.300.83',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.300.83',
	'[BNX2]: Endian fixes.',
	'2006-06-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4494d6e4loWJ1lr8Bryggc0idJtI0w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.46',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4112.300.83';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.46' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value' 	   	   and bug_severity.name=
	'unknown';



-- changeset '1.4649.1.21',

insert into changeset (cs_number,description,created,url) values (
	'1.4649.1.21',
	'[BNX2]: Turn off link during shutdown

Minor change in shutdown logic to effect a link down.',
	'2006-06-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44a46920RvUaWqj0P9tlrV1swKtpGg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.55',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4112.300.83';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.55' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';



-- changeset '1.4685.108.25',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.108.25',
	'[BNX2]: Fix tx race condition.

Fix a subtle race condition between bnx2_start_xmit() and bnx2_tx_int()
similar to the one in tg3 discovered by Herbert Xu:

CPU0 CPU1
bnx2_start_xmit()
        if (tx_ring_full) {
         tx_lock
         bnx2_tx()
         if (!netif_queue_stopped)
         netif_stop_queue()
         if (!tx_ring_full)
         update_tx_ring
         netif_wake_queue()
         tx_unlock
        }

Even though tx_ring is updated before the if statement in bnx2_tx_int() in
program order, it can be re-ordered by the CPU as shown above.  This
scenario can cause the tx queue to be stopped forever if bnx2_tx_int() has
just freed up the entire tx_ring.  The possibility of this happening
should be very rare though.

The following changes are made, very much identical to the tg3 fix:

1. Add memory barrier to fix the above race condition.

2. Eliminate the private tx_lock altogether and rely solely on
netif_tx_lock.  This eliminates one spinlock in bnx2_start_xmit()
when the ring is full.

3. Because of 2, use netif_tx_lock in bnx2_tx_int() before calling
netif_wake_queue().

4. Add memory barrier to bnx2_tx_avail().

5. Add bp->tx_wake_thresh which is set to half the tx ring size.

6. Check for the full wake queue condition before getting
netif_tx_lock in tg3_tx().  This reduces the number of unnecessary
spinlocks when the tx ring is full in a steady-state condition.',
	'2006-08-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44e4fbf0zqSRktTffdWYicpXb-Hhfw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.61',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4685.108.25';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.61' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'data_race'   	   	   and bug_severity.name=
	'unknown';


-- changeset '1.4685.220.2',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.220.2',
	'[BNX2]: Disable MSI on 5706 if AMD 8132 bridge is present.

MSI is defined to be 32-bit write.  The 5706 does 64-bit MSI writes
with byte enables disabled on the unused 32-bit word.  This is legal
but causes problems on the AMD 8132 which will eventually stop
responding after a while.

Without this patch, the MSI test done by the driver during open will
pass, but MSI will eventually stop working after a few MSIs are
written by the device.

AMD believes this incompatibility is unique to the 5706, and
prefers to locally disable MSI rather than globally disabling it
using pci_msi_quirk.',
	'2006-09-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=451db4ff241r3AUSSN3AtzClbae4SQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.67',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4685.108.25';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.67' 		   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';



-- changeset '1.4913.1.314',

insert into changeset (cs_number,description,created,url) values (
	'1.4913.1.314',
	'[BNX2]: Fix Xen problem.

This fixes the problem of not receiving packets in the Xen bridging
environment.  The Xen script sets the device''s MAC address to
FE:FF:FF:FF:FF:FF and puts the device in promiscuous mode.  The
firmware had problem receiving all packets in this configuration.

New firmware and setting the PROM_VLAN bit when in promiscuous mode
will fix this problem.',
	'2006-12-02',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45725f84Tv5u3zbRrsDPZzsyWQdNJg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.68.1.1',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.4913.1.314';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.68.1.1' 	 	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.5024',

insert into changeset (cs_number,description,created,url) values (
	'1.5024',
	'[BNX2]: Add an error check.

This patch adds a missing error check spotted by the Coverity checker.',
	'2006-12-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=457a0f1abxRagWaJ0cBtjnqf_96itA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.71',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5024';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.71' 	 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'return_check'   	   and bug_severity.name=
	'unknown';



-- changeset '1.5064.14.44',

insert into changeset (cs_number,description,created,url) values (
	'1.5064.14.44',
	'[BNX2]: Fix panic in bnx2_tx_int().

There was an off-by-one bug in bnx2_tx_avail().  If the tx ring is
completely full, the producer and consumer indices may be apart by
256 even though the ring size is only 255.  One entry in the ring is
unused and must be properly accounted for when calculating the number
of available entries.  The bug caused the tx ring entries to be
reused by mistake, overwriting active entries, and ultimately causing
it to crash.

This bug rarely occurs because the tx ring is rarely completely full.
We always stop when there is less than MAX_SKB_FRAGS entries available
in the ring.',
	'2006-12-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45862e33cPNmGlIwsikfvuJgvh_XJQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.72',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5064.14.44';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.72' 	 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.5064.14.45',

insert into changeset (cs_number,description,created,url) values (
	'1.5064.14.45',
	'[BNX2]: Fix bug in bnx2_nvram_write().

Length was not calculated correctly if the NVRAM offset is on a non-
aligned offset.',
	'2006-12-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45862e34QgwiuMnIQ9L9aDxkbeHB4g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.73',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5064.14.45';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.73' 	 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';




-- changeset '1.8060.4.88',

insert into changeset (cs_number,description,created,url) values (
	'1.8060.4.88',
	'[BNX2]: Disable parallel detect on an HP blade.

Because of some board issues, we need to disable parallel detect on
an HP blade.  Without this patch, the link state can become stuck
when it goes into parallel detect mode.',
	'2008-02-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47c0e929vjyfr0dVo9m29oRkp2kyLw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.193',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.8060.4.88';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.193' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';



-- changeset '1.7946.2.591',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.2.591',
	'[BNX2]: Fix ASYM PAUSE advertisement for remote PHY.

We were checking for the ASYM_PAUSE bit for 1000Base-X twice instead
checking for both the 1000Base-X bit and the 10/100/1000Base-T bit.
The purpose of the logic is to tell the firmware that ASYM_PAUSE is
set on either the Serdes or Copper interface.',
	'2008-01-31',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47a29203ncRYesN0rfMdn5ZLabIN3w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.190',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.7946.2.591';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.190' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.7946.2.489',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.2.489',
	'[BNX2]: Fix 5706 serdes link down bug.

1. Correct the MII expansion serdes control register definition.
2. Check an additional RUDI_INVALID bit when determining 5706S link.',
	'2008-01-31',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47a2919dFEUwxvKNfvJ9GH2SFI9Vlg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.184',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.7946.2.489';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.184' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.7946.118.1369',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.118.1369',
	'[BNX2]: Add link-down workaround on 5706 serdes.

In some blade systems using the 5706 serdes, the hardware sometimes
does not properly generate link down interrupts.  We add a workaround
in the driver''s timer to force a link-down when some PHY registers
report loss of SYNC.

The parallel detect logic is cleaned up slightly to better integrate
the workaround.',
	'2008-01-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=479e6242YEa68YIarCfOzAGF6m5Rrg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.179',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.7946.118.1369';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.179' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';



-- changeset '1.7946.118.1367',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.118.1367',
	'[BNX2]: Disable jumbo rx paging on 5709 Ax.

The chip has problem running in this mode and needs to be disabled.',
	'2008-01-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=479e6240-oaXLGigI6evWCPIP1y_jA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.177',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.7946.118.1367';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.177' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';



-- changeset '1.7831',

insert into changeset (cs_number,description,created,url) values (
	'1.7831',
	'[BNX2]: Fix RX packet rot.

Packets can be left in the RX ring if the NAPI budget is reached.
This is caused by storing the latest rx index at the beginning of
bnx2_rx_int().  We may not process all the work up to this index
if the budget is reached and so some packets in the RX ring may rot
when we later check for more work using this stored rx index.

The fix is to not store this latest hw index and only store the
processed rx index.  We use a new function bnx2_get_hw_rx_cons()
to fetch the latest hw rx index.',
	'2007-12-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=475e6a4chYpWVaqvN4P_IpNJApJ3aQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.156',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.7831';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.156' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';



-- changeset '1.7830',

insert into changeset (cs_number,description,created,url) values (
	'1.7830',
	'[BNX2]: Add PHY_DIS_EARLY_DAC workaround.

5709 Ax and Bx chips all need this workaround.',
	'2007-12-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=475e6a4bBjO_ms2MkqBcL8f8R4cGvA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.155',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.7830';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.155' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


-- changeset '1.7176.5.2',

insert into changeset (cs_number,description,created,url) values (
	'1.7176.5.2',
	'[BNX2]: Fix Serdes WoL bug.

The bug is in the code in bnx2_set_power_state() that assumes copper
devices when setting up WoL.  This is no longer true after adding WoL
support for Serdes devices.',
	'2007-10-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4716c447k5AUpUHUO3Gob-QJTSwPyg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.151',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.7176.5.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.151' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';

-- changeset '1.6557',

insert into changeset (cs_number,description,created,url) values (
	'1.6557',
	'[BNX2]: Refine napi poll loop.

Need to read and store sblk->status_idx before checking for more work.
The status idx is later written back to the hardware when enabling
interrupts to acknowledge how much work has been processed.  If the
order is reversed, we can end up acknowledging work we haven''t
processed.

When completing bnx2_poll(), we should always break out of the while
loop and return work_done instead of returning 0.',
	'2007-10-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=470f37e1E7zex3s27_ju8c8OZ8o2Pg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.150',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.6557';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.150' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_race'   	   and bug_severity.name=
	'unknown';

-- changeset '1.6247.140.866',

insert into changeset (cs_number,description,created,url) values (
	'1.6247.140.866',
	'[NET]: Fix NAPI completion handling in some drivers.

In order for the list handling in net_rx_action() to be
correct, drivers must follow certain rules as stated by
this comment in net_rx_action():

/* Drivers must not modify the NAPI state if they
 * consume the entire weight.  In such cases this code
 * still "owns" the NAPI instance and therefore can
 * move the instance around on the list at-will.
 */

A few drivers do not do this because they mix the budget checks
with reading hardware state, resulting in crashes like the one
reported by takano@axe-inc.co.jp.

BNX2 and TG3 are taken care of here, SKY2 fix is from Stephen
Hemminger.',
	'2007-10-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=470ec90dBhS2v7vlWKTTjKuHxTGMEg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.149',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.6247.140.866';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.149' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6247.140.834',

insert into changeset (cs_number,description,created,url) values (
	'1.6247.140.834',
	'[BNX2]: Fix default WoL setting.

Change the default WoL setting to match the NVRAM''s setting.  It
always defaulted to WoL disabled before and caused a lot of confusion
for users.',
	'2007-10-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=470d67d23F_6IVtX0Js5LGIXn3USfQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.147',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.6247.140.834';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.147' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6247.65.13',

insert into changeset (cs_number,description,created,url) values (
	'1.6247.65.13',
	'[BNX2]: Add PHY workaround for 5709 A1.

Add the DIS_EARLY_DAC PHY workaround for 5709 A1.  Without it, link
sometimes does not come up.

Update version to 1.6.5.',
	'2007-09-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46f2c68d_jlqERCpejRZldgWj8WiCg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.135',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.6247.65.13';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.135' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6171.2.8',

insert into changeset (cs_number,description,created,url) values (
	'1.6171.2.8',
	'[BNX2]: Fix suspend/resume problem.

The device would not resume properly if it was shutdown before the system
was suspended.  In such scenario where the netif_running state is 0,
bnx2_suspend() would not save the PCI state and so the memory enable bit
and bus master enable bit would be lost.

We fix this by always saving and restoring the PCI state in
bnx2_suspend() and bnx2_resume() regardless of netif_running() state.',
	'2007-08-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46b3f925EeU0_11X1bNYyX6RkYhnjg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.132',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.6171.2.8';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.132' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'bus_pm_protocol'   	   and bug_severity.name=
	'unknown';

-- changeset '1.6154.23.2',

insert into changeset (cs_number,description,created,url) values (
	'1.6154.23.2',
	'[BNX2]: Add delay before reading firmware version.

The management firmware may still be loading during bnx2_init_one()
because of the D3hot -> D0 transition and the firmware version may
not be available without waiting a bit.',
	'2007-07-18',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=469dd377PinTrrdFvqq684_YRY1gbQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.128',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.6154.23.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.128' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_timing'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6048.25.2',

insert into changeset (cs_number,description,created,url) values (
	'1.6048.25.2',
	'[BNX2]: Add missing wait in bnx2_init_5709_context().

For correctness, we need to wait for the MEM_INIT bit to be cleared
in the BNX2_CTX_COMMAND register before proceeding.

[Added return -EBUSY when the MEM_INIT bit doesn''t clear, suggested
by Jeff Garzik.]',
	'2007-06-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46686ccdGcjFiar2xKaY_btzrGNmiA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.110',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.6048.25.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.110' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_timing'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6048.25.1',

insert into changeset (cs_number,description,created,url) values (
	'1.6048.25.1',
	'[BNX2]: Fix netdev watchdog on 5708.

There''s a bug in the driver that only initializes half of the context
memory on the 5708.  Surprisingly, this works most of the time except
for some occasional netdev watchdogs when sending a lot of 64-byte
packets.  The fix is to add the missing code to initialize the 2nd
halves of all context memory.',
	'2007-06-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46686cccGrb4Thd5ZEv1rap6CUVx8g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.109',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.6048.25.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.109' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.5791.104.37',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.104.37',
	'[BNX2]: Fix TSO problem with small MSS.

Remove the check for skb->len greater than MTU when doing TSO.  When
the destination has a smaller MSS than the source, a TSO packet may
be smaller than the MTU at the source and we still need to process it
as a TSO packet.',
	'2007-05-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46686cccGrb4Thd5ZEv1rap6CUVx8g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.107',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5791.104.37';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.107' 	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.5791.104.22',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.104.22',
	'[BNX2]: Fix race conditions when calling register_netdev().
    
Hot-plug scripts can call bnx2_open() as soon as register_netdev() is
called in bnx2_init_one().  We need to call pci_set_drvdata() and
setup everything before calling register_netdev(). netif_carrier_off()
also needs to be moved to bnx2_open() to avoid race conditions with
the irq.',
	'2007-05-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=463a4403q7KLQpDKl_aWmRQe61IU5g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.96',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5791.104.22';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Hot-plug scripts can call bnx2_open() as soon as register_netdev() is
called in bnx2_init_one().  We need to call pci_set_drvdata() and
setup everything before calling register_netdev().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.96' 	   	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'control_race'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'netif_carrier_off() also needs to be moved to bnx2_open() to avoid race conditions with
the irq.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.96' 	   	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'control_data_race'   	   and bug_severity.name=
	'unknown';


-- changeset '1.5791.104.20',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.104.20',
	'[BNX2]: Fix register and memory test on 5709.

Tweak registers and memory test range for 5709.',
	'2007-05-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=463a43a6IQ8g3yDqeufrDARJZh3qdQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.94',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5791.104.20';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.94' 	   	   	   and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


-- changeset '1.5791.104.19',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.104.19',
	'[BNX2]: Block MII access when ifdown.

The device may be in D3hot state and should not allow MII register
access.',
	'2007-05-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=463a437bnaZCKiqnpt1CLVrCfCuArQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.93',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5791.104.19';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.93' 	   	   	and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'bus_pm_protocol'   	and bug_severity.name=
	'unknown';


-- changeset '1.5791.53.4',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.53.4',
	'[BNX2]: Fix occasional NETDEV WATCHDOG on 5709.

Tweak a register setting to prevent the tx mailbox from halting.

Update version to 1.5.8.',
	'2007-04-24',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=462e8649y6tS5dvREHsB85ktUKH7Xg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.87',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5791.53.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.87' 	   	   	and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value'   	and bug_severity.name=
	'unknown';

-- changeset '1.5791.1.17',

insert into changeset (cs_number,description,created,url) values (
	'1.5791.1.17',
	'[BNX2]: Fix nvram write logic.

The nvram dword alignment logic was broken when writing less than 4
bytes on a non-aligned offset.  It was missing logic to round the
length to 4 bytes.

The page erase code is also moved so that it is only called when
using non-buffered flash for better code clarity.

Update version to 1.5.7.',
	'2007-04-02',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=461167ffrCOQrqyY3d2E9et9v0rV5g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.86',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5791.1.17';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.86' 	   	   	and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_value' and bug_severity.name=
	'unknown';


-- changeset '1.5342.4.1',

insert into changeset (cs_number,description,created,url) values (
	'1.5342.4.1',
	'[BNX2]: PHY workaround for 5709 A0.

5709 A0 copper devices will not link up with some link partners
without this workaround.',
	'2007-02-02',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45c2fa6bOD2XGGKNH4RHa_EqOcXkMA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.80',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5342.4.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.80' 	   	   	and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'device_ordering' 	and bug_severity.name=
	'unknown';


-- changeset '1.5225.19.14',

insert into changeset (cs_number,description,created,url) values (
	'1.5225.19.14',
	'[BNX2]: Fix bug in bnx2_nvram_write().

The bug was a bogus pointer being passed to kfree().  The pointer was
incremented in the write loop and then passed to kfree().

The fix is to use align_buf to save the original address.',
	'2007-01-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45a3528ca7Hw_vk5SpCo9S9CTXnyVg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.77',
	'',			module.id, changeset.id from module, changeset where module.name=
	'bnx2' 			and changeset.cs_number=  
	'1.5225.19.14';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.77' 	   	   	and module_revision.module in (select id from module where name = 'bnx2')and bug_subcategory.name=
	'logic' 		and bug_severity.name=
	'unknown';







---------------------------------------------
-- Intel 810 frame buffer device 
---------------------------------------------


insert into module (name,description,path,created) values(
	'i810 fb', 
	'Intel 810 frame buffer device',
	'drivers/video/i810',
	'2001-01-01');


-- changeset '1.8153.2.20',

insert into changeset (cs_number,description,created,url) values (
	'1.8153.2.20',
	'i810fb: fix console switch regression

Since 4c7ffe0b9f7f40bd818fe3af51342f64c483908e ("fbdev: prevent drivers that
have hardware cursors from calling software cursor code") every call of
i810fb_cursor fails with -ENXIO because of a incorrect "!".

This hasn''t struck until eaa0ff15c30dc9799eb4d12660edb73aeb6d32c5 ("fix !
versus & precedence in various places") surrounded the expression with braces,
so that the intended behavior was inverted.  That caused ''pixel waste'' - the
same line of multi-colored pixels repeated over the whole screen - during
console switch.

This switches back to the original pre-4c7ffe0 behavior.',
	'2008-03-19',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47e1c3b2pJnIEZwMmrfObpqP6xsHrQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.65',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.8153.2.20';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.65' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.5353.30.184',

insert into changeset (cs_number,description,created,url) values (
	'1.5353.30.184',
	'[PATCH] Video: fb, add true ref_count atomicity

Some of fb drivers uses atomic_t in bad manner, since there are still some
race-prone gaps.  Use mutexes to protect open/close code sections with
ref_count testing and finally use simple uint.',
	'2007-02-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45d0a922f03v-0pm7lZPxM-i7X2c0Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.59',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.5353.30.184';

insert into bug (description, revision_removed, subcategory, severity) select 
	'ref_count was first compared to 0 and then incremented atomically.  The fix was to protect
both operations with a mutex',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.59' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'control_race'   	   and bug_severity.name=
	'unknown';



-- changeset '1.4685.221.159',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.221.159',
	'[PATCH] i810fb: Honor the return value of pci_enable_device

Check the return value of pci_enable_device().',
	'2006-10-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45227c13R0ycNBsU31fkgtpB-LgT1Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.58',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.4685.221.159';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.58' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'return_check'   	   and bug_severity.name=
	'unknown';


-- changeset '1.4112.11.4',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.11.4',
	'[PATCH] i810fb_cursor(): use GFP_ATOMIC

The console cursor can be called in atomic context.  Change memory
allocation to use the GFP_ATOMIC flag in i810fb_cursor().',
	'2006-03-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44217315Gg36ldP7cir4pPA04bpCEg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.52',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.4112.11.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.52' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';


-- changeset '1.4087.145.154',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.145.154',
	'[PATCH] i810fb: Do not probe the third i2c bus by default

Some time before 2.6.15, a third DDC channel was added to i810fb.  On
systems where these ddc pins are not connected, the probe takes about 10
seconds.

Add a boot/module option for i810fb to explicitly probe for the 3rd ddc bus
if needed.',
	'2006-02-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43e0e80dM24t9y6X13xppYr0GsIaTA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.51',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.4087.145.154';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.51' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'device_ordering'	   and bug_severity.name=
	'unknown';


-- changeset '1.4087.38.136',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.38.136',
	'[PATCH] i810fb: Fix suspend and resume hooks

The i810fb suspend and resume hooks have suffered bitrot over time.  Update to
current framework, but functionality still not guaranteed to work.',
	'2006-01-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43c3dadaIYkKkz571OExSajg7X4LWg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.49',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.4087.38.136';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.49' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'collateral'		   and bug_severity.name=
	'unknown';


-- changeset '1.3332.323.17',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.323.17',
	'[PATCH] i810fb: Change option ext_vga to extvga to match documentation

Reported by: Manuel Lauss <mano@roarinelk.homelinux.net>

He was getting random initial video modes depending on the kernel
configuration.  His option line includes ''extvga''.

The i810fb documentation describes the option ''extvga'', however the
driver accepts ''ext_vga''.  Besides ''extvga'' being ignored by i810fb,
it also confuses the option parser of i810fb and assigns ''extvga'' to
''mode_option''.  This leads to an incorrect video mode at boot time.',
	'2005-09-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4324f982Pshi0fcrYkQ6mrNowwGeQA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.44',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.3332.323.17';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.44' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'logic'		   	   and bug_severity.name=
	'unknown';



-- changeset '1.3332.323.16',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.323.16',
	'[PATCH] i810fb: Restore xres and yres option parameters

If i810fb successfully probed for the EDID, it will disregard the
boot option parameters ''xres'' and ''yres''.  Fix this regression.

Excellent testing done by Manuel Lauss <mano@roarinelk.homelinux.net>.',
	'2005-09-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4324f981bayV9Bcp_bcH1or3w9wpJA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.43',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.3332.323.16';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The driver ignored initialisation parameters',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.43' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';



-- changeset '1.3332.323.15',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.323.15',
	'[PATCH] i810fb: Fix oops if CONFIG_FB_I810_I2C is set to no

Reported by: Manuel Lauss <mano@roarinelk.homelinux.net>

compiled with CONFIG_FB_I810_I2C = n and CONFIG_FB_I810 = y
it oopses at boot in file drivers/video/i810/i810_main.c:1884',
	'2005-09-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4324f980BmHWxHItiX6pVZN7sjvSJQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.42',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.3332.323.15';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.42' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'logic'	   	   	   and bug_severity.name=
	'unknown';




-- changeset '1.3332.310.179',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.310.179',
	'[PATCH] i810fb: Stop LCD displays from flickering

Stop LCD displays from flickering during high loads.',
	'2005-09-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4321f8b4L5q4KjNjDgaJcxppamlCyQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.41',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.3332.310.179';

insert into bug (description, revision_removed, subcategory, severity) select 
	'+	/* Stop LCD displays from flickering */
+	i810_writel(MEM_MODE, mmio, i810_readl(MEM_MODE, mmio) | 4);',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.41' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'device_ordering'	   and bug_severity.name=
	'unknown';


-- changeset '1.3256.4.142',

insert into changeset (cs_number,description,created,url) values (
	'1.3256.4.142',
	'[PATCH] fbdev: fill in the access_align field.

Several drivers miss filling in the access_align field.  So this patch has
them fill it in.',
	'2005-06-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=42b8c818XKscqzjyamVglNMn602wrQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.37',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.3256.4.142';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.37' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'os_data_struct'	   and bug_severity.name=
	'unknown';


-- changeset '1.2850.1.46',

insert into changeset (cs_number,description,created,url) values (
	'1.2850.1.46',
	'[PATCH] i810fb: fix __initdata access

[hv]sync[12] are __initdata, causing mplayer to oops with the previous i810fb fix.

My fault, this fixes it. Sorry.',
	'2005-05-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4278e127Ug_HAhB_AmHpidGF1Phg1g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.36',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.2850.1.46';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.36' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'use_after_free'	   and bug_severity.name=
	'unknown';


-- changeset '1.1938.340.63',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.340.63',
	'[PATCH] fbdev: Fix broken fb_blank() implementation.

This patch fixes some of the drivers'' fb_blank() implementation which got
the usage of the VESA_* constants incorrectly and converts them to use the
new FB_BLANK-* constants.

I''m not sure if what I did is correct for all drivers, so maintainers,
please review.

(Note: For most of the drivers, FB_BLANK_NORMAL is treated as
FB_BLANK_UNBLANK, but returns a nonzero so fbcon will do a soft_blank).',
	'2004-11-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4193de09lVgX6eebE5dWtT8pX4-OGw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.28',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.1938.340.63';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.28' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'os_data_struct'	   and bug_severity.name=
	'unknown';


-- changeset '1.1938.111.61',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.111.61',
	'[PATCH] PCI: audit all callers of pci_register_driver() to work properly.

No, pci_register_driver() does not return the number of pci devices found, sorry.
No, if pci_register_driver() fails, you do not need to call pci_unregister_driver().',
	'2004-10-06',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41645eaaS9Tm7fBS7jRIh5MmwFlzOA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.21',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.1938.111.61';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.21' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'bus_config_protocol'	   and bug_severity.name=
	'unknown';


-- changeset '1.1803.144.124',

insert into changeset (cs_number,description,created,url) values (
	'1.1803.144.124',
	'[PATCH] i810fb fixes

1. Fixed cursor corruption if acceleration is enabled
2. Round up fields in var instead of rounding down
3. Set capabilities flags
4. Added myself to the MAINTAINERS file for i810fb
5. Make i810fb depend on X86 but not X86_64
6. Fixed typo in i810_init_monspecs(). Reported by Manuel Lauss
   <slauss@resi.at>.',
	'2004-08-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=412a4a99jLAxcAwp8Mzxe7841EPR7A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.16',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.1803.144.124';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Fixed cursor corruption if acceleration is enabled',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.16' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'device_value'	   	   and bug_severity.name=
	'unknown';


-- changeset '1.1046.589.39',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.589.39',
	'[PATCH] i810fb oops fix

The module device table is not NULL-terminated, so we run off the end during
probing and oops.',
	'2003-08-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f2a9f1927Ahv-Q-g8IBu5r5DUhzxg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.12',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'i810 fb'		and changeset.cs_number=  
	'1.1046.589.39';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.12' 	   		   and module_revision.module in (select id from module where name = 'i810 fb')and bug_subcategory.name=
	'os_data_struct'	   and bug_severity.name=
	'unknown';



---------------------------------------------
-- USB serial 
---------------------------------------------


insert into module (name,description,path,created) values(
	'usb serial', 
	'USB Serial Converter driver',
	'drivers/usb/serial/usb-serial.c',
	'1999-01-01');


-- changeset '1.7946.147.103',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.147.103',
	'USB: stop io performed by mos7720 upon close()

This fixes a problem where the mos7720 driver will make io to a device from
which it has been logically disconnected. It does so by introducing a flag by
which the generic usb serial code can signal the subdrivers their
disconnection and appropriate locking.',
	'2008-02-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47a39ee8GNyiI5Z7EiyO1f01kVleug');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.186',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.7946.147.103';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.186' 	   	   and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'disconnect_race'   	   and bug_severity.name=
	'unknown';


-- changeset '1.7946.147.37',

insert into changeset (cs_number,description,created,url) values (
	'1.7946.147.37',
	'USB: usb_serial: clean tty reference in the last close

When a usb serial adapter is used as console, the usb serial console
driver bumps the open_count on the port struct used but doesn''t attach
a real tty to it (only a fake one temporaly). If this port is opened later
using the regular character device interface, the open method won''t
initialize the port, which is the expected, and will receive a brand new
tty struct created by tty layer, which will be stored in port->tty.

When the last close is issued, open_count won''t be 0 because of the
console usage and the port->tty will still contain the old tty value. This
is the last ttyUSB<n> close so the allocated tty will be freed by the
tty layer. The usb_serial and usb_serial_port are still in use by the
console, so port_free() won''t be called (serial_close() ->
usb_serial_put() -> destroy_serial() -> port_free()), so the scheduled
work (port->work, usb_serial_port_work()) will still run. And
usb_serial_port_work() does:
(...)
        tty = port->tty;
        if (!tty)
                return;

        tty_wakeup(tty);
which causes (manually copied):

Faulting instruction address: 0x6b6b6b68
Oops: Kernel access of bad area, sig: 11 [#1]',
	'2008-02-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=47a39ea6wyn2_e75xex8dvjHElXI4g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.184',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.7946.147.37';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Classifying this bug as OS protocol violation rather than reference counting error,
because it is caused by a wrong assumption about the ordering of requests for the OS.  Namely,
that a close () will be called for every open, which is not the case',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.184' 	   	   and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_config_protocol'   	   and bug_severity.name=
	'unknown';




-- changeset '1.7416.44.35',

insert into changeset (cs_number,description,created,url) values (
	'1.7416.44.35',
	'USB: usb_serial_resume bug fix
Avoid potential null pointer dereference.',
	'2007-10-25',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4720ec32ZgTik_qrs20q-a0CTjl4Bg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.183',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.7416.44.35';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The error is in the interface between the generic usb serial driver
and device-specific bits.  Since it does not have anything to do with device or
OS interface, marking it as a generic program logic error',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.183' 	   	   and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';



-- changeset '1.6555.11.139',

insert into changeset (cs_number,description,created,url) values (
	'1.6555.11.139',
	'USB: serial core should respect driver requirements

This patch (as997) fixes a bug in the USB serial core.  The core needs
to pay attention to drivers'' requirements regarding the number and
type of endpoints a device has.

At the same time, the patch changes the NUM_DONT_CARE constant (which
is stored in a single-byte field) from -1 to a safer, unsigned value.
It also improves the kerneldoc for several fields in the
usb_serial_driver structure.

Finally, the patch replaces a list_for_each() with list_for_each_entry().',
	'2007-10-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=470fedbcvIV1TNci1W_jLPqn9aqQjw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.181',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.6555.11.139';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The error is in the interface between the generic usb serial driver
and device-specific bits.  Since it does not have anything to do with device or
OS interface, marking it as a generic program logic error',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.181' 	   	   and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


-- changeset '1.6555.11.98',

insert into changeset (cs_number,description,created,url) values (
	'1.6555.11.98',
	'USB: fix errornous assumption in the usb serial framework revealed by iuu_phoenix

the iuu_phoenix driver submits another URB from a completion handler.
This dictates a certain order of calls to usb_kill_urb() in kill_traffic().
As other drivers may do it the other way round, we need to use both
orders in kill_traffic().
This patch does so and should be merged before iuu_phoenix is merged.',
	'2007-10-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=470fed939W-5v48aW5PbgQP7mQht3Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.180',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.6555.11.98';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The error is in the interface between the generic usb serial driver
and device-specific bits.  Since it does not have anything to do with device or
OS interface, marking it as a generic program logic error',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.180' 	   	   and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';



-- changeset '1.6171.36.14',

insert into changeset (cs_number,description,created,url) values (
	'1.6171.36.14',
	'USB: fix BUG: sleeping function called from invalid context at /home/jeremy/hg/xen/paravirt/linux/drivers/usb/core/urb.c:524, in_atomic():1, irqs_disabled():0

Clearly there''s a bug in
drivers/usb/serial/usb-serial.c:usb_serial_put().  It shouldn''t call
kref_put() while holding a spinlock.',
	'2007-07-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46ae49ccIUboT0Ps4FE_L-wUrPj5Yg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.179',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.6171.36.14';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.179' 	   	   and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';



-- changeset '1.6171.36.8',

insert into changeset (cs_number,description,created,url) values (
	'1.6171.36.8',
	'USB: fix usb_serial_suspend(): buggy code

Am Montag 23 Juli 2007 schrieb Adrian Bunk:
> Commit ec22559e0b7a05283a3413bda5d177e42c950e23 added the following 
> function to drivers/usb/serial/usb-serial.c:
> 
[..]
> 
> The Coverity checker spotted the inconsequent NULL checking for "serial".
> 
> Looking at the code it also doesn''t seem to have been intended to always 
> return 0.

Coverity is right. The check for NULL is wrongly done and the error
return is lost.',
	'2007-07-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46ae49c6stL1x9QzXME75MoSeAk0qQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.178',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.6171.36.8';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.178' 	   	   and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'  		   and bug_severity.name=
	'unknown';



-- changeset '1.6048.146.68',

insert into changeset (cs_number,description,created,url) values (
	'1.6048.146.68',
	'synchronization in usb_serial_put

I think there is a race between usb_serial_put() and
usb_serial_get_by_index() (and get_free_serial()) with regards
to handling the serial port refcount.

usb_serial_get_by_index() gets a reference on the serial port under
table_lock while return_serial releases all the returned ports
from the table under the same lock.  However, the table_lock is not
taken around the call to kref_put, theoretically allowing to sneak
in and grab a reference after kref_put has already determined that
the reference count is zero (and before calling destroy_serial)
causing use after free.',
	'2007-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4696ba99JUrIoLP0sTUpAcKU1VJhCg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.177',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.6048.146.68';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.177'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';



-- changeset '1.5620.3.17',

insert into changeset (cs_number,description,created,url) values (
	'1.5620.3.17',
	'USB: further fix for usb-serial
this is a fix for the outstanding usb-serial issues.',
	'2007-03-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45f22b86h3byqXbXzstUZpXePunNsQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.173',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.5620.3.17';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The error is in the interface between the generic usb serial driver
and device-specific bits.  Since it does not have anything to do with device or
OS interface, marking it as a generic program logic error',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.173'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';



-- changeset '1.5620.3.16',

insert into changeset (cs_number,description,created,url) values (
	'1.5620.3.16',
	'USB: fix usb-serial device naming bug

Am Montag, 26. Februar 2007 15:16 schrieb Craig Schlenter:
> Hi Greg
> 
> 34ef50e5b1f96c2d8c0f3d28b7d407743806256c is definitely
> the source of the problem. Reverting that makes the
> ftdi port show up as ttyUSB0 again for me and it
> can actually be opened.

This patch should fix the issue.',
	'2007-03-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45f22b85ShJjTmhG4OAL9I_FnNvsRA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.172',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.5620.3.16';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The driver assigned the minor device number before calling get_free_serial () (use-before-initialise)',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.172'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';



-- changeset '1.5620.3.13',

insert into changeset (cs_number,description,created,url) values (
	'1.5620.3.13',
	'usb-serial: fix shutdown / device_unregister order

Ensure that the ->port_remove() callbacks get called before the
->shutdown() callback which makeing the order symmetric with
->attach() being called before ->port_probe().',
	'2007-03-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45f22b82dlEgpP-uPkVxpCaxwJO-8w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.171',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.5620.3.13';

insert into bug (description, revision_removed, subcategory, severity) select
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.171'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_config_protocol'	and bug_severity.name=
	'unknown';


-- changeset '1.5353.5.50',

insert into changeset (cs_number,description,created,url) values (
	'1.5353.5.50',
	'USB: race fixes for usb-serial, step 3
- fix an error code returned if a device has been disconnected',
	'2007-02-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45ca648f4vcDhQ_XPmfcyequGW4rHQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.170',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.5353.5.50';

insert into bug (description, revision_removed, subcategory, severity) select 
	'returned -EINVAL instead of -ENODEV',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.170'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'error_reporting'	and bug_severity.name=
	'unknown';


-- changeset '1.5353.5.49',

insert into changeset (cs_number,description,created,url) values (
	'1.5353.5.49',
	'USB: race fixes for usb-serial, step 2

- take BKL before looking up a driver to associate with a device to make
  sure the module is not unloaded after looking up but before association
  & bumping module count',
	'2007-02-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45ca648eAjvwN1tgp8JSxVRj6_aa2A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.169',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.5353.5.49';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.169'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';




-- changeset '1.5353.5.48',

insert into changeset (cs_number,description,created,url) values (
	'1.5353.5.48',
	'USB: race fixes for usb-serial step 1

- introduce a spinlock for serial_table to eliminate the window between
  looking up a device and getting a reference
- delay inscription of a new device into serial_table until it is fully
  initialised
- make sure disconnect() kills all URBs to avoid leckage across a soft unbind',
	'2007-02-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45ca648d71u1KHdNa69lzsMbgKyQBA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.168',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.5353.5.48';

insert into bug (description, revision_removed, subcategory, severity) select 
	'- introduce a spinlock for serial_table to eliminate the window between
looking up a device and getting a reference
- delay inscription of a new device into serial_table until it is fully
  initialised',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.168'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'- make sure disconnect() kills all URBs to avoid leckage across a soft unbind',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.168'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';


-- changeset '1.4685.152.1',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.152.1',
	'usbserial: Reference leak

A sufficiently-large number of USB serial devices causes a reference leak
when /proc/tty/drivers/usbserial is read.',
	'2006-09-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4506878c6AEqPKyex0aXh2-AsifObw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.159',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4685.152.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.159'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'reference_counting'	and bug_severity.name=
	'unknown';


-- changeset '1.4685.33.9',

insert into changeset (cs_number,description,created,url) values (
	'1.4685.33.9',
	'[PATCH] USB: fix usb-serial leaks, oopses on disconnect

This fix addresses two issues:
- Unattached port structures were not freed
- My initial fix for crash when eventd runs a work in a freed port
  did not go far enough',
	'2006-07-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=44b57fc0XQ5rVP2bj6ubMWRJJ5hysg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.157',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4685.33.9';

insert into bug (description, revision_removed, subcategory, severity) select 
	'- Unattached port structures were not freed',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.157'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'resource_leak'		and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'- My initial fix for crash when eventd runs a work in a freed port
did not go far enough (fixed by adding a flush_scheduled_work() call to port_free ())',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.157'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';



-- changeset '1.4112.323.115',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.323.115',
	'[PATCH] usbserial: Fixes wrong return values.

Some usbserial functions returns -EINVAL if the port doesn''t exist or if
it''s not opened. However, the right error code for such situations is
-ENODEV.',
	'2006-06-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4499c2b47PS3ptEZHxx6bVjutnhS4Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.153',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4112.323.115';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.153'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'error_reporting'	and bug_severity.name=
	'unknown';



-- changeset '1.4112.323.109',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.323.109',
	'[PATCH] USB Serial: clean tty fields on failed device open

If either the driver''s open() method or try_module_get() fails, we need to
set ''tty->driver_data'' and ''port->tty'' to NULL in serial_open(), otherwise
we''ll get an OOPS in usb_device_disconnect() when the device is disconnected.',
	'2006-06-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4499c2ae5EnHt-bS_gN_f3kp7A4Omg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.152',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4112.323.109';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.152'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_data_struct'	and bug_severity.name=
	'unknown';


-- changeset '1.4112.323.75',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.323.75',
	'[PATCH] USB: rmmod pl2303 after -28

Wait for the scheduled work to finish before freeing memory, prevent oops.
http://bugzilla.kernel.org/show_bug.cgi?id=6596',
	'2006-06-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4499c28chIDs5dF4AFO9L5dfBwXndQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.151',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4112.323.75';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.151'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';


-- changeset '1.4112.323.34',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.323.34',
	'[PATCH] USB console: fix disconnection issues

Prevent sending further output to a USB-serial console after the dongle is
disconnected, take care not to leak kref.',
	'2006-06-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4499c263x1kNFP9Qf8yOrsc-dz6HMQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.149',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4112.323.34';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The bug violated the "no bus operations after disconnect" rule.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.149'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_disconnect_protocol' and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.149'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'reference_counting'	and bug_severity.name=
	'unknown';


-- changeset '1.4112.206.14',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.206.14',
	'[PATCH] usbserial: Fixes leak in serial_open() error path.

If serial_open() fails at the port assignment or mutex_lock_interruptible()
is interrupted, the ''serial'' object will never be freed.

We should call kref_put() when those errors happens.',
	'2006-05-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4464dac4XKnoDinpA1ZQQbbkeiD04Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.147',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4112.206.14';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.147'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'reference_counting'	and bug_severity.name=
	'unknown';



-- changeset '1.4112.206.13',

insert into changeset (cs_number,description,created,url) values (
	'1.4112.206.13',
	'[PATCH] usbserial: Fixes use-after-free in serial_open().

If the device is disconnected while serial_open() is executing and
either try_module_get() or the device specific open function fails, the
kref_put() call in the ''bailout_kref_put'' label will free the memory
pointed out by ''port''.

The subsequent dereferences in the ''bailout_kref_put'' label will be
invalid.

The fix is just to assure kref_put() is called after any ''port'' usage.',
	'2006-05-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4464dac3355gspfvvxgR5OdsRQ59Nw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.146',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4112.206.13';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.146'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'use_after_free'	and bug_severity.name=
	'unknown';


-- changeset '1.369.80.4',

insert into changeset (cs_number,description,created,url) values (
	'1.369.80.4',
	'USB serial drivers

Several functions in the serial drivers can be called from bottom
half or interrupt context. They must use the GFP_ATOMIC flag for
calls to kmalloc() and usb_submit_urb().
  
Functions which must use GFP_ATOMIC:
1. All *_callback() functions.
2. Any code which is inside a spinlock.
3. write(), throttle(), unthrottle(), which may be called by
   the line discipline in bottom half context.
  
Functions which can use GFP_KERNEL:
1. open(), close(), startup(), shutdown(), set_termios().',
	'2002-03-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3c94e0faNnOPja38KopV67h0jAry_A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.26',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.369.80.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	result = usb_submit_urb(port->read_urb, GFP_KERNEL);
+	result = usb_submit_urb(port->read_urb, GFP_ATOMIC);',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.26'			  and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'unknown';


-- changeset '1.369.79.31',

insert into changeset (cs_number,description,created,url) values (
	'1.369.79.31',
	'USB serial driver core

- Moved all manipulation of port->open_count into the core.  Now the
  individual driver''s open and close functions are called only when the
  first open() and last close() is called.  Making the drivers a bit
  smaller and simpler.
- Fixed a bug if a driver didn''t have the owner field set.',
	'2002-03-21',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3c9a7f13-djID9faJIbA0nzu16oZBg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.27',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.369.79.31';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The error is in the interface between the generic usb serial driver
and device-specific bits.  Since it does not have anything to do with device or
OS interface, marking it as a generic program logic error',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.27'			  and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'	  		  and bug_severity.name=
	'unknown';


-- changeset '1.369.79.48',

insert into changeset (cs_number,description,created,url) values (
	'1.369.79.48',
	'USB serial core

Module count of a serial converter driver is currently not
decremented if a disconnect happens while the tty is held open.
The fix is to close the device in usb_serial_disconnect() so that
module refcounts are properly updated.',
	'2002-03-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3ca36c15JHLVl6CY-5nwbVaXZjn14A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.29',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.369.79.48';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.29'			  and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'reference_counting'	  and bug_severity.name=
	'unknown';


-- changeset '1.403.163.7',

insert into changeset (cs_number,description,created,url) values (
	'1.403.163.7',
	'[PATCH] USB: usbserial.c fixup

create_serial, get_free_serial and usb_serial_probe all do pretty much
the same thing. I''d like to reorg this into create_serial does all the
alloc and most of the setup, and get_free_serial just fills in the
MAGIC.

There''s currently a memory leak: if create_serial is called at probe
time or calc_ports time, and then get_free_serial returns NULL because
the table has no entries left, that usb_serial struct is leaked.

get_free_serial doesn''t check properly for free slots. The middle loop
doesn''t terminate when the end of the table is reached, although the
assignment loop later does. The effect is that stuff past the end of
the table is allowed to decide if there''s free space or not, and
occasionally it''ll say "yes" and then the assignment loop will only
allocate slots up to the end of the table, preventing memory
scribbling.

I haven''t fixed any of this just yet because I''m not sure what the
intended behaviour is. Should get_free_serial allocate as many slots
as possible, or just be all or nothing? Similarly, I don''t see a
problem with calling create_serial early in usb_serial_probe, and
removing the alloc code from get_free_serial; this would fix the leak.

Ah heck, here''s a patch. This is what I think things should look like.
get_free_serial is all or none, the leak is fixed and create_serial
does all the allocation.',
	'2002-07-19',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d385083CfgrhvBo5hXAmhSSj0zrAA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.35',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.403.163.7';

insert into bug (description, revision_removed, subcategory, severity) select 
	'There''s currently a memory leak: if create_serial is called at probe
time or calc_ports time, and then get_free_serial returns NULL because
the table has no entries left, that usb_serial struct is leaked.',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.35'			  and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'resource_leak'	  	  and bug_severity.name=
	'unknown';


-- changeset '1.573.23.6',

insert into changeset (cs_number,description,created,url) values (
	'1.573.23.6',
	'[PATCH] USB: fix for ezusb firmware download

This fixes a stupid error in the timeout value when downloading firmware
to a device.  The WhiteHEAT device now works properly with this patch.',
	'2002-09-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d8f7f81W19KnGpu-llhJFa9Ce2IYg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.41',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.573.23.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.41'			  and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'device_timing'  	  and bug_severity.name=
	'unknown';


-- changeset '1.738.17.15',

insert into changeset (cs_number,description,created,url) values (
	'1.738.17.15',
	'[PATCH] USB: 2.5.42 partial fix for older pl2303

On Sat, Oct 12, 2002 at 06:16:44PM -0700, Greg KH wrote:
> Now, would you mind taking a look at 2.5, and fixing this there too? :)

Here''s a half-successful attempt. With this patch, the device no longer
appears twice, and it always works on the first open (at least, so I''ve
observed up to this point). The open following a successful open usually
fails (roughly speaking, it appears to play dead), and the open following
a failed open usually (always?) succeeds.

So, on my PL-2303, it''s not perfect but it''s certainly livable.

This patch is based on the one I did for 2.4, and in fact, this code
functions when it''s plugged into 2.4.20-pre10 instead of 2.5.42. In that
scenario, the opens work 100% of the time.

I''d be interested in suggestions or comments regarding this patch.
Anyone who has a PL-2303 working under 2.5 might want to try this patch
just to make sure it doesn''t kill their working setup.',
	'2002-10-13',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3da9db22ma1KYbCJSv9vC6TDXBfHMQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.49',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.738.17.15';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.49'			  and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'device_value'  	  and bug_severity.name=
	'unknown';


-- changeset '1.797.166.17',

insert into changeset (cs_number,description,created,url) values (
	'1.797.166.17',
	'[PATCH] usb-serial.c disconnect race

Attached is a patch that changes the 2.5.x disconnect to be similar to
2.4.x disconnect. This doesn''t fix the race, but does shrink the
window such that I''ve never seen it trigger, even under testing
designed to do that.

There doesn''t seem to be a good way to fix the race. The fix should be
to have _disconnect force any sleeping semaphore holders to run to
completion between the end of the loop in the patch below and the spot
where the underlying memory is freed, but I don''t see a way to do
that.',
	'2002-11-27',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3de50e9duQF5XcE8hvy5SAIjY3sZSQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.57',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.797.166.17';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Curiously, the patch admits not fixing the race completely',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.57'			  and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'disconnect_race'  	  and bug_severity.name=
	'unknown';


-- changeset '1.797.203.1',

insert into changeset (cs_number,description,created,url) values (
	'1.797.203.1',
	'[PATCH] module unload race with usb serial drivers

the serial subdrivers may be unloading while we open.
This patch against 2.5 guards against that.',
	'2002-11-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3de87040Ez0LupQUEJ8cRbtXzYTnjw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.58',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.797.203.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				  module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.58'			  and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'control_race'  	  and bug_severity.name=
	'unknown';


-- changeset '1.879.36.8',

insert into changeset (cs_number,description,created,url) values (
	'1.879.36.8',
	'[PATCH] USB Serial patch for old pl2303 devices.

I got a PL2303 USB serial converter a few days ago, and got your driver
up and running fairly quickly.  The problem is that I got an oops when I
rmmod-ed the drivers.  The pl2303 uses two interfaces but registers only
the second (technically wrong, I guess, but should work).  When pl2303.o
is removed, it attempts to deregister the first interface (which has no
effect), so the second interface remains registered with usbserial.  The
old struct serial still points at the removed pl2303 driver so things go
pop when anything touches it.

I think the PL2303 hack in usb_serial_probe should not change the
"interface" variable, which gets stored in serial->interface, since
usbcore will register whatever "ifnum" says.  I think that''s enough
waffle.  The patch is below.  Keep up the good work!',
	'2003-01-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e1c6c1brXFs2k-mvx7VkNSTPFEy-g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.67',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.879.36.8';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.67'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_config_protocol'  	  and bug_severity.name=
	'unknown';



-- changeset '1.879.56.2',

insert into changeset (cs_number,description,created,url) values (
	'1.879.56.2',
	'[PATCH] USB: Fix problem of sending the wrong device_id to the usb serial driver on probe()',
	'2003-01-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e1f1ad4gNiEOaXPO2p4So_f6hiS_w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.68',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.879.56.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'the bug is in the interface between generic and device-specific parts',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.68'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'  	  	and bug_severity.name=
	'unknown';



-- changeset '1.1046.513.18',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.513.18',
	'[PATCH] USB: fix a nasty use-after-free bug in the usb-serial core.',
	'2003-07-15',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f148c38bc-jQSL3qIsCBlq9dzoFEg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.84',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1046.513.18';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.84'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'use_after_free'  	and bug_severity.name=
	'unknown';

-- changeset '1.1046.580.5',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.580.5',
	'USB: fix bug if open() fails in usb-serial device.',
	'2003-07-30',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f281e79SMywQ6oaU6ivv3tBOjciNQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.86',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1046.580.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.86'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'reference_counting'	and bug_severity.name=
	'unknown';


-- changeset '1.1153.42.63',

insert into changeset (cs_number,description,created,url) values (
	'1.1153.42.63',
	'[PATCH] USB: fix oops in keyspan and whiteheat devices when plugged in.',
	'2003-08-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f4fbdefhbnEVOXuRUqzTMv2fFWFAQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.91',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1153.42.63';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-		device_unregister(&serial->port[i]->dev);
+		if (serial->port[i]->dev.parent != NULL)
+			device_unregister(&serial->port[i]->dev);',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.91'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_data_struct'	and bug_severity.name=
	'unknown';


-- changeset '1.1267.30.18',

insert into changeset (cs_number,description,created,url) values (
	'1.1267.30.18',
	'[PATCH] USB: make sure we never reference a usbserial port after it has been unregistered.',
	'2003-09-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f68f436GINko1m35kcRSL1DrJfsYg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.92',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1267.30.18';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-		if (serial->port[i]->dev.parent != NULL)
+		if (serial->port[i]->dev.parent != NULL) {
 			device_unregister(&serial->port[i]->dev);
+			serial->port[i] = NULL;
+		}',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.92'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';


-- changeset '1.1371.511.4',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.511.4',
	'USB: fix usb-serial core to look at the proper interface descriptor',
	'2004-03-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=404ccf48bJTV4rI0WIjG7MLq2ROcZg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.96',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1371.511.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.96'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_data_struct'	and bug_severity.name=
	'unknown';


-- changeset '1.1757.37.25',

insert into changeset (cs_number,description,created,url) values (
	'1.1757.37.25',
	'USB: fix bug where removing usb-serial modules or usb serial devices could oops

This fixes the issue where the Generic driver would bind to all usb-serial
devices, so the disconnect would not properly go to the real driver that
controlled the device.  This was very bad when unloading the module with
the device still connected.',
	'2004-06-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40e1e028Fr4ibKQwjS7KCB4BbRmlsA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.106',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1757.37.25';

insert into bug (description, revision_removed, subcategory, severity) select 
	'interface between the main driver and device-specific bits',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.106'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';


-- changeset '1.1832.62.36',

insert into changeset (cs_number,description,created,url) values (
	'1.1832.62.36',
	'[PATCH] USB Serial: Correct a use of out of range variable

Bug found by Coverity:
http://linuxbugs.coverity.com/external/editbugparent.php?viewbugid=2137&checkers%5B%5D=all&status%5B%5D=BUG&status%5B%5D=UNINSPECTED&status%5B%5D=UNKNOWN&status%5B%5D=DON%27T%20CARE&status%5B%5D=PENDING&product%5B%5D=all&component%5B%5D=all&file=&fn=&sortby=reverse_rank&before=&after=&curpage=2&bugid=-1&comment=&reason=',
	'2004-09-08',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=413f4e00uPzzCeih8XrI44a5VnN5qQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.109',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1832.62.36';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.109'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';


-- changeset '1.1867.4.12',

insert into changeset (cs_number,description,created,url) values (
	'1.1867.4.12',
	'[PATCH] USB: missing check in usb/serial/usb-serial.c.

 This patch add a missing check in the call to bus_register() and
not initialise `result'' (which is not necessary).',
	'2004-09-14',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41473fe5Nkh2rsXXhdjEv_3RP0cZ-g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.112',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1867.4.12';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	bus_register(&usb_serial_bus_type);
+	result = bus_register(&usb_serial_bus_type);
+	if (result) {
+		err("%s - registering bus driver failed", __FUNCTION__);
+		goto exit_bus;
+	}',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.112'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'return_check'		and bug_severity.name=
	'unknown';


-- changeset '1.1938.276.1',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.276.1',
	'USB: fix up serial object reference count bug on error path.',
	'2004-10-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4183248ctIqTuq37s7bgCVivNYbsHQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.122',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1938.276.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.122'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'reference_counting'	and bug_severity.name=
	'unknown';


-- changeset '1.1938.346.2',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.346.2',
	'[PATCH] usb-serial: free interrupt_out_buffer in cleanup routines

This patch frees interrupt_out_buffer in the cleanup routines.',
	'2004-11-12',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4195291fOIEw4ruvTe0S0PpJ816F0Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.125',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1938.346.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.125'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'resource_leak'		and bug_severity.name=
	'unknown';


-- changeset '1.1938.455.11',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.455.11',
	'[PATCH] usb-serial: add tty_hangup on disconnect

When a USB serial device is disconnected, user applications performing a
read() now receive an error code, rather than waiting indefinitely. The
included patch is originally from Al Borchers, massaged to apply to
2.6.9 and 2.6.10-rc2. I''ve tested it on 2.6.9, but not on 2.6.10-rc2.',
	'2004-12-15',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41c0d053E7JzS2T3An7yIssuCaRskw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.126',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.1938.455.11';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.126'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_disconnect_protocol' and bug_severity.name=
	'unknown';


-- changeset '1.3097.23.4',

insert into changeset (cs_number,description,created,url) values (
	'1.3097.23.4',
	'[PATCH] USB: fix usb-serial generic initialization

At module load time, if a generic device is found, the tty information
for the device is not set up properly (as the tty structures aren''t initialized
yet.)  This can cause big problems for things like udev.  This patch fixes this.',
	'2005-05-31',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=429cd399IuXGbSs7Ty3vlLtiqVphzg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.130',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.3097.23.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.130'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'os_data_struct'	and bug_severity.name=
	'unknown';


-- changeset '1.4081.52.36',

insert into changeset (cs_number,description,created,url) values (
	'1.4081.52.36',
	'[PATCH] USB: usbserial: Adds missing checks and bug fix.

Checks if ''port'' is NULL before using it in all tty operations, this
can avoid NULL pointer dereferences.',
	'2006-01-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43bc42cfHaVV8iyHnW74PIcN5yBygw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.140',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4081.52.36';

insert into bug (description, revision_removed, subcategory, severity) select 
	'internal data structures',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.140'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';


-- changeset '1.4081.52.37',

insert into changeset (cs_number,description,created,url) values (
	'1.4081.52.37',
	'[PATCH] USB: usbserial: race-condition fix.

There is a race-condition in usb-serial driver that can be triggered if
a processes does ''port->tty->driver_data = NULL'' in serial_close() while
other processes is in kernel-space about to call serial_ioctl() on the
same port.

This happens because a process can open the device while there is
another one closing it.

The patch below fixes that by adding a semaphore to ensure that no
process will open the device while another process is closing it.

Note that we can''t use spinlocks here, since serial_open() and
serial_close() can sleep.',
	'2006-01-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=43bc42d01BUcVMawPPUMw9aGZM95sg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.141',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4081.52.37';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.141'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';


-- changeset '1.4087.247.29',

insert into changeset (cs_number,description,created,url) values (
	'1.4087.247.29',
	'[PATCH] USB Serial: fix use-after-free bug in usb-serial core

This fixes a use-after-free bug in the usb-serial core.  It is simple to
trigger this (open a usb-serial port, then yank the device out before
closing the port.)  Thanks to Stefan Seyfried <seife@suse.de> for
reporting this, and to the slab debugging code which enabled it to be
tracked down.',
	'2006-03-06',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=440ca99cNYADjSmc1nCKuGMWhAfipA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.143',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usb serial'		and changeset.cs_number=  
	'1.4087.247.29';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.143'			and module_revision.module in (select id from module where name = 'usb serial')and bug_subcategory.name=
	'control_race'		and bug_severity.name=
	'unknown';



---------------------------------------------
-- usbnet 
---------------------------------------------


insert into module (name,description,path,created) values(
	'usbnet', 
	'USB Network driver infrastructure',
	'drivers/net/usb/usbnet.c',
	'2000-01-01');


-- changeset '1.43',

insert into changeset (cs_number,description,created,url) values (
	'1.43',
	'A couple of fixes listed in the file header',
	'2002-02-05',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3c60219fWb3BhhnS7niGe34Wc7kqHQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.4',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.43';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Bugfixes for uhci oops-on-unplug',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.4' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'disconnect_race'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'misc fixes; mostly, discard more TTL-mangled rx packets',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.4' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


-- changeset '1.112',

insert into changeset (cs_number,description,created,url) values (
	'1.112',
	'- Greg KH: usbnet fix
- Johannes Erdfelt: uhci.c bulk queueing fixes',
	'2002-02-05',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3c603f8aA-hxAG2JNgBVu6Y7EKCJLg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.6',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.112';

insert into bug (description, revision_removed, subcategory, severity) select 
	'bulk urbs were created without USB_ASYNC_UNLINK flag, which could cause
blocking when unlink_urb was called from atomic context',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.6' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';


-- changeset '1.151',

insert into changeset (cs_number,description,created,url) values (
	'1.151',
	'- Dave Jones: more merging, fix up last merge..
- release to sync with Dave',
	'2002-02-05',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3c6071efqIAblbgGZ1EwGGk83c3knA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.12',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.151';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	flags = in_interrupt () ? GFP_ATOMIC : GFP_KERNEL;
+	flags = in_interrupt () ? GFP_ATOMIC : GFP_NOIO; /* might be used for nfs */',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.12' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';


-- changeset '1.786.61.20',

insert into changeset (cs_number,description,created,url) values (
	'1.786.61.20',
	'USB: fix the usb net drivers due to interrupt urb no automatic resubmission change to the usb core.',
	'2002-10-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3dbd9518lPvR1r2DfwEID6ssyIY_hA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.36',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.786.61.20';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.36' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'collateral'  		   and bug_severity.name=
	'unknown';



-- changeset '1.838.46.3',

insert into changeset (cs_number,description,created,url) values (
	'1.838.46.3',
	'[PATCH] usbnet:  zaurus, oops, etc

This patch:

- Removes Pavel''s Zaurus-private crc32 code; the base patch
   was from Pavel.

- Addresses two issues Toby Milne reported against the Zaurus:
   (a) if skbs had extra framing added (z, net1080, gl620a),
       the original size (now too small) was used on tx;
   (b) added FLAG_FRAMING_Z so rx packets had enough space

- Removes an oops from the driver model conversion (saved
   the wrong pointer).  Disconnecting wasn''t healthy.

- Forward-ports some ethtool support from the 2.4 version:
   PDAs are always connected, so report them that way.

- Stubs in some PXA-250 support for non-Zaurus PDAs.
   This is currently commented out; so far those PDAs
   only run Linux for bleeding edge developers.

- Minor cleanups.',
	'2002-12-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3dfe1c0dHptLRRcQ4LSxswgc4qyaMA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.43',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.838.46.3';

insert into bug (description, revision_removed, subcategory, severity) select 
	' if skbs had extra framing added (z, net1080, gl620a),
the original size (now too small) was used on tx;',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.43' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'added FLAG_FRAMING_Z so rx packets had enough space',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.43' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'Removes an oops from the driver model conversion (saved the wrong pointer).  Disconnecting wasn''t healthy.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.43' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'collateral'   		   and bug_severity.name=
	'unknown';


-- changeset '1.971.87.7',

insert into changeset (cs_number,description,created,url) values (
	'1.971.87.7',
	'[PATCH] USB usbnet: dynamic config, cdc-ether, net1080

This patch:

   - Makes usbnet pay attention to device descriptors in
     the most common cases there''s less need to embed device
     hardware details.  This lets it work with high speed
     devices, and should help interop with the newer ARM/PXA
     kernels usb-eth (same vid/pid, but different endpoints).

   - Adds some new CDC Ethernet support, which is partly enabled:
     the Zaurus SL-5500 code uses it, with the current FRAMING_Z
     flag overriding normal CDC framing on-the-wire.  (Most of
     the other minidrivers use CDC framing, except Net1080
     and GeneSys.)

   - Merges a patch from Johannes to recover from some net1080
     framing errors by flushing the fifos ... the chip gets into
     a wierd mode, this makes the link more robust.  (Thanks!)

   - Gets rid of a family of cpu/logfile saturating loops that
     could show up in early stages of disconnect processing,
     while we''re getting rx/tx errors continuously since khubd
     hasn''t tried to disconnect() us yet.  (Pathological case,
     with lots of logging enabled: khubd never gets scheduled!)

   - Uses deeper queues at high speed, so the host controllers
     can stay busy transferring packets even when IRQs get held
     off for several milliseconds.  That pipelining gives better
     throughput too - 4x more with one device, says TTCP.

With the possible exception of multicast support, this code
should be a fine replacement to cdc-ether ... certainly
its faster for high speed devices.  Some later patch should
likely do a switch-over.  (Which would  also resolve a Zaurus
hotplugging bug:  cdc-ether doesn''t blacklist it.)',
	'2003-04-04',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e8e1fefN4i6nqQ7-HL92XsM1kQGkQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.48',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.971.87.7';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Merges a patch from Johannes to recover from some net1080
framing errors by flushing the fifos ... the chip gets into
a wierd mode, this makes the link more robust.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.48' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';



-- changeset '1.1046.480.7',

insert into changeset (cs_number,description,created,url) values (
	'1.1046.480.7',
	'[PATCH] USB: usbnet, don''t NET_XMIT_DROP

Martin Diehl pointed out that the network layer treats
a tx drop notification in special ways, so usbnet
shouldn''t be using them.  This fixes that problem,
which could potentially explain some infrequent oopsing
seen in the network layer.  (With memory so tight that
an URB can''t be gotten.)',
	'2003-07-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f0ce303He7Opacc2uXzWsReX_mpcQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.58',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1046.480.7';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.58' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'error_reporting'   	   and bug_severity.name=
	'unknown';


-- changeset '1.1119.2.11',

insert into changeset (cs_number,description,created,url) values (
	'1.1119.2.11',
	'[PATCH] USB: usbnet, prevent exotic rtnl deadlock

Turns out that when PM is in use, some D3cold resume paths
could have one thread, holding the network lock, deadlock
in flush_scheduled_work() since an event task is waiting
for that same lock.  Fix is to call that later, when the
lock isn''t held.',
	'2003-08-06',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f3188e96ZvihFbrvj03hV5cyGNBDQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.65',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1119.2.11';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.65' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'disconnect_race'   	   and bug_severity.name=
	'unknown';


-- changeset '1.1123.22.5',

insert into changeset (cs_number,description,created,url) values (
	'1.1123.22.5',
	'[PATCH] USB: error check for claiming second interface in usbnet
- fail if second interface is already claimed',
	'2003-08-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f3814cb4HHnBI71GtV27PSYWrMQNA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.66',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1123.22.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.66' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'return_check'   	   and bug_severity.name=
	'unknown';


-- changeset '1.1123.22.6',

insert into changeset (cs_number,description,created,url) values (
	'1.1123.22.6',
	'[PATCH] USB: genelink_tx_fixup fails to check for memory allocation failure 
this can oops.
 - check for mem alloc failure',
	'2003-08-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f3814d8nTfBsxzHIKbDTv6EcLjBIA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.67',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1123.22.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.67' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'return_check'   	   and bug_severity.name=
	'unknown';


-- changeset '1.1123.22.22',

insert into changeset (cs_number,description,created,url) values (
	'1.1123.22.22',
	'[PATCH] USB: usbnet.c - trailing ''else'' that probably breaks net1080

A trailing else in the #ifdef CONFIG_USB_NET1080 block that would
prevent it from properly filling the bulk URB.',
	'2003-08-15',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f3d12e9SBIopejDx24ynC4yQfKJWA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.68',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1123.22.22';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.68' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'typo'   	   	   and bug_severity.name=
	'unknown';



-- changeset '1.1153.42.53',

insert into changeset (cs_number,description,created,url) values (
	'1.1153.42.53',
	'[PATCH] USB: usbnet, cdc ethernet descriptor parsing fixes

This makes the new CDC Ethernet code handle more devices:

  - Uses the active config, not just the default one, if it''s
    coping descriptors ''in wrong place quirk''.  (bugfix)

  - Uses usb_ifnum_to_if() to get interfaces.  (bugfix)

  - AMBIT USB cable modems have bogus CDC Union descriptors,
    workaround by switching master and slave.  (add quirk)

  - To make it easier the next time we run into firmware
    that violates the class spec, add debug messages saying
    exactly why it''s giving up on a given CDC device.

Net result, this code now handles at least one more
cable modem design.',
	'2003-08-27',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f4d4d7eyV4Cq30yDJpWlmQIRH8fWQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.72',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1153.42.53';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Uses the active config, not just the default one, if it''s
coping descriptors ''in wrong place quirk''.  (bugfix)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.72' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'os_data_struct'  	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Uses usb_ifnum_to_if() to get interfaces.  (bugfix)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.72' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'AMBIT USB cable modems have bogus CDC Union descriptors,
workaround by switching master and slave.  (add quirk)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.72' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';



-- changeset '1.1267.30.11',

insert into changeset (cs_number,description,created,url) values (
	'1.1267.30.11',
	'[PATCH] USB: Use num_altsetting in usbnet and usbtest

This patch fixes usbnet and usbtest.  When checking endpoints, they
iterate through all interfaces out to intf->max_altsetting, but they
should only go up to intf->num_altsetting.  The max_altsetting value
refers to the amount of space _allocated_ for usb_host_interface
structures, not the number that are really in use.',
	'2003-09-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f6775c16Z6Oyox3cc-fibeuBzC39w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.74',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1267.30.11';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.74' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'os_data_struct'  	   and bug_severity.name=
	'unknown';


-- changeset '1.1267.30.30',

insert into changeset (cs_number,description,created,url) values (
	'1.1267.30.30',
	'[PATCH] USB: ethtool_ops and ax8817x fixes for usbnet

This is a resend of all of my recent usbnet additions which
center on adding ethtool_ops support.  All have been re-diffed
against the latest usbnet from BK.

Changes:
* Add ethtool_ops support to usbnet
* Add GPIO bit toggling upon initialization of ax8817x devices
  so that xmit works properly
* Add Wake-On-Lan and eeprom dump support for ax8817x devices
  (this only adds the config support for WOL, actual operation
   is more of a USB/Power Management function)
* Change ax8817x_set_multicast to not kmalloc for the filter
  since there was no clean way to free the memory after URB submit.
  Instead, we make use of the 20 byte dev->data.',
	'2003-09-19',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f6b911fLBYY00JFVegeBLCSF2_YTg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.75',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1267.30.30';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Add GPIO bit toggling upon initialization of ax8817x devices so that xmit works properly',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.75' 	   	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_ordering'  	   and bug_severity.name=
	'unknown';



-- changeset '1.5979',

insert into changeset (cs_number,description,created,url) values (
	'1.5979',
	'USB: address FIXME in usbnet w.r.t drivers claiming multiple interfaces

This fixes the issue of drivers claiming multiple interfaces. Operations
are stopped as soon as an interface is suspend and resumed only as
all interfaces have been resumed.',
	'2007-05-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4653e322qbjTftuwXSER4448pwxLWw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.167',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.5979';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.167' 	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'os_pm_protocol'  	   and bug_severity.name=
	'unknown';


-- changeset '1.5394.4.34',

insert into changeset (cs_number,description,created,url) values (
	'1.5394.4.34',
	'usbnet: init fault (oops) cleanup, whitespace fixes

This cleans up some error handling paths in usbnet device probing,
one of them could cause oopsing, e.g. with some RNDIS devices.

It also removes some extraneous whitespace.',
	'2007-02-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45d63f22bspJGxgOK0Qxv0_f8ILmuw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.163',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.5394.4.34';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The error is in the interface between the generic usbnet driver
and device-specific bits.  Since it does not have anything to do with device or
OS interface, marking it as a generic program logic error',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.163' 	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'logic'  	   	   and bug_severity.name=
	'unknown';


-- changeset '1.4900.33.13',

insert into changeset (cs_number,description,created,url) values (
	'1.4900.33.13',
	'USB: use MII hooks only if CONFIG_MII is enabled

Fix mcs7830 patch

The recent mcs7830 update to make the MII support sharable goofed various
pre-existing configurations in two ways:

  - it made the usbnet infrastructure reference MII symbols even
    when they''re not needed in the kernel being built

  - it didn''t enable MII along with the mcs7830 minidriver

This patch fixes these two problems.

However, there does seem to be a Kconfig reverse dependency bug in that MII
gets wrongly enabled in some cases (like USBNET=y and USBNET_MII=n), I think
I''ve noticed that same problem in other situations too.  So the result can
mean kernels being bloated by stuff that''s needlessly enabled ... better
than wrongly being disabled, but contributing to bloat.',
	'2006-11-03',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=454b9f29OTyclXH8KCQC15M1o6K5yg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.158',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.4900.33.13';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The error is in the interface between the generic usbnet driver
and device-specific bits.  Since it does not have anything to do with device or
OS interface, marking it as a generic program logic error',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.158' 	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'logic'  	   	   and bug_severity.name=
	'unknown';



-- changeset '1.4867.51.41',

insert into changeset (cs_number,description,created,url) values (
	'1.4867.51.41',
	'usbnet: add a mutex around phy register access

When working on the mcs7830, I noticed the need for a mutex in its
mdio_read/mdio_write functions. A related problem seems to be present
in the asix driver in the respective functions.

This introduces a mutex in the common usbnet driver and uses it
from the two hardware specific drivers.',
	'2006-10-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45354f4f3_XpiGmspm_krrvxSH1nGA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.156',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.4867.51.41';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.156' 	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'control_race' 	   	   and bug_severity.name=
	'unknown';


-- changeset '1.3332.194.23',

insert into changeset (cs_number,description,created,url) values (
	'1.3332.194.23',
	'[PATCH] usbnet oops fix

There''s a ''return the wrong SKB'' error in the GL620A cable minidriver
(for ''usbnet'') which can oops.  This would not appear when talking
Linux-to-Linux, only Linux-to-Windows (for recent Linuxes).',
	'2005-08-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=430be29ayUK6zAlp3EXFAj93Nv1_3g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.133',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.3332.194.23';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.133' 	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'logic' 	   	   and bug_severity.name=
	'unknown';


-- changeset '1.2530.1.13',

insert into changeset (cs_number,description,created,url) values (
	'1.2530.1.13',
	'[PATCH] USB: better usbnet zaurus/mdlm/... fix

This is a somewhat more comprehensive fix for the problem of devices
like the newer Zaurii ... or in this case some Motorola cell phones.

To recap, the problem''s root cause is that these devices aren''t using
standard USB class specifications for their network links, and so far
we''ve had to add lots of device-specific driver entries.  The vendor
fix abuses the CDC MDLM descriptors (they _could_ have conformed to
the spec, but didn''t) and defines a ''Belcarra firmware'' pseudo-class.

This patch recognizes that pseudo-class by the GUIDs in those descriptors,
and handles the devices that just use the Zaurus framing.',
	'2005-04-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4269758eYTycZDW8OjzBcO1vh_WQoQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.126',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.2530.1.13';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.126' 	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'	   	   and bug_severity.name=
	'unknown';



-- changeset '1.2442.1.27',

insert into changeset (cs_number,description,created,url) values (
	'1.2442.1.27',
	'[PATCH] USB: usbnet and zaurus zl-5600

Hmm, another case of a Zaurus ROM not telling the expected conformance lie,
this patch handles the lies told by the SL5600.',
	'2005-04-18',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=42645347m_7-JO2UvT2x816VuOGGVA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.124',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.2442.1.27';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.124' 	   	   and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'	   	   and bug_severity.name=
	'unknown';



-- changeset '1.2181.4.71',

insert into changeset (cs_number,description,created,url) values (
	'1.2181.4.71',
	'[PATCH] USB: usbnet minor bugfixes

Two bugfixes to usbnet.  The important one is that it''s OK when the
minidriver doesn''t want to use a status endpoint.  That shouldn''t
be treated as an error ... errors prevent probe() from succeeding!

The minor fix is that software driven interface shutdown should neither
be accounted as an error, nor generate a diagnostic.',
	'2005-03-24',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=42434dc1R14f9BiHgEBiQOo-Tyxt9w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.120',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.2181.4.71';

insert into bug (description, revision_removed, subcategory, severity) select 
	'It''s OK when the minidriver doesn''t want to use a status endpoint.  That shouldn''t
be treated as an error (The error is in the interface between the generic usbnet driver
and device-specific bits.  Since it does not have anything to do with device or
OS interface, marking it as a generic program logic error)',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.120' 	   	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'logic'	   	   	and bug_severity.name=
	'unknown';



-- changeset '1.2181.4.14',

insert into changeset (cs_number,description,created,url) values (
	'1.2181.4.14',
	'[PATCH] USB: usbnet fix for Zaurus C-860

This patch resolves a recent problem with the Zaurus C-860 support.

A change to correct handling of Zaurii that are lying about their support
for the "CDC Ethernet" class specification broke the C-860, which tells
an entirely different lie (that it supports "CDC MDLM", providing access
to a cell phone modem).  The code expecting it to be telling a lie about
CDC Ethernet support naturally misbehaved.  (Sharp should straighten out
its story.  The 2.6 OpenZaurus kernels don''t have any such issues...)

The fix is just to recognize this bogus MDLM stuff and ignore it.
This patch also includes the two MDLM descriptors in <linux/usb_cdc.h>
although they''re not currently used.',
	'2005-03-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=423a3035Tkc79QReMnWw9RDipwLfiA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.119',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.2181.4.14';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.119' 	   	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'  	and bug_severity.name=
	'unknown';



-- changeset '1.1994.7.31',

insert into changeset (cs_number,description,created,url) values (
	'1.1994.7.31',
	'[PATCH] USB: Add ASIX AX88772 10/100 Ethernet support to usbnet

* Add support for the ASIX AX88772 10/100 Ethernet chip
* Fix ax8817x_bind to use allocated buffer to avoid DMA on the stack
* Fix ax8817x_bind error handling to ensure all resources are freed on failure
* Fix ax8817x_get_eeprom routines to return valid eeprom data',
	'2005-03-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=422d4b8bwjitxkcSYZUceJG2ONVR7Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.113',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1994.7.31';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Fix ax8817x_bind to use allocated buffer to avoid DMA on the stack',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.113' 	   	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'wrong_argument'  	and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'Fix ax8817x_bind error handling to ensure all resources are freed on failure',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.113' 	   	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'resource_leak'  	and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Fix ax8817x_get_eeprom routines to return valid eeprom data',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.113' 	   	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'  	and bug_severity.name=
	'unknown';


-- changeset '1.1938.401.10',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.401.10',
	'[PATCH] USB: ax8817x/usbnet, no GFP_KERNEL blocking in_irq

This removes some nasty might_sleep() warnings.
Worth fixing before 2.6.10-final IMO; we don''t
actually _know_ of oopses this bug caused ...

The AX8817x link detection code was calling usb_submit_urb() with
GFP_KERNEL in a completion handler (IRQ handler).  That''s a no-no.',
	'2004-11-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41a515f202LhQVmJoySGfA34TB5BVA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.108',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1938.401.10';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.108' 	   	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'block_in_atomic_context' 	and bug_severity.name=
	'unknown';


-- changeset '1.1938.113.20',

insert into changeset (cs_number,description,created,url) values (
	'1.1938.113.20',
	'[PATCH] USB: remove calls to usb_unlink_urb in net/usbnet.c',
	'2004-10-18',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41743ad1uO7T6hCcCeG21UWvTMMwEg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.104',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1938.113.20';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Calling usb_unlink_urb and deallocating the urb immediately, without waiting for completion',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.104' 	   	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'bus_data_protocol' 	and bug_severity.name=
	'unknown';


-- changeset '1.1757.90.17',

insert into changeset (cs_number,description,created,url) values (
	'1.1757.90.17',
	'[PATCH] USB: ax8817x_unbind does not free the interrupt URB after unlinking

ax8817x_unbind does not free the interrupt URB after unlinking.

Noticed that the net->status already has a flag for link so my
private structure variable for link was redundant.  Worked around
this and was able to kill off the unique ax8817x_get_link() function
in the process.',
	'2004-07-14',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40f5aacfBNQYdweQv6xnBA6QYQZIQA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.96.1.3',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1757.90.17';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.96.1.3' 	   	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'resource_leak'		and bug_severity.name=
	'unknown';


-- changeset '1.1722.123.45',

insert into changeset (cs_number,description,created,url) values (
	'1.1722.123.45',
	'[PATCH] USB: usbnet shouldn''t oops on cdc error path

If the string descriptor fetch works, no problem.  Else
make the disconnect() logic - newly called because of
''recent'' changes to interface claiming logic - use its
fast cleanup path, since we''re still not fully set up yet.

Should resolve OSDL bugid 2714; please merge.',
	'2004-06-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40ca48d2bZBFr70Yy-7NOojx279byw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.95',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1722.123.45';

insert into bug (description, revision_removed, subcategory, severity) select 
	'If probe fails, the kernel can invoke disconnect.  To handle this correctly,
usb_set_intfdata(info->data, NULL) is used to prevent disconnect from deallocating 
things that have not been allocated yet',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.95' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'bus_disconnect_protocol'and bug_severity.name=
	'unknown';


-- changeset '1.1371.763.10',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.763.10',
	'[PATCH] USB: usbnet and pl2301/2302 reset

Handle some PL-2301/2302 devices better.',
	'2004-04-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=408830e65cl6bROnmrdOCU4XWT8aDQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.88',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1371.763.10';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	return pl_set_QuickLink_features (dev,
+	/* some units seem to need this reset, others reject it utterly.
+	 * FIXME be more like "naplink" or windows drivers.
+	 */
+	(void) pl_set_QuickLink_features (dev,
 		PL_S_EN|PL_RESET_OUT|PL_RESET_IN|PL_PEER_E);
+	return 0;',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.88' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.1371.639.32',

insert into changeset (cs_number,description,created,url) values (
	'1.1371.639.32',
	'[PATCH] USB: usbnet, minor probe() fault fix

Some hardware had the poor taste to misbehave during probe(),
which turned up a minor bug.  This fixes it:  don''t try to
free a network device that hasn''t been registerd.',
	'2004-03-26',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=406493298yMieNBam-GZYgSU2_ko8Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.86',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1371.639.32';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.86' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'free_unallocated'	and bug_severity.name=
	'unknown';


-- changeset '1.1337.41.3',

insert into changeset (cs_number,description,created,url) values (
	'1.1337.41.3',
	'[PATCH] USB: ax8817x fixes in usbnet.c

* ax8817x_set_multicast - use address of dev->data, not contents
* ax8817x_write_async_cmd - free request and urb if submit fails',
	'2003-10-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3f986b47PQwk9BLVGrLwJm1HOqDnoQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.76',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.1337.41.3';

insert into bug (description, revision_removed, subcategory, severity) select 
	'ax8817x_set_multicast - use address of dev->data, not contents
-		u8 *multi_filter = (u8 *)dev->data;
+		u8 *multi_filter = (u8 *)&dev->data;',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.76' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'logic'			and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'ax8817x_write_async_cmd - free request and urb if submit fails',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.76' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'resource_leak'		and bug_severity.name=
	'unknown';

--
-- asix-specific
--

-- changeset '1.7927',

insert into changeset (cs_number,description,created,url) values (
	'1.7927',
	'asix fixes

* usb_control_message() to/from stack (breaks on e.g. arm); some
  places did kmalloc() for buffer, some just worked from stack.
  Added kmalloc()/memcpy()/kfree() in asix_read_cmd()/asix_write_cmd(),
  removed that crap from callers.
* Fixed a leak in ax88172_bind() - on success it forgot to kfree() the
  buffer.
* Endianness bug in ax88178_bind() - we read a word from eeprom and work with
  it without converting to host-endian',
	'2007-12-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=476ddba8To70cgI32W7E1UK28EvnCg');

insert into module_revision (rev_number, description, module, changeset) select 
	'asix1.27',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.7927';

insert into bug (description, revision_removed, subcategory, severity) select 
	'usb_control_message() to/from stack (breaks on e.g. arm); some
  places did kmalloc() for buffer, some just worked from stack.
  Added kmalloc()/memcpy()/kfree() in asix_read_cmd()/asix_write_cmd(),
  removed that crap from callers.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'asix1.27' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'wrong_argument'	and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Fixed a leak in ax88172_bind() - on success it forgot to kfree() the buffer',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'asix1.27' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'resource_leak'		and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Endianness bug in ax88178_bind() - we read a word from eeprom and work with
it without converting to host-endian',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'asix1.27' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';



-- changeset '1.5394.4.15',

insert into changeset (cs_number,description,created,url) values (
	'1.5394.4.15',
	'USB: asix - Fix endian issues in asix_tx_fixup()

The attached patch fixes endian issues in asix_tx_fixup() that prevented
AX88772 and AX88178 devices from working on big-endian systems.  With
the attached patch, all three chips are reported to work on big endian.',
	'2007-02-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45d63f0flbCx8U1wzRrRwcpISwQdhw');

insert into module_revision (rev_number, description, module, changeset) select 
	'asix1.20',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.5394.4.15';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'asix1.20' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_value'		and bug_severity.name=
	'unknown';


-- changeset '1.5256.1.2',

insert into changeset (cs_number,description,created,url) values (
	'1.5256.1.2',
	'USB: asix: Detect internal PHY and enable/use accordingly

Different AX88772 dongles use different PHYs; the chip is capable of using
both a primary and secondary PHY, and supports an internal and external PHY.

It appears that some DUB-E100 devices use the internal PHY, so trying to use
an external one will not work (note that this is different across revisions,
as well; the "A" and "B" revs of the DUB-E100 use different PHYs!).  The data
sheet for the AX88772 chip specifies that the internal PHY id will be 0x10,
so if that''s read from the EEPROM, we should use that rather than attempting
to use an external PHY.',
	'2007-01-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45b514b0nRf1_kz-rUag_E8LnUeoQg');

insert into module_revision (rev_number, description, module, changeset) select 
	'asix1.17',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.5256.1.2';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'asix1.17' 	   		and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.5225.1.19',

insert into changeset (cs_number,description,created,url) values (
	'1.5225.1.19',
	'USB: asix: Fix AX88772 device PHY selection

A small typo in ax88772_bind() prevents the device from selecting the
proper PHY, leaving the device useless.  The attached patch fixes this.
If this patch can be added to the 2.6.19.x series as well, that would be
helpful for end-users.',
	'2007-01-05',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=459eb2c7t5hhOLnNZEX7js1fDgdSoA');

insert into module_revision (rev_number, description, module, changeset) select 
	'asix1.16',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'usbnet'		and changeset.cs_number=  
	'1.5225.1.19';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Despite what the comment says, it looks more like a device protocol bug:
 	if ((ret = asix_write_cmd(dev, AX_CMD_SW_PHY_SELECT,
-				0x0000, 0, 0, buf)) < 0) {
+				1, 0, 0, buf)) < 0) {',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'asix1.16' 	 	and module_revision.module in (select id from module where name = 'usbnet')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';



------------------------------------------------------
-- CATC EL1210A NetMate USB Ethernet driver
------------------------------------------------------


insert into module (name,description,path,created) values(
	'catc', 
	'CATC EL1210A NetMate USB Ethernet driver',
	'drivers/net/usb/catc.c',
	'2001-01-01');

-- changeset '1.5791.73.46'

insert into changeset (cs_number,description,created,url) values (
	'1.5791.73.46',
	'USB: fix catc error handling

this driver ignores errors while starting the transmit queue. It will
never be reported stopped as the completion handler won''t run
and it will never be started again as it will be considered started.
This patch adds error handling.',
	'2007-04-27',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=46325d1d10OXf-O1wNyvFvvzX-Co4w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.49',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'catc'		and changeset.cs_number=  
	'1.5791.73.46';

insert into bug (description, revision_removed, subcategory, severity) select 
	'The driver never restarts the tx queue after an error',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.49' 	 		and module_revision.module in (select id from module where name = 'catc')and bug_subcategory.name=
	'device_ordering'	and bug_severity.name=
	'unknown';


-- changeset '1.373.8.5'

insert into changeset (cs_number,description,created,url) values (
	'1.373.8.5',
	'[PATCH] catc: GFP_KERNEL in interrupt or with spinlocks held
bad usage of mem_flags in catc.',
	'2002-04-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3cb48682F14HQWRfTcS87k2YuF5lXQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.14',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'catc'		and changeset.cs_number=  
	'1.373.8.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.14' 	 		and module_revision.module in (select id from module where name = 'catc')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'unknown';


------------------------------------------------------
-- Driver for KL5KUSB101 based USB->Ethernet
------------------------------------------------------

insert into module (name,description,path,created) values(
	'kaweth', 
	'Driver for KL5KUSB101 based USB->Ethernet',
	'drivers/net/usb/kaweth.c',
	'2000-01-01');

-- changeset '1.3891.12.19'

insert into changeset (cs_number,description,created,url) values (
	'1.3891.12.19',
	'[PATCH] USB: fix race in kaweth disconnect

this patch from Herbert Xu fixes a race by moving termination of
the URBs into close() exclusively.',
	'2005-11-17',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=437cda3fL6HpvXV9pVfTIgrquP0kZQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.79',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.3891.12.19';

insert into bug (description, revision_removed, subcategory, severity) select 
	'If disconnect happenned concurrently with start_xmit, both functions
could free the same urb',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.79' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'disconnect_race' 	and bug_severity.name=
	'unknown';


-- changeset '1.3332.522.72'

insert into changeset (cs_number,description,created,url) values (
	'1.3332.522.72',
	'[PATCH] USB: Kaweth.c udelay patch

On some arch (like arm) udelay cannot be called with value greater that
2000.',
	'2005-10-28',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4362b8e0QrLvPpgwYOhjFyClHBjXPg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.78',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.3891.12.19';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	udelay(10000);
+	mdelay(10);',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.78' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'wrong_argument' 	and bug_severity.name=
	'unknown';


-- changeset '1.3192.8.4'

insert into changeset (cs_number,description,created,url) values (
	'1.3192.8.4',
	'[PATCH] fix for kaweth broken by changes in the networking layer',
	'2005-06-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=42b1a2b64MBg44p51XR0AylHjvP3Eg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.74',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.3192.8.4';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-		if (!act_state)
+		if (act_state)
 			netif_carrier_on(kaweth->net);
 		else
 			netif_carrier_off(kaweth->net);',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.74' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'collateral'	 	and bug_severity.name=
	'unknown';


-- changeset '1.1938.189.23'

insert into changeset (cs_number,description,created,url) values (
	'1.1938.189.23',
	'[PATCH] fix for kaweth broken by changes in the networking layer',
	'2004-10-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4176f6caJQAd4YZ9pErtYB9HPTlnIg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.67',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.1938.189.23';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	kaweth = netdev_priv(dev);
+	kaweth = netdev_priv(netdev);',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.67' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'typo'	 		and bug_severity.name=
	'unknown';


-- changeset '1.1757.37.13'

insert into changeset (cs_number,description,created,url) values (
	'1.1757.37.13',
	'[PATCH] USB: kaweth not handling ESHUTDOWN

kaweth fails to cleanly shut down operations upon shutdown of the
controller.',
	'2004-06-24',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40db173dpekEoRT3XqmnUf-xeARGEg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.58',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.1757.37.13';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Disconnect control logic in this driver seems totally broken.  Disconnect
first cancels rx urb (synchronously), then tx urb (asynchronously) and if the 
latted fails with -EINPROGRESS code, waits for a completion notification, which
is actually sent by the _rx_ callback, not tx callback.  The bug is that 
if rx completion is triggered with -ESHUTDOWN completion code, it does forgot
to send the completion notification.  The fix is to send the notification from
the rx callback, which still looks wrong to me.',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.58' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'disconnect_race'	and bug_severity.name=
	'unknown';


-- changeset '1.1722.18.5'

insert into changeset (cs_number,description,created,url) values (
	'1.1722.18.5',
	'[PATCH] USB: fix fix to kaweth.c

the previous patch was buggy. The state must be set _before_ the
condition is checked, or there''s a window missing a wakeup.
This incremental change set fixes that.

  - fix race condition with current->state',
	'2004-05-24',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40b28d68zK3bDMX_HYHB71FmSMrwJQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.56',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.1722.18.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.56' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'data_race'		and bug_severity.name=
	'unknown';


-- changeset '1.889.105.9'

insert into changeset (cs_number,description,created,url) values (
	'1.889.105.9',
	'[PATCH] USB: kaweth fix

this is the length calculation fix against 2.5.
  - fix DHCP problem with correct length calculation',
	'2003-02-07',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e4350beNr_WvEXlAyBIXjLZzV78Rg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.38',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.889.105.9';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.38' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'device_value' and bug_severity.name=
	'unknown';


-- changeset '1.879.14.5'

insert into changeset (cs_number,description,created,url) values (
	'1.879.14.5',
	'[PATCH] USB: kaweth freeing skbs

this is the 2.5 version of the 2.4 fix
  - proper freeing of skbs',
	'2003-01-06',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3e1a18caZuSMbNzwtcnzeIf5Ae9x7A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.36',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.879.14.5';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	dev_kfree_skb(skb);
+	dev_kfree_skb_irq(skb);',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.36' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'unknown';


-- changeset '1.497.103.6'

insert into changeset (cs_number,description,created,url) values (
	'1.497.103.6',
	'[PATCH] open/close fix for kaweth

this handles the error case.',
	'2002-09-11',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d7fe032OfsYKxV2deytvoRSv6s04w');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.25',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.497.103.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.25' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'return_check' 		and bug_severity.name=
	'unknown';


-- changeset '1.373.8.4'

insert into changeset (cs_number,description,created,url) values (
	'1.373.8.4',
	'[PATCH] USB kaweth: fix races between timeout, xmit and disconnect

USB kaweth driver update

 - always set the ASYNC_UNLINK flag for the tx urb
 - correct cancelling the tx urb under all circumstances
 - detect a running disconnect method in xmit path
 - fix a potential deadlock by using GFP_NOIO',
	'2002-04-10',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3cb48034iLQO4HsxHYTZPHzyJ4tC7g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.17',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.373.8.4';


insert into bug (description, revision_removed, subcategory, severity) select 
	'- always set the ASYNC_UNLINK flag for the tx urb
The problem seems to be that tx_timeout is called from interrupt
context and so it is illegal to perform synchronous unlink, so we need 
asynchronous unlink',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.17' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'- correct cancelling the tx urb under all circumstances
- detect a running disconnect method in xmit path
Not sure about the exact bug.
a state variable is introduces to signal disconnect to start_xmit, to
prevent race',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.17' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'disconnect_race' and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'fix a potential deadlock by using GFP_NOIO',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.17' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'unknown';



-- changeset '1.200'

insert into changeset (cs_number,description,created,url) values (
	'1.200',
	'[PATCH] USB Kaweth driver update

Here''s a patch against 2.5.3 for the USB kaweth driver that does the
following:
        - removes SMP deadlock
        - removes nfs deadlock
        - fixes a memory leak when the firmware is not loaded.
        - few other minor cleanups.
This patch was done by Oliver Neukum.',
	'2002-02-05',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3c6084116PadVI7x7xCAvyA5lmsf6Q');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.11',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'kaweth'		and changeset.cs_number=  
	'1.200';


insert into bug (description, revision_removed, subcategory, severity) select 
	'-	dr = kmalloc(sizeof(struct usb_ctrlrequest), 
-                     in_interrupt() ? GFP_ATOMIC : GFP_KERNEL);
+	dr = kmalloc(sizeof(struct usb_ctrlrequest), GFP_ATOMIC);',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.11' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'block_in_atomic_context' and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'fixes a memory leak when the firmware is not loaded',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.11' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'resource_leak' 	and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'the driver did not unlink urb''s on disconnect',
				module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.11' 	 		and module_revision.module in (select id from module where name = 'kaweth')and bug_subcategory.name=
	'bus_disconnect_protocol' 		and bug_severity.name=
	'unknown';



------------------------------------------------------
-- Driver for C-Media CMI8338 and 8738 PCI soundcards
------------------------------------------------------


insert into module (name,description,path,created) values(
	'cmipci', 
	'Driver for C-Media CMI8338 and 8738 PCI soundcards',
	'sound/pci/cmipci.c',
	'2000-01-01');


-- changeset '1.373.169.1'

insert into changeset (cs_number,description,created,url) values (
	'1.373.169.1',
	'- sound/{core,pci}/*.c
- fix copy_{to,from}_user error handling (thanks to Rusty for pointing this out)',
	'2002-05-19',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3ce82bebQb9adGSfdsyuZodRbvkTjA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.6',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.373.169.1';

insert into bug (description, revision_removed, subcategory, severity) select 
	'-	if (! cm->channel[CM_CH_PLAY].ac3_shift)
-		return copy_from_user(runtime->dma_area + frames_to_bytes(runtime, pos),
-				      src, frames_to_bytes(runtime, count));
+	if (!cm->channel[CM_CH_PLAY].ac3_shift) {
+		if (copy_from_user(runtime->dma_area +
+				   frames_to_bytes(runtime, pos), src,
+				   frames_to_bytes(runtime, count)))
+			return -EFAULT;
+		return 0;
+	}',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.6' 	   	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'error_reporting'   	   and bug_severity.name=
	'unknown';



-- changeset '1.573.98.21'

insert into changeset (cs_number,description,created,url) values (
	'1.573.98.21',
	'[PATCH] ALSA update [12/12] - 2002/10/01

  - deleted sound/pci/ice1712.c
  - fixed Makefile to point to sound/pci/ice1712 directory
  - added Ensoniq SoundScape header file and HWDEP IFACE
  - CS4231
    - added CS4231_HW_AD1845 and register definitions for AD1845
  - USB
    - added snd-rawmidi.o to the snd-usb-audio''s dependency if sequencer is
  - PCI DMA allocation
    - fixed the wrapper again
  - AC''97 codec
    - added HSD11246 identification (Conexant), a bit improved proc contents
  - CMIPCI
    - changed the DMA configuration for period size
    - fixed compile with SOFT_AC3 option
    - added PCM_INFO_PAUSE to hw settings.  now pause should work properly
    - corrected the modem on/off bit (FLINK)
  - ICE1712
    - compilation fixes
  - intel8x0
    - use mmio for codec on nforce (pci resource 2)
    - clean up and fix ali5455 codes
  - RME32
    - enable 88.2/96kHz on capture with CS8414
  - VIA82xx
    - fixed the size of allocated bd array to be released
    - fixed the allocation size of idx_table
  - PPC Awacs
    - replaced one more mdelay() with schedule
    - try to touch mic boost on screamer at init
  - USB Audio
    - reset each interface at initialization
    - reset the old interface if a new interface is chosen
    - don''t claim the interface which already claimed',
	'2002-10-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3d9a4cc2RFwnSK4LGookDZfUp_6uUA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.14',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.573.98.21';

insert into bug (description, revision_removed, subcategory, severity) select 
	'added PCM_INFO_PAUSE to hw settings.  now pause should work properly',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.14' 	   	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'corrected the modem on/off bit (FLINK)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.14' 	   	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.1371.65.61'

insert into changeset (cs_number,description,created,url) values (
	'1.1371.65.61',
	'ALSA CVS update - Takashi Iwai <tiwai@suse.de>
CMIPCI driver
- set XCHGDAC bit implicitly on MC4/6 models for fixing wrong
  playback on some boards.
- removed ''Exchange DAC'' control from such a model.',
	'2003-11-26',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=3fc52801i4GqoRNseoLe1qC-cTzKLQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.31',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.1371.65.61';

insert into bug (description, revision_removed, subcategory, severity) select 
	'added PCM_INFO_PAUSE to hw settings.  now pause should work properly',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.31' 	   	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


-- changeset '1.1371.208.24'

insert into changeset (cs_number,description,created,url) values (
	'1.1371.208.24',
	'ALSA CVS update - Takashi Iwai <tiwai@suse.de>
CMIPCI driver
- initialize the iec958 status at opening the normal PCM device, too.',
	'2004-01-20',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=400d4ca1l-h7mu5Fn4xC1oDGoNdcqg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.35',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.1371.208.24';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.35' 	   	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.1757.32.33'

insert into changeset (cs_number,description,created,url) values (
	'1.1757.32.33',
	'ALSA CVS update
CMIPCI driver
don''t sleep in prepare callback',
	'2004-06-23',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40d96709V4AYZRvjF5y-8mZmAMwTtg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.45',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.1757.32.33';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.45' 	   	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';


-- changeset '1.1757.70.13'

insert into changeset (cs_number,description,created,url) values (
	'1.1757.70.13',
	'ALSA CVS update
CMIPCI driver
Fix the i/o port range of gameport on cmipci

Gameport use only 1 I/O port not 8.
Attached patch fix gameport on CMIPCI soundcards.',
	'2004-07-01',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=40e44b18aVN09-irQXNbQ0ftyTKufA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.47',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.1757.70.13';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.47' 	   	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';


-- changeset '1.1938.334.60'

insert into changeset (cs_number,description,created,url) values (
	'1.1938.334.60',
	'pci_disable_device() is called properly in the removal and error
paths.  Also, the pci_set_master() is added to the resume callbacks if
missing (just to be sure).',
	'2004-11-29',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=41aad966Ud-w5G62cwgVukw9bbtqYg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.52.1.1',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.1938.334.60';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.52.1.1'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'bus_config_protocol'  	   	   and bug_severity.name=
	'unknown';



-- changeset '1.4112.328.46'

insert into changeset (cs_number,description,created,url) values (
	'1.4112.328.46',
	'[ALSA] cmipci - Disable integrated mpu401 as default

Enable the support of mpu401 PCI port only when mpu_port=1 module
option is given, i.e. disabled as default.
It turned out that the check of integrated midi port isn''t perfect
and caused hang-ups on some boards.',
	'2006-06-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=449af089SultdonoGRwsQCFjMAsyEA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.77',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.4112.328.46';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.77'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_ordering' 	   and bug_severity.name=
	'unknown';


-- changeset '1.4867.75.6'

insert into changeset (cs_number,description,created,url) values (
	'1.4867.75.6',
	'[ALSA] Various fixes for suspend/resume of ALSA PCI drivers

- Check the return value of pci_enable_device() and request_irq()
  in the suspend.  If any error occurs there, disable the device
  using snd_card_disconnect().
- Call pci_set_power_state() properly with pci_choose_state().
- Fix the order to call pci_set_power_state().
- Removed obsolete house-made PM codes in some drivers.',
	'2006-10-22',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=453b30feKnkB9U_ee-jwYR3MfnA0gQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.83',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.4867.75.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Check the return value of pci_enable_device() and request_irq()
in the suspend.  If any error occurs there, disable the device
using snd_card_disconnect()',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.83'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'return_check' 	   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Fix the order to call pci_set_power_state().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.83'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'os_pm_protocol' 	   	   and bug_severity.name=
	'unknown';


-- changeset '1.5225.18.6'

insert into changeset (cs_number,description,created,url) values (
	'1.5225.18.6',
	'[ALSA] _snd_cmipci_uswitch_put doesn''t set zero flags

Playing with spdif output on cmipci i''ve noticed the SPDO5V option does
not change appropriate bits the register.
The _snd_cmipci_uswitch_put checks the change in flags in wrong way.
If ''active'' state of an option corresponds to a _zero_ bits in a hw
register then function fails. The SPDO5V is the sample.
In the most cases ''active'' state of option is set through an non-zerio
bits in a register. This case works fine.
The fix attached.
Unfortunately i was unable to change spdif output voltage anyway.
Although the register changes right at least.',
	'2007-01-09',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=45a34cfaYUc87sBbHWso0m-H6E-jOg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.85',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.5225.18.6';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.85'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.118'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.118',
	'[ALSA] cmipci: fix handling of FM/MIDI port addresses

Make sure that the MPU-401 MIDI and OPL-3 FM devices are used only on
those chips where they are supported, and that the correct port
addresses are used.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714c3came9eikNgx_agliQ1l1wZRw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.89',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.118';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.89'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_ordering'	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.124'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.124',
	'[ALSA] cmipci: make the test for integrated MIDI port address more robust

Unused bytes in the I/O register range are likely to have the value 0x00
instead of 0xff, so test against both values when checking for the
presence of the integrated MIDI port.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714c3d0ANbfzNf4xyW9rJSiitjTAA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.90',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.124';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.90'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.141'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.141',
	'[ALSA] cmipci: remove invalid channels constraint

Remove the constraint that sets the channel limit for the first playback
device to that of the second one; the first device supports only stereo.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714c3e1FGqpx8dThvjzrTHKuY1qmw');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.94',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.141';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.94'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.144'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.144',
	'[ALSA] cmipci: check that the legacy MIDI port works

Check that the UART_EN bit actually enabled the MPU-401 port.
Apparently, C-Media thinks that it is a good idea to be paranoid here.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714c3e4f7alYfOXpEAKSsKOiDpL2A');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.97',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.144';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.97'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.198'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.198',
	'[ALSA] cmipci: fix version 37 detection

Use the proper value for the bit that identifies chip version 37.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714cfb4udHil4SYu_2_KEodnw_MbQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.100',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.198';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.100'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.201'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.201',
	'[ALSA] cmipci: reset the chip when initializing

When the chip is initialized, reset it.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714cfb7eAYsdvI6LsaSnZzbpVt1sA');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.103',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.201';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.103'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.204'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.204',
	'[ALSA] cmipci: remove 5.0 format

Disallow playback of five channels because the hardware does not support
it (or nobody knows how to do it).',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714cfbaZi0RksQ5BRXzalpAxhuJBg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.106',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.204';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.106'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.205'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.205',
	'[ALSA] cmipci: reorganize set_dac_channels()

By reorganizing the code that sets the CHB3DxC bits we can not only
simplify this code but also fix the bug where the CHB3D8C bit was not
reset when playing a stereo stream after a 7.1 stream.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714cfbbS4475BDpJph9y5rID2MnCg');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.107',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.205';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.107'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.206'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.206',
	'[ALSA] cmipci: fix lookup of double rates

When using one of the double sampling rates, use half the sample rate to
look up in the rates[] table, otherwise we stumble over the BUG().',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714cfbcOaXLhzlXhZk9ctWDZaQF1g');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.108',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.206';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.108'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


-- changeset '1.6702.3.208'

insert into changeset (cs_number,description,created,url) values (
	'1.6702.3.208',
	'[ALSA] cmipci: fix distortion on rear channels

When playing multichannel data, the rear channels can get distorted if
the last sample of the last played stereo stream was not zero.  To avoid
this, add a hack to play a few silence samples after the stream is
stopped.',
	'2007-10-16',
	'http://linux.bkbits.net:8080/linux-2.6/?PAGE=cset&REV=4714cfbeRF78iHe4N5ZvhNjRWz_zyQ');

insert into module_revision (rev_number, description, module, changeset) select 
	'1.110',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'cmipci'		and changeset.cs_number=  
	'1.6702.3.208';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'1.110'    	   	   and module_revision.module in (select id from module where name = 'cmipci')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


------------------------------------------------------
-- Driver for Mellanox InfiniHost III Ex InfiniBand host controller
------------------------------------------------------

insert into module (name,description,path,created) values(
	'mthca', 
	'Driver for Mellanox InfiniHost III Ex InfiniBand host controller',
	'drivers/infiniband/hw/mthca',
	'2004-12-27');

-- Changeset/module revision data not entered for this driver

-- changeset 'unknown'
insert into changeset (cs_number,description,created,url) values (
	'unknown',
	'',
	'2004-12-27',
	'');

insert into module_revision (rev_number, description, module, changeset) select 
	'unknown',
	'', 			module.id, changeset.id from module, changeset where module.name=
	'mthca'			and changeset.cs_number=  
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] InfiniBand: add missing break between cases

Add a missing break statement between RC and UD cases in mthca_post_send().
 This fixes a possible oops for protocols that use the RC transport.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   		   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: fix posting sends with immediate data

When posting a work request with immediate data, put the immediate data in the
immediate data field of the hardware work request (rather than overwriting
the flags field).',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: map context for RDMA responder in mem-free mode

Fix RDMA in mem-free mode: we need to make sure that the RDMA context memory
is mapped for the HCA.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: Set QP static rate correctly

Fix offset of static_rate in QP context.  Pointed out by Dror Goldenberg.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: Set RDMA/atomic capabilities correctly

mthca apparently had the meanings of the max_rd_atomic and max_dest_rd_atomic
QP attributes backwards.  max_rd_atomic limits the maximum number of
outstanding RDMA/atomic requests as an initiator (on a send queue), and
max_dest_rd_atomic specifies the resources allocated to handle RMDA/atomic
requests from the remote end of the connection.  We were programming our QP
context with these values swapped.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: Use correct port width capability value

When we call the INIT_IB firmware command to bring up a port, use
the actual port width capability returned by the QUERY_DEV_LIM
command instead of always trying to enable both 1X and 4X.  This
fixes breakage seen when the firmware is build to allow 4X only.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB: really reset QPs

When we modify a QP to the RESET state, completely clean up the QP
so that it is really and truly reset.
LR: the OS does not expect any completions from a reset queue, so the CQ must me cleaned',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_protocol'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB: Initialize qp->wait

Add missing call to init_waitqueue_head().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'spinlock_api'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: assign ACK timeout field correctly

The hardware reads the ACK timeout field from the most significant 5
bits of struct mthca_qp_path ackto field, not the least significant
bits.  This fix has the driver put the timeout in the right place.
Without this, we get a timeout that is 2^8 times too small.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: fix posting of first work request

Fix posting first WQE for mem-free HCAs: we need to link to previous
WQE even in that case.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: Fix device removal memory leak

Clean up QP table array on device removal.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'resource_leak'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: Better limit checking and reporting

Check the sizes of CQs, QPs and SRQs when creating objects, and fail
instead of creating too-big queues.  Also return real limits instead
of just plausible-sounding values from mthca_query_device().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: correct modify QP attribute masks for UC

The UC transport does not support RDMA reads or atomic operations, so
we should not require or even allow the consumer to set attributes
relating to these operations for UC QPs.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'       and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: check P_Key index in modify QP

Make sure that the P_Key index passed into mthca_modify_qp() is
within the device P_Key table.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: fix posting of atomic operations

The size of work requests for atomic operations was computed
incorrectly in mthca: all sizeofs need to be divided by 16.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: fix posting long lists of receive work requests

In Tavor mode, when posting a long list of receive work requests, a
doorbell must be rung every 256 requests.  Add code to do this when
required.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: do not disable RDMA writes if no responder resources

Responder resources are only required to handle RDMA reads and atomic
operations, not RDMA writes.  So the driver should allow RDMA writes
even if responder resources are set to 0.  This is especially
important for the UC transport -- with the old code, it was impossible
to enable RDMA writes for UC QPs.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Safer max_send_sge/max_recv_sge calculation

Calculation of QP capabilities still is not exactly right in mthca:
max_send_sge/max_recv_sge fields returned in create_qp can exceed the
handware supported limits.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: reset QPs last pointers when transitioning to reset state

last pointer is not updated when QP is modified to reset state.  This
causes data corruption if WQEs are already posted on the queue.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   		   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: check RDMA limits

Add limit checking on rd_atomic and dest_rd_atomic attributes:
especially for max_dest_rd_atomic, a value that is larger than HCA
capability can cause RDB overflow and corruption of another QP.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: correct log2 calculation

Fix thinko in rd_atomic calculation: ffs(x) - 1 does not find the next
power of 2 -- it should be fls(x - 1).',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   		   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: do not change driver copy of attributes if modify QP fails

Only change the driver copy of the QP attributes in modify QP after
checking the modify QP command completed successfully.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	           and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix IB_QP_ACCESS_FLAGS handling.

This patch corrects some corner cases in managing the RAE/RRE bits in
the mthca qp context.  These bits need to be zero if the user requests
max_dest_rd_atomic of zero.  The bits need to be restored to the value
implied by the qp access flags attribute in a previous (or the
current) modify-qp command if the dest_rd_atomic variable is changed
to non-zero.

In the current implementation, the following scenario will not work:
RESET-to-INIT  set QP access flags to all disabled (zeroes)
INIT-to-RTR     set max_dest_rd_atomic=10, AND
         set qp_access_flags = IB_ACCESS_REMOTE_READ | IB_ACCESS_REMOTE_ATOMIC

The current code will incorrectly take the access-flags value set in
the RESET-to-INIT transition.

We can simplify, and correct, this IB_QP_ACCESS_FLAGS handling: it is
always safe to set qp access flags in the firmware command if either
of IB_QP_MAX_DEST_RD_ATOMIC or IB_QP_ACCESS_FLAGS is set, so lets
just set it to the correct value, always.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix corner cases in max_rd_atomic value handling in modify QP

sae and sre bits should only be set when setting sra_max.  Further, in
the old code, if the caller specifies max_rd_atomic = 0, the sre and
sae bits are still set, with the result that the QP ends up with
max_rd_atomic = 1 in effect.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: check port validity in modify_qp

Modify_qp should check that the physical port number provided
is a legal value.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'api_usage'   	           and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: max_inline_data handling tweaks

Fix a case where copying max_inline_data from a successful create_qp
capabilities output to create_qp input could cause EINVAL error:

mthca_set_qp_size must check max_inline_data directly against
max_desc_sz; checking qp->sq.max_gs is wrong since max_inline_data
depends on the qp type and does not involve max_sg.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Fixes to SQEr->RTS transition in modify_qp:
1. The flag IB_QP_ACCESS_FLAGS is optional for UC qps',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'Fixes to SQEr->RTS transition in modify_qp:
2. The SQEr state is not supported for RC qps',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: fix for RTR-to-RTS transition in modify QP

PKEY_INDEX is not a legal parameter in the RTR->RTS transition.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Initialize grh_present before using it

build_mlx_header() was using sqp->ud_header.grh_present before it was
initialized by mthca_read_ah().  Furthermore, header->grh_present is
set by ib_ud_header_init, so there is no need to set it again in
mthca_read_ah().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   		   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Check alternate P_Key index when setting alternate path

Check that the alternate P_Key index is in range when setting the
alternate path for a QP.  Also make a cosmetic touch up to the debug
message printed when the main P_Key index is out of range.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'  	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Check that sgid_index and path_mtu are valid in modify_qp

Add a check that the modify QP parameters sgid_index and path_mtu are
valid, since they might come from userspace.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix uninitialized variable in mthca_alloc_qp()

mthca_alloc_sqp() by mthca_set_qp_size() need to set qp->transport
before calling mthca_set_qp_size(), since the value is used there.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   		   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix modify QP error path

If the call to mthca_MODIFY_QP() failed, then mthca_modify_qp() would
still do some things it should not, such as store away attributes for
special QPs.  Fix this, and simplify the code, by simply jumping to
the exit path if mthca_MODIFY_QP() fails.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   		   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'LR: For completion queues
	IB/mthca: Fix race in reference counting

Fix races in in destroying various objects.  If a destroy routine
waits for an object to become free by doing

        wait_event(&obj->wait, !atomic_read(&obj->refcount));
        now clean up and destroy the object 

and another place drops a reference to the object by doing

        if (atomic_dec_and_test(&obj->refcount))
         wake_up(&obj->wait);',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'control_race'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'LR: For qps
	IB/mthca: Fix race in reference counting

Fix races in in destroying various objects.  If a destroy routine
waits for an object to become free by doing

        wait_event(&obj->wait, !atomic_read(&obj->refcount));
        now clean up and destroy the object 

and another place drops a reference to the object by doing

        if (atomic_dec_and_test(&obj->refcount))
         wake_up(&obj->wait);',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'control_race'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'LR: For SRQs
	IB/mthca: Fix race in reference counting

Fix races in in destroying various objects.  If a destroy routine
waits for an object to become free by doing

        wait_event(&obj->wait, !atomic_read(&obj->refcount));
        now clean up and destroy the object 

and another place drops a reference to the object by doing

        if (atomic_dec_and_test(&obj->refcount))
         wake_up(&obj->wait);',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'control_race'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix posting lists of 256 receive requests for Tavor

If we post a list of length 256 exactly, nreq in doorbell gets set to
256 which is wrong: it should be encoded by 0.  This is because we
only zero it out on the next WR, which may not be there.  The solution
is to ring the doorbell after posting a WQE, not before posting the
next one.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix memory leak on modify_qp error paths

Some error paths after the mthca_alloc_mailbox() call in mthca_modify_qp()
just do a "return -EINVAL" without freeing the mailbox.  Convert these
returns to "goto out" to avoid leaking the mailbox storage.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'resource_leak'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Make all device methods truly reentrant

Documentation/infiniband/core_locking.txt says:

  All of the methods in struct ib_device exported by a low-level
  driver must be fully reentrant.  The low-level driver is required to
  perform all synchronization necessary to maintain consistency, even
  if multiple function calls using the same object are run
  simultaneously.

However, mthca modify_qp, modify_srq and resize_cq methods are
currently not reentrant.  Add a mutex to the QP, SRQ and CQ structures
so that these calls can be properly serialized.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'control_race'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Make fence flag work for send work requests

The fence bit needs to be set in the doorbell too, not just the WQE.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix potential AB-BA deadlock with CQ locks

When destroying a QP, mthca locks both the QP send CQ and receive
CQ.  However, the following scenario is perfectly valid:

    QP_a: send_cq == CQ_x, recv_cq == CQ_y
    QP_b: send_cq == CQ_y, recv_cq == CQ_x

The old mthca code simply locked send_cq and then recv_cq, which in
this case could lead to an AB-BA deadlock if QP_a and QP_b were
destroyed simultaneously.

We can fix this by changing the locking code to lock the CQ with the
lower CQ number first, which will create a consistent lock ordering.
Also, the second CQ is locked with spin_lock_nested() to tell lockdep
that we know what we are doing with the lock nesting.

This bug was found by lockdep.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'control_race'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Return port number for unconnected QPs in query_qp

port_num was not being returned for unconnected QPs.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Return correct number of bits for static rate in query_qp

Incorrect number of bits was taken for static_rate field.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Use mmiowb after doorbell ring

We discovered a problem when running IPoIB applications on multiple
CPUs on an Altix system. Many messages such as:

ib_mthca 0002:01:00.0: SQ 000014 full (19941644 head, 19941707 tail, 64 max, 0 nreq)

appear in syslog, and the driver wedges up.

Apparently this is because writes to the doorbells from different CPUs
reach the device out of order. The following patch adds mmiowb() calls
after doorbell rings to ensure the doorbell writes are ordered.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_race'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Do not execute QUERY_QP firmware command for QP in RESET state

If a QP being queried is in the RESET state, do not execute the
QUERY_QP firmware command (because it will fail).',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Use correct structure size in call to memset()

When clearing the ib_ah_attr parameter in to_ib_ah_attr(), use sizeof
*ib_ah_attr instead of sizeof *path.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Allow the QP state transition RESET->RESET

RESET->RESET is an allowed QP state transition, so mthca should handle
it correctly, by just returning success without involving the firmware.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix error path in mthca_alloc_memfree()

The garbled logic in mthca_alloc_memfree() causes it to return 0, even
if it fails to allocate all doorbell records.  Fix it to return -ENOMEM
when it fails.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix posting >255 recv WRs for Tavor

Fix posting lists of > 255 receive WRs for Tavor: rq.next_ind must
be updated each doorbell, otherwise the next doorbell will use an
incorrect index.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix RESET to ERROR transition

According to the IB spec, a QP can be moved from RESET to the ERROR 
state, but mthca firmware does not support this and returns an error if 
we try.  Work around this FW limitation by moving the QP from RESET to
INIT with dummy parameters and then transitioning from INIT to ERROR.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix handling of send CQE with error for QPs connected to SRQ

mthca_free_err_wqe() currently treats both send and receive CQEs
identically if a QP is using an SRQ.  But for Tavor hardware, send
CQEs with error can be chained together even if the RQ is part of SRQ,
so we may miss some CQEs.

Fix by following the WQE chain for all send CQEs even for non-SRQ QPs.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/ib_mthca: Pre-link receive WQEs in Tavor mode

We have recently discovered that Tavor mode requires each WQE in a
posted list of receive WQEs to have a valid NDA field at all times.
This requirement holds true for regular QPs as well as for SRQs.  This
patch prelinks the receive queue in a regular QP and keeps the free
list in SRQ always properly linked.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix check of max_send_sge for special QPs

The MLX transport requires two extra gather entries for sends (one for
the header and one for the checksum at the end, as the comment says).
However the code checked that max_recv_sge was not too big, instead of
checking max_send_sge as it should have.  Fix the code to check the
correct condition.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'2. mthca_request_regions error handling was borken
   (wrong order of cleanups). For example on all errors
   pci_release_region was called which is wrong if the region
   was not yet mapped. And other such cleanups.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: fix MTT allocation in mem-free mode

Fix bug in MTT allocation in mem-free mode.

I misunderstood the MTT size value returned by the firmware -- it is really
the size of a single MTT entry, since mem-free mode does not segment the MTT
as the original firmware did.  This meant that our MTT addresses ended up
being off by a factor of 8.  This meant that our MTT allocations might
overlap, and so we could overwrite and corrupt earlier memory regions when
writing new MTT entries.

We fix this by always using our 64-byte MTT segment size.  This allows some
simplification of the code as well, since there is no reason to put the MTT
segment size in a variable -- we can always use our enum value directly.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: Fix memory leak on device close

Remember to free the multicast group context memory table.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'resource_leak'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: Report correct atomic capability

Return correct atomic capability flag from mthca query function.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: report page size capability

Report the device real page size capability in mthca_query_device().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: check return value in mthca_dev_lim call

Check error return on call to mthca_dev_lim for Tavor
(as is done for memfree).',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix max_srq_sge returned by ib_query_device for Tavor devices

The driver allocates SRQ WQEs size with a power of 2 size both for
Tavor and for memfree. For Tavor, however, the hardware only requires
the WQE size to be a multiple of 16, not a power of 2, and the max
number of scatter-gather allowed is reported accordingly by the
firmware (and this is the value currently returned by
ib_query_device() and ibv_query_device()).

If the max number of scatter/gather entries reported by the FW is used
when creating an SRQ, the creation will fail for Tavor, since the
required WQE size will be increased to the next power of 2, which
turns out to be larger than the device permitted max WQE size (which
is not a power of 2).

This patch reduces the reported SRQ max wqe size so that it can be used
successfully in creating an SRQ on Tavor HCAs.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix access to MTT and MPT tables on non-cache-coherent CPUs

We allocate the MTT table with alloc_pages() and then do pci_map_sg(),
so we must call pci_dma_sync_sg() after the CPU writes to the MTT
table.  This works since the device will never write MTTs on mem-free
HCAs, once we get rid of the use of the WRITE_MTT firmware command.
This change is needed to make that work, and is an improvement for
now, since it gives FMRs a chance at working.

For MPTs, both the device and CPU might write there, so we must
allocate DMA coherent memory for these.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_race'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix use-after-free on device restart',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'use_after_free'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Do not read reserved fields in mthca_QUERY_ADAPTER()

For memfree devices, the firmware QUERY_ADAPTER command does not
return vendor_id, device_id, and revision_id; do not return these
fields in the QUERY_ADAPTER function for memfree devices.

Instead, for memfree devices, initialize the rev_id field of the mthca
device via init_node_data (MAD IFC query), as is done in the
query_device verb implementation.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Avoid integer overflow when dealing with profile size

mthca_make_profile() returns the size in bytes of the HCA context
layout it creates, or a negative value if an error occurs.  However,
the return value is declared as u64 and the memfree initialization
path casts this value to int to test if it is negative.  This makes it
think incorrectly than an error has occurred if the context size
happens to be bigger than 2GB, since this turns into a negative int.

Fix this by having mthca_make_profile() return an s64 and testing
for an error by checking whether this 64-bit value itself is negative.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix max_sge value returned by query_device

The mthca driver returns the maximum number of scatter/gather entries
returned by the firmware as the max_sge value when device properties
are queried.  However, the firmware also reports a limit on the
maximum descriptor size allowed, and because mthca takes into account
the worst case send request overhead when checking whether to allow a
QP to be created, the largest number of scatter/gather entries that
can be used with mthca may be limited by the maximum descriptor size
rather than just by the actual s/g entry limit.

This means that applications cannot actually create QPs with
max_send_sge equal to the limit returned by ib_query_device().  Fix
this by checking if the maximum descriptor size imposes a lower limit
and if so returning that lower limit.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: detect SRQ overflow

The hardware relies on us keeping one extra work request that never
gets used in SRQs.  Add checks to the SRQ work request posting
functions so that they fail when someone is about to use up that extra
work request, rather than when someone uses the very last work request.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: fix WQE size calculation in create-srq

Thinko: 64 bytes is the minimum SRQ WQE size (not the maximum).',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Return actual capacity from create_srq

Have mthca create_srq method return the actual capacity of the SRQ
that gets created.  Also update comments in <rdma/ib_verbs.h> to
clarify that this is what is expected from ib_create_srq().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Query SRQ srq_limit fixes

Fix endianness handling of srq_limit: it is big-endian in the context
structure, so we need to swab it before returning it.

Also add support for srq_limit query for Tavor (non-MemFree) HCAs.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Check that SRQ WQE size does not exceed device max value

Guarantee the calculated work queue entry size does not exceed the max
allowable WQE size when creating an SRQ.  This is a problem with Arbel
in Tavor-compatibility mode because the current WQE size computation
method rounds up to next power of 2.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Check SRQ limit in modify SRQ operation

When setting the shared receive queue (SRQ) watermark in a modify SRQ
operation, make sure that the supplied value is not larger than the
full size of the SRQ.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix initial SRQ logsize for mem-free HCAs

When initializing an mthca SRQ, the log_srq_size field should be the
log of the number of SRQ WQEs, not the log of the number of bytes in
the SRQ.

This affects only mthca drivers for memfree HCAs which set the initial
srq wqe counter (in the SW2HW transition) to a non-zero value.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: fix reset value endianness

MTHCA_RESET_VALUE must always be swapped, since the HCA expects to see
it in big-endian order and we write it with writel.  This means on
little-endian systems we have to swap it to big-endian order before
writing, and on big-endian systems we need to swap it to make up for
the additional swap that writel will do.  This fixes resetting the HCA
on big-endian machines.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value' 	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: restore missing PCI registers after reset

mthca does not restore the following PCI-X/PCI Express registers after reset:
  PCI-X device: PCI-X command register
  PCI-X bridge: upstream and downstream split transaction registers
  PCI Express : PCI Express device control and link control registers

This causes instability and/or bad performance on systems where one of
these registers is set to a non-default value by BIOS.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'bus_pm_protocol'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] InfiniBand/mthca: initialize mutex  earlier

The cap_mask_mutex needs to be initialized before ib_register_device(),
because device registration will call client init functions that may try to
modify the capability mask.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: allow address handle creation in interrupt context

Make address handle verbs usable from interrupt context.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: Fix memset size

Fix memset to use sizeof *props instead of just sizeof props.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: Fix memory leak on error path

Free page_list buffer on error path of mthca_reg_phys_mr().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'resource_leak'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: Report correct max_msg_sz

Set the max_msg_sz port property correctly in mthca port_query
function.  Also zero out the attr struct so that we do not leave
any other members uninitialized.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: fix page shift calculation in mthca_reg_phys_mr()

For all pages except possibly the last one, the byte beyond the buffer
end must be page aligned.  Therefore, when computing the page shift to
use, OR the end addresses of the buffers as well as the start
addresses into the mask we check.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix offset in query_gid method

GuidInfo records have 8 byte GUIDs in them, so an index should be
multiplied by 8 to get an offset.  mthca_query_gid() was incorrectly
multiplying by 16.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Query port fix

Fill in "max_vl_num" (encoded according to VLCap field in the PortInfo MAD)
and "init_type_reply" values in the ib_query_port() verb.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: fix calculation of RDB shift

Fix calculation of rdb_shift by using original number of QPs, not
their slot in profile[] (which will be rearranged when we sort it).',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: allocate correct number of doorbell pages

Doorbell record pages are allocated in HCA page size chunks (always 4096
bytes), so we need to divide by 4096 and not PAGE_SIZE when figuring out how
many pages we will need space for.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: release mutex on doorbell alloc error path

Release mutex on error return path from mthca_alloc_db().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'lock_balance'	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: Fix doorbell record resource leak

If we allocate a bunch of doorbell records and then free them, we will
end up with completely empty pages, which we then free.  However, when
we come back to allocate more doorbell pages, we have to reallocate
those empty pages rather than always trying to take a slot that we have
never used.  If we do not, we eventually use up every slot and fail to
allocate a doorbell record, even though we have plenty of free space.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: fix memory user DB table leak

Free the memory allocated in mthca_init_user_db_tab() when releasing
the db_tab in mthca_cleanup_user_db_tab().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'resource_leak'	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix thinko in mthca_table_find()

break only escapes from the innermost loop, and we want to escape both
loops and return an answer.  Noticed by Ishai Rabinovitch.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix off-by-one in FMR handling on memfree

mthca_table_find() will return the wrong address when the table entry
being searched for is exactly at the beginning of a sglist entry
(other than the first), because it uses >= when it should use >.

Example: assume we have 2 entries in scatterlist, 4K each, offset is
4K.  The current code will return first entry + 4K when we really want
the second entry.

In particular this means mapping an FMR on a memfree HCA may end up
writing the page table into the wrong place, leading to memory
corruption and also causing the HCA to use an incorrect address
translation table.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'Add missing sg_init_table() in mthca_map_user_db()

Usually harmless, since the scatterlist is always hard-coded to a length
of 1, but it triggers a BUG() if CONFIG_DEBUG_SG=y, so we better fix it.
This fixes <http://bugzilla.kernel.org/show_bug.cgi?id=9934>.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'api_usage'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Clear ICM pages before handing to FW

Current memfree FW has a bug which in some cases, assumes that ICM
pages passed to it are cleared.  This patch uses __GFP_ZERO to
allocate all ICM pages passed to the FW.  Once firmware with a fix is
released, we can make the workaround conditional on firmware version.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: Do not enter QP into MCG more than once.

Avoid entering a QP as member of a multicast group multiple times.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: multiple fixes for multicast group handling
	 1.Fix leak of mailbox memory in error handling on multicast group operations.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'resource_leak'	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'Multicast group management fixes:
2 Free AMGM indices at detach and in attach error handling.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'resource_leak'	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'Multicast group management fixes:
	. Fix amount to shift for aligning next_gid_index in mailbox: it
  starts at bit 6, not bit 5.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Do not cancel commands on a signal

We have run into the following problem: if a task receives a signal
while in the process of e.g. destroying a resource (which could be
because the relevant file was closed) mthca could bail out from trying
to take a command interface semaphore without performing the
appropriate command to tell hardware that the resource is being
destroyed.

As a result we see messages like
 ib_mthca 0000:04:00.0: HW2SW_CQ failed (-4)

In this case, hardware could access the resource after the memory has
been freed, possibly causing memory corruption.

A simple solution is to replace down_interruptible() by down() in
command interface activation.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'control_race'	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] InfiniBand/mthca: add needed rmb() in event queue poll

Add an rmb() between checking the ownership bit of an event queue entry and
reading the contents of the EQE.  Without this barrier, the CPU could read
stale contents of the EQE before HW writes the EQE but have the read of the
ownership bit reordered until after HW finishes writing, which leads to the
driver processing an incorrect event.  This was actually observed to happen
when multiple completion queues are in heavy use on an IBM JS20 PowerPC 970
system.

Also explain the existing rmb() in completion queue poll (there for the same
reason) and slightly improve debugging output.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_race'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: Initialize eq->nent before we use it

In mthca_create_eq(), we call get_eqe() before setting eq->nent.  This
is wrong, because get_eqe() uses eq->nent.  Fix this, and clean up the
code a little while we are at it.  (We got lucky with the current code,
because eq->nent was cleared to 0, which get_eqe() made happen to do
the right thing)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: fix off by one in clr_int calculation

We should use the first word of the clear interrupt register if
the bit we are after is < 32, not < 31.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: Always re-arm EQs in mthca_tavor_interrupt()

We should always re-arm an event queue interrupt in
mthca_tavor_interrupt() if the corresponding bit is set in the event
cause register (ECR), even if we did not find any entries in the EQ.
If we do not, then there is a window where we miss an EQ entry and then
get stuck because we do not get another EQ event.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: create_eq with size not a power of 2

Fix mthca_create_eq for when the EQ size is not a power of 2.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Coverity fix to mthca_init_eq_table()

Fix bug found by coverity: the loop body never executed, because it
was doing for (i = 0; i < MTHCA_EQ_CMD; ++i), but MTHCA_EQ_CMD is 0.
The correct loop bound is MTHCA_NUM_EQ, to loop over all EQs.
',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'typo'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] InfiniBand: remove unbalance refcnt decrement

Fix unbalanced QP reference count decrement (introduced with QP lock
optimization patch)',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'control_race' 	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: only free doorbell records in mem-free mode

On error path, only free doorbell records if we are in mem-free mode.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB/mthca]: Fix error CQ entry handling on mem-free HCAs

Fix handling of error CQ entries on mem-free HCAs: the doorbell count
is never valid so we should not look at it.  This fixes problems exposed
by new HCA firmware.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';

insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: fix wraparound handling in mthca_cq_clean()

Handle case where prod_index has wrapped around and become less than
cq->cons_index by checking that their difference as a signed int is
positive rather than comparing directly.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fill in vendor_err field in completion with error

Fill vendor_err field in completion with error.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_data_struct'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: memfree completion with error FW bug workaround

Memfree firmware is in rare cases reporting WQE index == base - 1 in
receive completion with error, instead of (rq size - 1); base is 0 in
mthca.  Here is a patch to avoid kernel crash and report a correct WR
id in this case.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix PRM compliance problem in atomic-send completions

According to the Tavor and Arbel programmer reference manuals, the
number of bytes transferred is not provided in the byte_cnt field of
the CQ entry for atomic operation completions.  For atomic operations,
the number of bytes transferred is always 8 (when the status is
"success"), and this constant value should always be used by the
driver in the ib_wc entry returned, rather than using the CQE.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Set cleaned CQEs back to HW ownership when cleaning CQ

mthca_cq_clean() updates the CQ consumer index without moving CQEs
back to HW ownership.  As a result, the same WRID might get reported
twice, resulting in a use-after-free.  This was observed in IPoIB CM.
Fix by moving all freed CQEs to HW ownership.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_ordering'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: Fix off by one bug in mthca_map_cmd

The loop in mthca_map_cmd() would fill one entry past the end of the
mailbox buffer before calling the firmware command.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: fix QP size limits for mem-free HCAs

Unlike tavor, the max work queue size is an exact power of 2 for arbel
mode, despite what the documentation (of the QUERY_DEV_LIM firmware
command) says.  Without this patch, on Arbel, we can start with a QP
of a valid size and get above the reported limit after rounding to the
next power of two.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix memory leak of multicast group structures

Convert "/ (1 << lg)" to ">> lg" for a slight code size reduction.

add/remove: 0/0 grow/shrink: 0/1 up/down: 0/-24 (-24)
function                                     old     new   delta
mthca_map_cmd                                613     589     -24',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix MAD extended header format for MAD_IFC firmware command

Several fields in an incoming MAD extended info header were passed
into the MAD_IFC firmware command at incorrect offsets (mostly off by
4 bytes).  As the result, the HCA will fail to generate traps in which
this info is needed (e.g. traps which include the GRH of the incoming
packet), in violation of the IB spec.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Change command token on timeout

The FW command token is currently only updated on a command completion
event. This means that on command timeout, the same token will be
reused for new command, which results in a mess if the timed out
command *does* eventually complete.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Use mmiowb() to avoid firmware commands getting jumbled up

Firmware commands are sent to the HCA by writing multiple words to a
command register block.  Access to this block of registers is
serialized with a mutex.  However, on large SGI systems, problems were
seen with multiple CPUs issuing FW commands at the same time, because
the writes to the register block may be reordered within the system
interconnect and reach the HCA in a different order than they were
issued (even with the mutex).  Fix this by adding an mmiowb() before
dropping the mutex.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_race'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[IB] mthca: fix typo in catastrophic error polling

Fix a typo in the rearming of the catastrophic error polling timer: we
should rearm the timer as long as the stop flag is _not_ set.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'[PATCH] IB/mthca: fix static rate returned by mthca_ah_query

mthca_ah_query returs the static rate of the address handle in internal mthc
format.  fix it to use rate encoding from enum ib_rate, which is what users
expect.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'device_value'   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Fix mthca_array_clear() thinko

mthca_array_clear() does not clear the slot if the used count is
positive. This leads to crashes in mthca_qp_event() since that uses
mthca_array_get() to check that the qp is valid.

Discovered by Ali Ayoub.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'logic'   	   	   and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'IB/mthca: Use IRQ safe locks to protect allocation bitmaps

It is supposed to be OK to call mthca_create_ah() and mthca_destroy_ah()
from any context.  However, for mem-full HCAs, these functions use the
mthca_alloc() and mthca_free() bitmap helpers, and those helpers use
non-IRQ-safe spin_lock() internally.  Lockdep correctly warns that
this could lead to a deadlock.  Fix this by changing mthca_alloc() and
mthca_free() to use spin_lock_irqsave().',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'block_in_atomic_context'  and bug_severity.name=
	'unknown';


insert into bug (description, revision_removed, subcategory, severity) select 
	'Found during work on the Dingo mthca driver:
I cannot figure out what could prevent the following sequence of events
in the mthca driver:

mthca_process_mad --> forward_trap--> ib_post_send_mad--> ib_send_mad

Now, ib_send_mad can either ib_post_send immediately or it can queue the
wr in the overflow_list if the qp is full.  In the latter case we have
a queued wr whose address handle (ah) is the one attached to the mad by
the forward_trap function.  Here is the relevant line from mthca_mad.c:

send_buf->ah = dev->sm_ah[port_num - 1]

While the wr seats in the overflow_list, mthca_process_mad is called
again:

mthca_process_mad --> smp_snoop--> update_sm_ah--> ib_destroy_ah(dev->sm_ah[port_num - 1])

The last call may deallocate the ah that is still in use by the queued
wr.',
				   module_revision.id, bug_subcategory.id, bug_severity.id from module_revision, bug_subcategory, bug_severity where module_revision.rev_number=
	'unknown'    	   	   and module_revision.module in (select id from module where name = 'mthca')and bug_subcategory.name=
	'os_config_protocol'  and bug_severity.name=
	'unknown';


